/*****************************************************************************/
/*                                                                           */
/*  FILE        :rexpci20_gpib.c                                             */
/*  DATE        :Fri, August 8, 2008                                         */
/*  DESCRIPTION :REX-PCI20 API Library for Linux                             */
/*  HISTORY     :                                                            */
/*     v1.0.0:  2008.07.16  original issue                                   */
/*     v1.0.1:  2008.08.08  modified delimiter and interrupt                 */
/*                                                                           */
/*****************************************************************************/

#include "rexpci20_gpib.h"
#include "def9914.h"

///#define _DEBUG

#define DMATRANSFER 1
#define PROTRANSFER 0

#define MAX_DEVICES				16

typedef struct _ENUM_UNIT{
	unsigned short Attached;
	unsigned short UnitId;
} ENUM_UNIT, *PENUM_UNIT;

typedef struct _UNIT_OPENCONTROL{
	unsigned short  Attached;
	unsigned short  UnitId;
	unsigned int    hDrv;
	unsigned short  OpenMark;
	unsigned short  gpibMyAdrs;
	unsigned short  delim1;			// delimiter flag for Talker mode
	unsigned short  delim2;			// delimiter flag for Listner mode
	unsigned int    gpibTimeout;
} UNIT_OPENCONTROL, *PUNIT_OPENCONTROL;


int hSys = 0;		// ファイルディスクリプタハンドル
int hSys_one = 0;	// REX-PCI20 一台接続時使用ハンドル
unsigned short g_MyUnitId;	// REX-PCI20 UnitID
UNIT_OPENCONTROL  OpenCtl[MAX_DEVICES];

// interrupt
unsigned short  fStartEvent = 0;
unsigned short  fInterruptComplete = 0;

// timer
int g_Starttim;
int g_Stoptim;
unsigned char g_ovFlg = 0;

// used gp_count
int	g_DataCount = 0;

// used gp_setdelay
int	g_DelayCount = 1;

// prototype declaration
int BufLength(char* buf);
short ChangeBuf(char* buf1, char* buf2);
void SetTimeOut1(int sec);
unsigned char IsTimeOut1(void);
int GetStatus(void);
unsigned char wsrq(void);
unsigned char wsrqb(void);
void setmst();
void standby();
char cah(unsigned char *Data, unsigned char *Eoi);
char csh(unsigned char Data);
int ctlk1( char *buf );
int ctalk1( char *buf );
int ctalk2( char *buf );
int clistn( char *buf );
void GetUnitIdFromhDrv(int hDriver);
void *StartInterruptThread(void *arg);
pthread_t	thread_id;


/*****************************************************************************/
/*               gp_attach                                                   */
/*****************************************************************************/
/*  解説 :ライブラリ初期化                                                   */
/*****************************************************************************/
int gp_attach()
{
	char	pathName[16];
	unsigned short unitNo;

	memset(OpenCtl, 0x00, sizeof(UNIT_OPENCONTROL)*MAX_DEVICES);
	for(unitNo = 0; unitNo < MAX_DEVICES; unitNo++)
	{
		memset(pathName, 0x00, sizeof(pathName));
		sprintf(pathName, "/dev/gpib%d", unitNo);
		hSys_one = open(pathName,O_RDWR);
		if(hSys_one<0){
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: open error %s\n", pathName);
#endif
		}else{
			hSys = hSys_one;
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: open = %s, hSys = %d\n", pathName, hSys);
#endif
			break;
		}
	}
	return hSys;
}

/*****************************************************************************/
/*               gp_detach                                                   */
/*****************************************************************************/
/*  解説 :ライブラリ終了                                                     */
/*****************************************************************************/
int gp_detach(int num)
{
	int	iRet;
	unsigned short UnitNo;

	for(UnitNo = 0; UnitNo < MAX_DEVICES; UnitNo++)
	{
		if((OpenCtl[UnitNo].OpenMark == 1)&&(num == OpenCtl[UnitNo].hDrv))
		{
			iRet = close(num);
			OpenCtl[UnitNo].UnitId = 0;
			OpenCtl[UnitNo].hDrv = 0;
			OpenCtl[UnitNo].OpenMark = 0;
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: gp_detach: UnitId %d Closed.\n", UnitNo);
#endif
			return iRet;
		}
	}

#ifdef _DEBUG
	printf("PCI20_LIB_MASTER: gp_detach:did not do at all\n");
#endif
	iRet = -1;
	return iRet;
}

/*****************************************************************************/
/*               gp_cardinfo                                                 */
/*****************************************************************************/
/*  解説   :PCカードの情報を返します。                                       */
/*  引数   :pIOBase :iobase情報変数へのポインタ                              */
/*         :pIrqNo  :irq情報変数へのポインタ                                 */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_cardinfo(unsigned int *pIOBase, unsigned int *pIrqNo)
{
	int	iRet;
	CARD_RESOURCE		card;

	iRet=ioctl(hSys, IOCTL_GP_GET_RESOURCE, &card);
	if(iRet<0){
#ifdef _DEBUG
		printf("gp_cardinfo: get card resource error\n");
#endif
	}else{
		*pIOBase = card.IOBaseLow;
		*pIrqNo = card.IrqLevel;
	}

	return iRet;
}

/*****************************************************************************/
/*               BufLength                                                   */
/*****************************************************************************/
/*  解説   :文字列の長さを返す                                               */
/*  引数   :buf:文字列が格納されたバッファ                                   */
/*  戻り値 :文字列の長さ                                                     */
/*****************************************************************************/
int BufLength(char* buf)
{
	int	dwLength;

	dwLength = 0;
	/* buf はあらかじめゼロをセットしておく*/
	while(buf[dwLength])
	{
		dwLength++;
	}

	return dwLength;
}

/*****************************************************************************/
/*               ChangeBuf                                                   */
/*****************************************************************************/
/*  解説   :渡されたGPIBアドレス(文字列)を解析し、数値に変換する。           */
/*          渡されるGPIBアドレス例                                           */
/*          "0"〜"30"、二次アドレスは"96"(0+0x60)〜"126"(30+0x60)            */
/*  引数   :buf1:変換前文字列                                                */
/*         :buf2:変換後文字列(0〜30または96〜126の値)                        */
/*  戻り値 :変換後文字列の長さ                                               */
/*****************************************************************************/
short ChangeBuf(char* buf1, char* buf2)
{
	short	wLength1, wLength2, cnt;

	// 変換前文字列の長さを取得
	wLength1 = (short)BufLength(buf1);

	wLength2 = 0;
	cnt = 0;
	while(wLength1)
	{
		// TAB(0x09)、コンマ(0x2C)、スペース(0x20)の場合
		if((buf1[cnt] == 0x09)||(buf1[cnt] == 0x20)||(buf1[cnt] == 0x2c))
		{
			cnt++;
		}
		// 0〜9の文字列の場合
		else if((buf1[cnt] <= 0x39)&&(buf1[cnt] >= 0x30))
		{
			cnt++;

			// 0〜9の文字列の場合(二桁目のチェック)
			if((buf1[cnt] <= 0x39)&&(buf1[cnt] >= 0x30))
			{
				cnt++;
				// 0〜9の文字列の場合(三桁目のチェック)
				if((buf1[cnt] <= 0x39)&&(buf1[cnt] >= 0x30))
				{
					cnt++;
					buf2[wLength2] = (buf1[cnt-3] - 0x30) * 100 + (buf1[cnt-2] - 0x30) * 10 + (buf1[cnt-1] - 0x30);
					// 取得した値が126より大きければ設定エラー
					// もしくはREX-PCI20 IDと重複していれば設定エラー
					if((buf2[wLength2] > 126)||(buf2[wLength2] == (char)OpenCtl[g_MyUnitId].gpibMyAdrs))
					{
#ifdef _DEBUG
						printf("PCI20_LIB_MASTER: ChangeBuf ERR1 : buf2[%d] : %x\n", wLength2, buf2[wLength2]);
#endif
						return 0;
					}
					wLength2++;
					// 0〜9の文字列の場合(四桁ならば設定エラー)
					if((buf1[cnt] <= 0x39)&&(buf1[cnt] >= 0x30))
					{
#ifdef _DEBUG
						printf("PCI20_LIB_MASTER: ChangeBuf ERR2 : buf1[%d] : %x\n", cnt, buf1[cnt]);
#endif
						return 0;
					}
					wLength1--;
				}
				// 二桁の場合
				else
				{
					buf2[wLength2] = (buf1[cnt-2] - 0x30) * 10 + (buf1[cnt-1] - 0x30);
					if(((buf2[wLength2] > 30)&&(buf2[wLength2] < 96))||(buf2[wLength2] == (char)OpenCtl[g_MyUnitId].gpibMyAdrs))
					{
#ifdef _DEBUG
						printf("PCI20_LIB_MASTER: ERR3 : buf2[%d] : %x\n", wLength2, buf2[wLength2]);
#endif
						return 0;
					}
					wLength2++;
				}
				wLength1--;
			}
			// 一桁の場合
			else
			{
				buf2[wLength2] = buf1[cnt-1] - 0x30;
				if(buf2[wLength2] == (char)OpenCtl[g_MyUnitId].gpibMyAdrs)
				{
#ifdef _DEBUG
					printf("PCI20_LIB_MASTER: ERR4 : buf2[%d] : %x\n", wLength2, buf2[wLength2]);
#endif
					return 0;
				}
				wLength2++;
			}
		}
		else
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: ERR5 : buf1[%d] : %x\n", cnt, buf1[cnt]);
#endif
			return 0;
		}
		wLength1--;
	}
	return wLength2;
}

/*****************************************************************************/
/*               SetTimeOut1                                                 */
/*****************************************************************************/
/*  解説   :タイムアウト時間の設定を行います。                               */
/*          タイムアウトの確認はIsTimeOut1で行います。                       */
/*  引数   :sec:タイムアウト時間(sec)                                        */
/*  戻り値 :なし                                                             */
/*****************************************************************************/
void SetTimeOut1(int sec)
{
	int		cnt;
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv, &tz);
	g_Starttim = tv.tv_sec;

	g_ovFlg = 0;
	g_Stoptim = g_Starttim;
	for(cnt = 1; cnt <= sec; cnt++)
	{
		g_Stoptim++; // +1
		if(g_Stoptim == 0)
			g_ovFlg = 1;
	}
}

/*****************************************************************************/
/*               IsTimeOut1                                                  */
/*****************************************************************************/
/*  解説   :タイムアウト時間が経過したかを確認します。                       */
/*          タイムアウト時間の設定はSetTimeOut1で行います。                  */
/*  引数   :sec:タイムアウト時間(sec)                                        */
/*  戻り値 :なし                                                             */
/*****************************************************************************/
unsigned char IsTimeOut1(void)
{
	int		Nowtim;
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv, &tz);
	Nowtim = tv.tv_sec;

	if((g_ovFlg == 1) && (Nowtim > g_Starttim))
		return 0;

	if(g_Stoptim <= Nowtim)
		return 1;
	else
		return 0;
}

/*****************************************************************************/
/*               GetStatus                                                   */
/*****************************************************************************/
/*  解説   :直前のコマンドに対するステータスを取得  (未実装)                 */
/*  戻り値 :直前のコマンドに対するステータス                                 */
/*****************************************************************************/
int GetStatus(void)
{
	return 0;
}

/*****************************************************************************/
/*               gp_GetDeviceID                                              */
/*****************************************************************************/
/*  解説   :GPIBアドレスを取得  (未実装)                                     */
/*  戻り値 :0                                                                */
/*****************************************************************************/
int gp_GetDeviceID(int hDev, unsigned short UnitId, char* pDevId)
{
	*pDevId = 1;

	return 0;
}

/*****************************************************************************/
/*               OutPort                                                     */
/*****************************************************************************/
/*  解説   :指定の9914レジスタにデータを書き込む                             */
/*  引数   :Reg     :9914のオフセット値                                      */
/*         :bOutVal :書き込むデータ                                          */
/*  戻り値 :出力値(エラー時は0xFFFF)                                         */
/*****************************************************************************/
unsigned short OutPort(short Reg, short bOutVal)
{
	int	iRet;
	IOPORT_UCHAR		bIOPort;

	bIOPort.Offset = Reg;
	bIOPort.Value = (unsigned char)bOutVal;

	iRet=ioctl(hSys, IOCTL_GP_WRITE_PORT_UCHAR, &bIOPort);
	if(iRet<0)
		return 0xFFFF;
	else
		return 0;
}

/*****************************************************************************/
/*               InPort                                                      */
/*****************************************************************************/
/*  解説   :指定の9914レジスタからデータを読み込む                           */
/*  引数   :Reg :9914のオフセット値                                          */
/*  戻り値 :読み込んだ値(エラー時は0xFFFF)                                   */
/*****************************************************************************/
unsigned short InPort(short Reg)
{
	int	iRet;
	IOPORT_UCHAR		bIOPort;

	bIOPort.Offset = Reg;

	iRet=ioctl(hSys, IOCTL_GP_READ_PORT_UCHAR, &bIOPort);
	if(iRet<0)
		return 0xFFFF;
	else
		return bIOPort.Value;
}

/*****************************************************************************/
/*               gp_wtb                                                      */
/*****************************************************************************/
/*  解説   :ATNラインをTRUEにしてコマンド文字列を送信します。                */
/*  引数   :buf:送信コマンド文字列                                           */
/*  戻り値 : 0 :正常終了                                                     */
/*         :53 :バスタイムアウト                                             */
/*         :64 :データが正しくない                                           */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_wtb(char * buf)
{
	int	iRet;
	int	bufLen;
	int	loop;

	bufLen = BufLength(buf);
	if(bufLen == 0)
		return 64;

	setmst();	// ATN  set
	for ( loop = 0 ; loop < bufLen ; loop++ )
	{
		iRet = csh(buf[loop]);	// handshake & data out 
		if ( iRet != 0 ) 
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER:csh: error = %d\n", iRet);
#endif
			break;
		}
#ifdef _DEBUG
		printf("PCI20_LIB_MASTER:gp_wtb: wrote data = '%x'\n", buf[loop]);
#endif
	}
	standby();

#ifdef _DEBUG
	printf("PCI20_LIB_MASTER:gp_wtb: Len(%d) = %d\n", bufLen, loop);
#endif
	//g_DataCount = loop;

    return iRet;
}

/*****************************************************************************/
/*               gp_dataout                                                  */
/*****************************************************************************/
/*  解説   :GPIBバス上にデータ送信。データモードのデータ送信を行う           */
/*  引数   :buf     :送信コマンド文字列                                      */
/*         :bufLen  :送信文字列サイズ                                        */
/*         :DMAflag : 0:gp_wrd(d), 1:gp_tfrout(d)                            */
/*  戻り値 : 0 :正常終了                                                     */
/*         :53 :バスタイムアウト                                             */
/*         :64 :受信文字列用バッファサイズが正しくない                       */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_dataout(char* buf, int bufLen, unsigned char DMAflag)
{
	int	iRet;
	int	loop;
	short MyUnitId;

	MyUnitId = g_MyUnitId;
#if 0
	struct GpibCommand gpib_cmd;
	
	gpib_cmd.requesttype = 0x40;
	if(DMAflag)
		gpib_cmd.request = Vendor_bRequest_GP_TFROUT;
	else
		gpib_cmd.request = Vendor_bRequest_GP_WRT;
#endif

	for ( loop = 0 ; loop < bufLen ; loop++ )
	{
		if ( (loop == bufLen-1 ) && (OpenCtl[MyUnitId].delim1 & 0x800)){
			// force EOI with Last data
			OutPort(AUXCR, feoi);
		}
		//++ 080805
		else if ( (loop == bufLen-1 ) && (OpenCtl[MyUnitId].delim1 <= 0xFF) && (OpenCtl[MyUnitId].delim1 & 0x80) ){
			// force EOI with Last data
			OutPort(AUXCR, feoi);
		}
		//--

		iRet = csh(buf[loop]);	// handshake & data out 
		if ( iRet != 0 ) 
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER:csh: error = %d\n", iRet);
#endif
			break;
		}
//#ifdef _DEBUG
//		printf("PCI20_LIB_MASTER:gp_dataout: wrote buf[%d] = '%x'\n", loop, buf[loop]);
//#endif
	}

#ifdef _DEBUG
	printf("PCI20_LIB_MASTER:gp_dataout: Len(%d) = %d\n", bufLen, loop);
#endif
	g_DataCount = loop;

    return iRet;
}

/*****************************************************************************/
/*               gp_datain                                                   */
/*****************************************************************************/
/*  解説   :GPIBバスからデータ受信。データモードのデータ受信を行う           */
/*  引数   :buf     :受信文字列用バッファ                                    */
/*         :bufLen  :受信文字列サイズ                                        */
/*         :DMAflag : 0:gp_red(d), 1:gp_tfrin(s)                             */
/*  戻り値 : 0 :正常終了                                                     */
/*         :24 :デリミタ受信正常終了                                         */
/*         :53 :バスタイムアウト                                             */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_datain(char* buf, int bufLen, unsigned char DMAflag)
{
	int				iRet;
	int	loop;
	unsigned char	eoi;
	short MyUnitId;
	unsigned char	dummy;

	MyUnitId = g_MyUnitId;

	OutPort(AUXCR, hdfa); // set HDF Mode

	for ( loop = 0 ; loop < bufLen ; loop++ )
	{
		iRet = cah((unsigned char *)buf+loop, &eoi);	// handshake & data in 
		if ( iRet != 0 ) 
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER:cah: error = %d\n", iRet);
#endif
			break;
		}
//#ifdef _DEBUG
//		printf("PCI20_LIB_MASTER:gp_datain: read buf[%d] = '%x'\n", loop, buf[loop]);
//#endif

		if ( eoi ) 
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER:cah: EOI get !!\n", iRet);
#endif
			loop++;
			iRet = 24;
			break;
		}

		if(!DMAflag){
			if ( OpenCtl[MyUnitId].delim2 & 0x400 )
			{
				;
			}
			else if ( (OpenCtl[MyUnitId].delim2 & 0x200) && buf[loop-1] == 0x0d && buf[loop] == 0x0a ) // CR+LF
			{
#ifdef _DEBUG
				printf("PCI20_LIB_MASTER:cah: get CR+LF !!\n", iRet);
#endif
				loop++;
				loop = loop - 2;
				iRet = 24;
				break;
			}
			// 080808 - デリミタコードが 0x80D, 0x80A の場合にタイムアウトになってしまうため修正
			//else if ( buf[loop] == OpenCtl[MyUnitId].delim2 )
			else if ( buf[loop] == (OpenCtl[MyUnitId].delim2 & 0xFF) )
			{
#ifdef _DEBUG
				printf("PCI20_LIB_MASTER:cah: get delm !!\n", iRet);
#endif
				loop++;
				loop--;
				iRet = 24;
				break;
			}
		}
		OutPort(AUXCR, rhdf);
	}

#ifdef _DEBUG
	printf("PCI20_LIB_MASTER:gp_datain: Len(%d) = %d\n", bufLen, loop);
#endif
	g_DataCount = loop;

    return iRet;
}

/*****************************************************************************/
/*               gp_datain_End                                               */
/*****************************************************************************/
/*  解説   :UNTを発行し、ハンドシェイクを終了する                            */
/*  引数   :なし                                                             */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_datain_End(void)
{
	int			iRet;
	unsigned short wRet;
	char		cmdbuf[4];

	SetTimeOut1(3);
	while(1)
	{
		wRet = InPort(BSR);
		if(wRet == 0xFFFF)
			return -1;
		if(!(wRet & 0x40))
			break;

		if(IsTimeOut1())
			break;
	}

	memset(cmdbuf, 0x00, sizeof(cmdbuf));

	cmdbuf[0] = UNT;
	cmdbuf[1] = 0x00;
	iRet = gp_wtb(cmdbuf);

	if(iRet != 0)
	{
		return iRet;
	}

	wRet = OutPort(AUXCR, rhdf);
	wRet = OutPort(AUXCR, _hdfa);

	if(wRet == 0xFFFF)
		return -1;
	else
		return 0;
}

/*****************************************************************************/
/*               gp_init                                                     */
/*****************************************************************************/
/*  解説   :PCI-GPIBの機器アドレスをセット                                   */
/*  引数   :GpAdrs :GPIB機器アドレス                                         */
/*         :IOBase :未使用                                                   */
/*         :IrqNo  :未使用                                                   */
/*  戻り値 : 0 :正常終了                                                     */
/*         :60 :指定GPIB機器アドレスが正しくない                             */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_init(short GpAdrs, short IOBase, short IrqNo)
{
	unsigned short wRet;
	short MyUnitId;

	if(InPort(BSR) == 0xff){
		return -1;
	}

	MyUnitId = g_MyUnitId = GpAdrs;
	//gp_GetDeviceID(hSys_one, 0, (char*)&g_MyUnitId);
	OpenCtl[MyUnitId].UnitId = g_MyUnitId;
	OpenCtl[MyUnitId].hDrv = hSys_one;
	OpenCtl[MyUnitId].OpenMark = 1;
	OpenCtl[MyUnitId].gpibTimeout = 10;		//10s
	OpenCtl[MyUnitId].gpibMyAdrs = GpAdrs;	
	OpenCtl[MyUnitId].delim1	= 0x00;		// Talker mode delimiter CR+LF
	OpenCtl[MyUnitId].delim2	= 0x0a;		// listner mode delimiter LF

	wRet = OutPort(ADR, GpAdrs & 0x1f);		// my address set
	wRet = OutPort(IMR0, 0);				// Int Mask Register0 all mask
	wRet = OutPort(IMR1, 1);				// Int Mask Register1 all mask
	wRet = OutPort(SPMR, 0);
	wRet = OutPort(AUXCR, swrst);			// TMS9914 SoftWare Reset
	wRet = OutPort(AUXCR, _swrst);			// TMS9914 goto Active
//080703	wRet = OutPort(AUXCR, hdfa);	// set HDF Mode

	if(wRet==0xFFFF)
		return -1;
	else
		return 0;
}

/*****************************************************************************/
/*               gp_cli                                                      */
/*****************************************************************************/
/*  解説   :IFCラインを一定期間(10msec)TRUEにします。                        */
/*  引数   :なし                                                             */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_cli(void)
{
	unsigned short wRet;

	wRet = OutPort(AUXCR, sic);	// force IFC line "true": IFC='L'
	gp_delay(10);
	wRet = OutPort(AUXCR, _sic);	// IFC line "false": IFC='H'

	if(wRet==0xFFFF)
		return -1;
	else
		return 0;
}

/*****************************************************************************/
/*               gp_ren                                                      */
/*****************************************************************************/
/*  解説   :RENラインをTRUEにします。                                        */
/*  引数   :なし                                                             */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_ren(void)
{
	unsigned short wRet;

	wRet = OutPort(AUXCR, sre);

	if(wRet==0xFFFF)
		return -1;
	else
		return 0;
}

/*****************************************************************************/
/*               gp_tmout                                                    */
/*****************************************************************************/
/*  解説   :バスタイムアウト時間の設定を変更します。初期値は10秒です。       */
/*  引数   :SecTime :タイムアウト時間(sec)                                   */
/*  戻り値 : 0 :正常終了                                                     */
/*****************************************************************************/
int gp_tmout(int SecTime)
{
	short MyUnitId;

	MyUnitId = g_MyUnitId;

	OpenCtl[MyUnitId].gpibTimeout = SecTime;	//10s
	return 0;
}

/*****************************************************************************/
/*               gp_delay                                                    */
/*****************************************************************************/
/*  解説   :遅延時間を設けます。                                             */
/*  引数   :UsecTime :遅延時間(usec)                                         */
/*  戻り値 : 0 :正常終了                                                     */
/*****************************************************************************/
int gp_delay(int UsecTime)
{
	sleep(UsecTime/1000);
	return 0;
}

/*****************************************************************************/
/*               gp_setdelay                                                 */
/*****************************************************************************/
/*  解説   :ATNラインをTRUE又はFALSEにする際のディレイ時間を設定します。     */
/*          初期値は0usecです。                                              */
/*  引数   :DelayTime :ディレイ時間(usec)                                    */
/*  戻り値 : 0 :正常終了                                                     */
/*****************************************************************************/
int gp_setdelay(int DelayTime)
{
	g_DelayCount = DelayTime;
	return 0;
}

/*****************************************************************************/
/*               gp_clr                                                      */
/*****************************************************************************/
/*  解説   :DCL または SDC コマンド送信                                      */
/*  引数   :adrs :GPIB機器アドレスを示す文字列                               */
/*  戻り値 : 0 :正常終了                                                     */
/*         :53 :バスタイムアウト                                             */
/*         :63 :アドレスが正しくない                                         */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_clr(char * adrs)
{
	int	iRet;
	char	cmdbuf[1024]; // send cmd buf
	int		cnt, cnt2;	// counter
	char *	pChangeAdrs;
	short	AdrsLen;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	if(BufLength(adrs) == 0){
		// DCL transfer
		cmdbuf[0] = DCL;
		cmdbuf[1] = 0x00;
		iRet = gp_wtb(cmdbuf);
	}else{
		// SDC transfer
		pChangeAdrs = (char *)malloc(BufLength(adrs));
		AdrsLen = ChangeBuf(adrs, pChangeAdrs);
		// GPIBシステムに接続できるInterfaceは15台まで。+二次アドレス機器を考慮すると
		// (15-REX-PCI20)×31 = 434とおりのアドレス設定が論理的にできるはず。
		// そこで、用意するコマンド送信用のバッファサイズを868byteとする
		if((AdrsLen == 0)||(AdrsLen > 868))
		{
			free(pChangeAdrs);
			return 63;
		}
		cnt = 0;
		cmdbuf[cnt++] = UNL;
		for(cnt2 = 0; cnt2 < AdrsLen; cnt2++)
		{
			// first listener address
			if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
				cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
			// second listener address
			else
				cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
		}

		cmdbuf[cnt++] = SDC;
		cmdbuf[cnt] = 0x00; // NULL

		iRet = gp_wtb(cmdbuf);

		free(pChangeAdrs);
	}
	return iRet;
}

/*****************************************************************************/
/*               gp_myadr                                                    */
/*****************************************************************************/
/*  解説   :PCI-GPIBのGPIBアドレスを取得                                     */
/*          gp_init()で設定されたGPIBアドレスを返します。                    */
/*  引数   :なし                                                             */
/*  戻り値 :PCI-GPIBのGPIBアドレス                                           */
/*****************************************************************************/
int gp_myadr(void)
{
	short	MyUnitId;

	MyUnitId = g_MyUnitId;

    return OpenCtl[MyUnitId].gpibMyAdrs;
}

/*****************************************************************************/
/*               gp_srqCheck                                                 */
/*****************************************************************************/
/*  解説   :SRQラインの状態を返します。                                      */
/*  引数   :なし                                                             */
/*  戻り値 : 1 :SRQがTRUE                                                    */
/*         : 0 :SRQがFALSE                                                   */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_srqCheck(void)
{
	unsigned short	wData;

	// read Bus Status Register
	wData = InPort(BSR);
	if(wData == 0xFFFF) // read error?
		return -1;
	
	if(wData & 0x04) // SRQ is TRUE
		return 1;
	else			// SRQ is FALSE
		return 0;
}

/*****************************************************************************/
/*               gp_wrt                                                      */
/*****************************************************************************/
/*  解説   :リスナを指定し、GPIBバス上にデータ送信                           */
/*  引数   :adrs :GPIB機器アドレスを示す文字列                               */
/*         :buf  :送信文字列                                                 */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :63 :アドレスが正しくない                                         */
/*         :64 :データが正しくない                                           */
/*****************************************************************************/
int gp_wrt(char* adrs, char* buf)
{
	int				iRet;
	short			wRet;
	char			cmdbuf[1024];		// コマンド送信用バッファ
	int				cnt, cnt2;
	char*			pChangeAdrs;
	short			AdrsLen;
	int				bufLen;
	char*			pDataBuf;
	short			MyUnitId;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
		return 63;

	bufLen = BufLength(buf);
	if(bufLen == 0)
		return 64;

	// デリミタコード付加分(3Byte)追加してデータバッファ確保
	pDataBuf = (char *)malloc(bufLen + 3);
	if(pDataBuf == NULL){
		return 64;
	}

	strncpy(pDataBuf, buf, bufLen);

	// デリミタ指定に倣い、デリミタコードを付加する
	// デリミタコード付加に伴い、送信データサイズも更新する。
	if( (OpenCtl[MyUnitId].delim1 & 0x200) // CR+LF
		|| (OpenCtl[MyUnitId].delim1 == 0x00) ) //080805 CR+LF条件の追加
	{
		pDataBuf[bufLen++] = 0x0D; // CR
		pDataBuf[bufLen++] = 0x0A; // LF
	}
	//++ 080808 - デリミタコードが 0x80D, 0x80A の場合にEOIしか出力されないため修正
	else if(OpenCtl[MyUnitId].delim1 == 0x80D)
	{
		pDataBuf[bufLen++] = 0x0D; // CR
	}
	else if(OpenCtl[MyUnitId].delim1 == 0x80A)
	{
		pDataBuf[bufLen++] = 0x0A; // LF
	}
	//--
	else if(OpenCtl[MyUnitId].delim1 & 0x400)
	{
		; // デリミタコードなし
	}
	//++ 080808 - ２バイトの拡張デリミタの場合は設定を行わない
	//else
	else if((OpenCtl[MyUnitId].delim1 != 0x80)
		 && (OpenCtl[MyUnitId].delim1 <= 0xFF))
	{
		pDataBuf[bufLen++] = (char)OpenCtl[MyUnitId].delim1 & 0x7F; // デリミタコードあり
	}
	//--

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		free(pDataBuf);
		return 63;
	}

	// まず、コマンドモード送信を行う
	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	cnt = 0;
	cmdbuf[cnt++] = UNL;
	cmdbuf[cnt++] = TAG | (char)OpenCtl[MyUnitId].gpibMyAdrs;	// 自分をトーカ指定
	for(cnt2 = 0; cnt2 < AdrsLen; cnt2++)
	{
		// 一次リスナアドレス
		if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
			cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
		// 二次アドレス
		else
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
	}

	cmdbuf[cnt] = 0x00;	// NULL終端

	OutPort(AUXCR, ton);	// TMS9914 Talk only mode
	iRet = gp_wtb(cmdbuf);	// コマンドモード送信
	//iRet = ctalk( cmdbuf );

	free(pChangeAdrs);

	if(iRet != 0){
		free(pDataBuf);
		return iRet;
	}

	// データモードのデータ送信を行う
	iRet = gp_dataout(pDataBuf, bufLen, PROTRANSFER);
	if(iRet == 0)
	{
		// デリミタ指定に倣い、デリミタコード分のハンドシェイク数を減らす
		if( (OpenCtl[MyUnitId].delim1 & 0x200) // CR+LF
			|| (OpenCtl[MyUnitId].delim1 == 0x00) ) //080805 CR+LF条件の追加
		{
			g_DataCount = g_DataCount - 2;
		}
		//++ 080808 - デリミタコードが 0x80D, 0x80A の場合にEOIしか出力されないため修正
		else if(OpenCtl[MyUnitId].delim1 == 0x80D)
		{
			g_DataCount = g_DataCount - 1;
		}
		else if(OpenCtl[MyUnitId].delim1 == 0x80A)
		{
			g_DataCount = g_DataCount - 1;			
		}
		//--
		else if(OpenCtl[MyUnitId].delim1 & 0x400)
		{
			; // デリミタコードなし
		}
		//++ 080808 - ２バイトの拡張デリミタの場合はデリミタコードはなし
		//else
		else if((OpenCtl[MyUnitId].delim1 != 0x80)
			 && (OpenCtl[MyUnitId].delim1 <= 0xFF))
		{
			g_DataCount--; // デリミタコードあり
		}
		//--
	}
	if(iRet == 53)
	{
		if(g_DataCount != 0)
			g_DataCount--;

		OutPort(AUXCR, nbaf);
	}

	if( (OpenCtl[MyUnitId].delim1 & 0x800) // EOI
		|| ((OpenCtl[MyUnitId].delim1 <= 0xFF) && (OpenCtl[MyUnitId].delim1 & 0x80)) ) //080805 EOI条件追加
	{
		OutPort(AUXCR, _ton);	// 自身のトーカ解除
	}

	free(pDataBuf);

    return iRet;
}

/*****************************************************************************/
/*               gp_wrtd                                                     */
/*****************************************************************************/
/*  解説   :GPIBバスにデータ送信。データモードのみ                           */
/*  引数   :buf       :送信文字列                                            */
/*         :bufLenOrg :送信文字列長                                          */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :64 :データが正しくない                                           */
/*****************************************************************************/
int gp_wrtd(char* buf, int bufLenOrg)
{
	int				iRet;
	char*			pDataBuf;
	int				bufLen;
	short			MyUnitId;

	MyUnitId = g_MyUnitId;

	bufLen = bufLenOrg;
	if(bufLen == 0)
		return 64;

	pDataBuf = (char *)malloc(bufLen + 3);
	if(pDataBuf == NULL){
		return 64;
	}

	strncpy(pDataBuf, buf, bufLen);

	// デリミタ指定に倣い、デリミタコードを付加する
	// デリミタコード付加に伴い、送信データサイズも更新する。
	if( (OpenCtl[MyUnitId].delim1 & 0x200) // CR+LF
		|| (OpenCtl[MyUnitId].delim1 == 0x00) ) //080805 CR+LF条件の追加
	{
		pDataBuf[bufLen++] = 0x0D; // CR
		pDataBuf[bufLen++] = 0x0A; // LF
	}
	//++ 080808 - デリミタコードが 0x80D, 0x80A の場合にEOIしか出力されないため修正
	else if(OpenCtl[MyUnitId].delim1 == 0x80D)
	{
		pDataBuf[bufLen++] = 0x0D; // CR
	}
	else if(OpenCtl[MyUnitId].delim1 == 0x80A)
	{
		pDataBuf[bufLen++] = 0x0A; // LF
	}
	//--
	else if(OpenCtl[MyUnitId].delim1 & 0x400)
	{
		; // デリミタコードなし
	}
	//++ 080808 - ２バイトの拡張デリミタの場合は設定を行わない
	//else
	else if((OpenCtl[MyUnitId].delim1 != 0x80)
		 && (OpenCtl[MyUnitId].delim1 <= 0xFF))
	{
		pDataBuf[bufLen++] = (char)OpenCtl[MyUnitId].delim1 & 0x7F; // デリミタコードあり
	}
	//--

	iRet = gp_dataout(pDataBuf, bufLen, PROTRANSFER);
	if(iRet == 0)
	{
		if( (OpenCtl[MyUnitId].delim1 & 0x200) // CR+LF
			|| (OpenCtl[MyUnitId].delim1 == 0x00) ) //080805 CR+LF条件の追加
		{
			g_DataCount = g_DataCount - 2;
		}
		//++ 080808 - デリミタコードが 0x80D, 0x80A の場合にEOIしか出力されないため修正
		else if(OpenCtl[MyUnitId].delim1 == 0x80D)
		{
			g_DataCount = g_DataCount - 1;
		}
		else if(OpenCtl[MyUnitId].delim1 == 0x80A)
		{
			g_DataCount = g_DataCount - 1;
		}
		//--
		else if(OpenCtl[MyUnitId].delim1 & 0x400)
		{
			; // デリミタコードなし
		}
		//++ 080808 - ２バイトの拡張デリミタの場合はデリミタコードはなし
		//else
		else if((OpenCtl[MyUnitId].delim1 != 0x80)
			 && (OpenCtl[MyUnitId].delim1 <= 0xFF))
		{
			g_DataCount--; // デリミタコードあり
		}
		//--
	}

	if(iRet == 53)
	{
		if(g_DataCount != 0)
			g_DataCount--;

		OutPort(AUXCR, nbaf);
	}

	if( (OpenCtl[MyUnitId].delim1 & 0x800) //EOI
		|| ((OpenCtl[MyUnitId].delim1 <= 0xFF) && (OpenCtl[MyUnitId].delim1 & 0x80)) ) //080805 EOI条件追加
	{
		OutPort(AUXCR, _ton);
	}

	free(pDataBuf);

    return iRet;
}

/*****************************************************************************/
/*               gp_wait                                                     */
/*****************************************************************************/
/*  解説   :指定時間処理を待機                                               */
/*  引数   :WaitSecTime :wait時間(秒単位)                                    */
/*  戻り値 :なし                                                             */
/*****************************************************************************/
void gp_wait(int WaitSecTime)
{
	int TimeOut;
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv, &tz);
	TimeOut = tv.tv_sec + WaitSecTime;

	while(TimeOut > tv.tv_sec)
	{
		gettimeofday(&tv, &tz);
		;
	}
}

/*****************************************************************************/
/*               gp_trg                                                      */
/*****************************************************************************/
/*  解説   :GET コマンド送信                                                 */
/*  引数   :adrs :GPIB機器アドレスを示す文字列                               */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :63 :アドレスが正しくない                                         */
/*****************************************************************************/
int gp_trg(char* adrs)
{
	int			iRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	char*		pChangeAdrs;
	short		AdrsLen;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
	{
		return 63;
	}

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		return 63;
	}

	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	cnt = 0;
	cmdbuf[cnt++] = UNL;
	for(cnt2 = 0; cnt2 < AdrsLen; cnt2++)
	{
		if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
			cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
		else
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
	}

	cmdbuf[cnt++] = GET;
	cmdbuf[cnt] = 0x00;

	iRet = gp_wtb(cmdbuf);

	free(pChangeAdrs);
    return iRet;
}

/*****************************************************************************/
/*               gp_wsrq                                                     */
/*****************************************************************************/
/*  解説   :指定時間SRQを待つ                                                */
/*  引数   :WaitSecTime :指定時間(秒単位)                                    */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :タイムアウト                                                 */
/*****************************************************************************/
int gp_wsrq(int WaitSecTime)
{
	unsigned char	bData;
	int	TimeOut;
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv, &tz);
	TimeOut = tv.tv_sec + WaitSecTime;

	while(TimeOut > tv.tv_sec)
	{
		gettimeofday(&tv, &tz);
		bData = wsrq();
		if(bData & 0x02)
			return 0;
	}

	return -1;
}

unsigned char wsrq(void)
{
	unsigned short wData;
	unsigned char bData;

	wData = InPort(ISR1);

	if(wData == 0xFFFF)
		bData = 0;
	else
		bData = (unsigned char)(wData & 0xFF);

    return bData;
}

/*****************************************************************************/
/*               gp_wsrqb                                                    */
/*****************************************************************************/
/*  解説   :指定時間SRQを待つ(バスステータスを見る)                          */
/*  引数   :WaitSecTime :指定時間(秒単位)                                    */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :タイムアウト                                                 */
/*****************************************************************************/
int gp_wsrqb(int WaitSecTime)
{
	unsigned char	bData;
	int	TimeOut;
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv, &tz);
	TimeOut = tv.tv_sec + WaitSecTime;

	while(TimeOut > tv.tv_sec)
	{
		gettimeofday(&tv, &tz);
		bData = wsrqb();
		if(bData & 0x04)
			return 0;
	}

	return -1;
}

unsigned char wsrqb(void)
{
	unsigned short wData;
	unsigned char bData;

	wData = InPort(BSR);

	if(wData == 0xFFFF)
		bData = 0;
	else
		bData = (unsigned char)(wData & 0xFF);

    return bData;
}

//
//  Set master mode ( ATN "true" )
//
void setmst()
{
	gp_delay( g_DelayCount );
	OutPort(AUXCR, tca);
	gp_delay( g_DelayCount );
}

//
//  standby
//      set  ATN ----> faulse
//
void standby()
{
	gp_delay( g_DelayCount );
	OutPort(AUXCR, gts);
	gp_delay( g_DelayCount );
}


/*****************************************************************************/
/*               cah                                                         */
/*****************************************************************************/
/*  解説   :1バイトデータ入力                                                */
/*  引数   :Data :入力データ返答変数ヘのポインタ                             */
/*         :Eoi  :eoi検出判定変数ヘのポインタ                                */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*****************************************************************************/
char cah(unsigned char *Data, unsigned char *Eoi)
{
	unsigned short wRet;
	short	MyUnitId;

	MyUnitId = g_MyUnitId;

	SetTimeOut1(OpenCtl[MyUnitId].gpibTimeout);

	// 送信完了するかタイムアウトになるまで待つ
	while(1)
	{
		// インタラプトステータスレジスタ0をリード
		wRet = InPort(ISR0);
		if(wRet == 0xFFFF)
			return -1;

		// B1ビット(0x20)またはENDビット(0x08)が立っていたら受信完了
		if(wRet & 0x28)
		{
			break;
		}

		// 指定時間経過したらタイムアウトエラー
		if(IsTimeOut1())
			return 53;
	} // End of while(1)

	// データインレジスタからデータをリード
	*Data = (unsigned char)InPort(DIR);

	if(wRet & 0x08){
		*Eoi = 1;
	}else{
		*Eoi = 0;
	}

	return 0;
}

/*****************************************************************************/
/*               csh                                                         */
/*****************************************************************************/
/*  解説   :1バイトデータ出力                                                */
/*  引数   :Data :出力データ                                                 */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*****************************************************************************/
char csh(unsigned char Data)
{
	unsigned short wRet;
	short	MyUnitId;

	MyUnitId = g_MyUnitId;

	SetTimeOut1(OpenCtl[MyUnitId].gpibTimeout);

	// データアウトレジスタに対してデータをライト
	wRet = OutPort(CDOR, Data);
	if(wRet == 0xFFFF)
		return -1;

	// 送信完了するかタイムアウトになるまで待つ
	while(1)
	{
		// インタラプトステータスレジスタ0をリード
		wRet = InPort(ISR0);
		// B0ビット(0x10)が立っていたら送信完了
		if(wRet & 0x10)
			break;

		if(IsTimeOut1())
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: csh error [%x] \n", wRet);
#endif
			return 53;
		}
	} // End of while(1)

	return 0;
}

//
//  CRLK1 : UNL -- LAG -- Command byte
//
int ctlk1( char *buf )
{
	int		error_sts;

	setmst();		// ATN ---> true

	error_sts = csh( buf[0] );		// UNL command out
	if ( error_sts != 0 )
	{
 		standby();
		return -1;
	}

	error_sts = csh( buf[1] );
 	if ( error_sts != 0 )
 	{
 		standby();
 		return -1;
 	}
	return 0;
}

//
//   CTALK  : UNL--- MTA --- LA
//
int ctalk( char *buf )
{
	int		error_sts;

	setmst();		// ATN ---> true

	OutPort(AUXCR, ton);	// TMS9914 Talk only mode

	error_sts = csh( buf[0] );	// UNL  command
	if ( error_sts != 0 )
	{
		standby();
		return 53;
	}

	error_sts = csh( buf[1] );	// address code sw readout & my talk adrs set
	if ( error_sts != 0 )
	{
		standby();
		return 53;
	}

	error_sts = csh( buf[2] );
 	if ( error_sts != 0 )
 	{
 		standby();
		return 53;
 	}
	standby();
	return 0;
}

//
//   CTALK2  : UNT--- UNL--- MTA --- LA
//
int ctalk2( char *buf )
{
	int		error_sts;

	setmst();		// ATN ---> true

	error_sts = csh( buf[0] );				// UNT  command
	if ( error_sts != 0 )
	{
		standby();
		return 53;
	}

	error_sts = csh( buf[1] );				// UNL  command
	if ( error_sts != 0 )
	{
		standby();
		return 53;
	}

	error_sts = csh( buf[2] );	// address code sw readout & my talk adrs set
	if ( error_sts != 0 )
	{
		standby();
		return 53;
	}

	error_sts = csh( buf[3] );
 	if ( error_sts != 0 )
 	{
 		standby();
		return 53;
 	}

	OutPort(AUXCR, ton);	// TMS9914 Talk only mode
	standby();
	return 0;
}

//
// CLISTN  : UNL -- MLA -- TA
//
int clistn( char *buf )
{
	int		error_sts;

	setmst();		// ATN True

	error_sts = csh( buf[0] );	// UNL command out
	if ( error_sts != 0 )
	{
		standby();
 		return 53;
	}
 	error_sts = csh( buf[1] );	// TAG pick up & output
 	if ( error_sts != 0 )
 	{
		standby();
 		return 53;
 	}

	error_sts = csh( buf[2] );	// send LAG
 	if ( error_sts != 0 )
 	{
		standby();
 		return 53;
 	}

	error_sts = csh( buf[3] ); 	// address readout & my litn address set
	if ( error_sts != 0 ) 
	{
		standby();
 		return 53;
	}

	OutPort(AUXCR, lon);	// set listn mode
	standby();
	return 0;
}

/*****************************************************************************/
/*               gp_rds                                                      */
/*****************************************************************************/
/*  解説   :シリアルポールを実行。ステータスバイトを取得                     */
/*  引数   :adrs        :GPIB機器アドレスを示す文字列                        */
/*         :status_byte :ステータスバイトを格納するバッファ                  */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :63 :アドレスが正しくない                                         */
/*****************************************************************************/
int gp_rds(char* adrs, unsigned char* status_byte)
{
	int			iRet;
	short		wRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	int			num;
	char*		pChangeAdrs;
	short		AdrsLen;
	short		MyUnitId;
	unsigned char	eoi;

#ifdef _DEBUG
	printf( adrs );
	printf("\n");
#endif

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
		return 63;

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
#ifdef _DEBUG
	printf("AdrsLen = %04x \n", AdrsLen );
#endif
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		return 63;
	}

	wRet = OutPort(AUXCR, hdfa);	// データ入力ごとにRFDホールドオフがおこるようにセット

	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	cnt = 0;
	cnt2 = 0;

	// 送信コマンドを準備
	cmdbuf[cnt++] = UNL;
	cmdbuf[cnt++] = LAG | (char)OpenCtl[MyUnitId].gpibMyAdrs;
	cmdbuf[cnt++] = SPE;
	cmdbuf[cnt++] = TAG | pChangeAdrs[cnt2++];

#ifdef _DEBUG
	printf("OpenCtl[MyUnitId].gpibMyAdrs = %04x, pChangeAdrs[cnt2++] = %04x\n", 
			OpenCtl[MyUnitId].gpibMyAdrs, pChangeAdrs[cnt2++]);
#endif

	// 二次アドレスの指定があれば二次アドレスを続けてセット
	if((AdrsLen > cnt2)&&(pChangeAdrs[cnt2] & SAG))
		cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2++];
	cmdbuf[cnt++] = 0x00;

	//wRet = OutPort(AUXCR, hdfa);	// set HDF Mode
	wRet = OutPort(AUXCR, lon);	// set listn mode
	iRet = gp_wtb(cmdbuf);	// コマンドモード送信
	if(iRet != 0)
	{
		return iRet;
	}

	// ステータスバイトを受信
	num = 0;
	if(cah(status_byte+num, &eoi))
	{
		wRet = OutPort(AUXCR, rhdf);	// RFDホールドオフを解除する
		wRet = OutPort(AUXCR, _hdfa);	// データ入力毎にRFDホールドオフがおきないようにセット
		free(pChangeAdrs);
		return 53;
	}

	// cnt2のここでの初期値は1か2
	// ひきつづき、トーカアドレスの指定があるとき
	for(; cnt2 < AdrsLen; cnt2++)
	{
		cnt = 0;
		cmdbuf[cnt++] = TAG | pChangeAdrs[cnt2++];
		// 二次アドレスの指定があれば二次アドレスを続けて送信する
		if((AdrsLen > cnt2)&&(pChangeAdrs[cnt2] & SAG))
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
		else
			cnt2--;

		cmdbuf[cnt++] = 0x00;

		iRet = gp_wtb(cmdbuf);	// コマンドモード送信
		wRet = OutPort(AUXCR, rhdf);	// RFDホールドオフを解除する

		// ステータスバイトを受信
		num++;
		if(cah(status_byte+num, &eoi))
		{
			wRet = OutPort(AUXCR, rhdf);
			wRet = OutPort(AUXCR, _hdfa);
			free(pChangeAdrs);
			return 53;
		}
	} // End of for(; cnt2 < AdrsLen; cnt2++)

	// 送信コマンドを準備
	cmdbuf[0] = SPD;
	cmdbuf[1] = UNT;
	cmdbuf[2] = 0x00;

	iRet = gp_wtb(cmdbuf);	// コマンドモードを送信

	wRet = OutPort(AUXCR, rhdf);
	wRet = OutPort(AUXCR, _hdfa);

	free(pChangeAdrs);
	return iRet;
}

/*****************************************************************************/
/*               gp_rds1                                                     */
/*****************************************************************************/
/*  解説   :シリアルポールを実行。ステータスバイトを取得                     */
/*          最後にUNTを発行しません                                          */
/*  引数   :adrs        :GPIB機器アドレスを示す文字列                        */
/*         :status_byte :ステータスバイトを格納するバッファ                  */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :63 :アドレスが正しくない                                         */
/*****************************************************************************/
int gp_rds1(char* adrs, unsigned char* status_byte)
{
	int			iRet;
	short		wRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	int			num;
	char*		pChangeAdrs;
	short		AdrsLen;
	short		MyUnitId;
	unsigned char	eoi;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
		return 63;

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		return 63;
	}

	wRet = OutPort(AUXCR, hdfa);

	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	cnt = 0;
	cnt2 = 0;

	cmdbuf[cnt++] = UNL;
	cmdbuf[cnt++] = LAG | (char)OpenCtl[MyUnitId].gpibMyAdrs;
	cmdbuf[cnt++] = SPE;
	cmdbuf[cnt++] = TAG | pChangeAdrs[cnt2++];
	if((AdrsLen > cnt2)&&(pChangeAdrs[cnt2] & SAG))
		cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2++];
	cmdbuf[cnt++] = 0x00;

	//wRet = OutPort(AUXCR, hdfa);	// set HDF Mode
	wRet = OutPort(AUXCR, lon);	// set listn mode
	iRet = gp_wtb(cmdbuf);	// コマンドモード送信
	if(iRet != 0)
	{
		return iRet;
	}

	num = 0;
	if(cah(status_byte+num, &eoi))
	{
		wRet = OutPort(AUXCR, rhdf);
		wRet = OutPort(AUXCR, _hdfa);
		free(pChangeAdrs);
		return 53;
	}

	for(; cnt2 < AdrsLen; cnt2++)
	{
		cnt = 0;
		cmdbuf[cnt++] = TAG | pChangeAdrs[cnt2++];
		if((AdrsLen > cnt2)&&(pChangeAdrs[cnt2] & SAG))
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
		else
			cnt2--;

		cmdbuf[cnt++] = 0x00;

		iRet = gp_wtb(cmdbuf);
		wRet = OutPort(AUXCR, rhdf);

		num++;
		if(cah(status_byte+num, &eoi))
		{
			wRet = OutPort(AUXCR, rhdf);
			wRet = OutPort(AUXCR, _hdfa);
			free(pChangeAdrs);
			return 53;
		}
	} // End of for(; cnt2 < AdrsLen; cnt2++)

	cmdbuf[0] = SPD;
	cmdbuf[1] = 0x00;

	iRet = gp_wtb(cmdbuf);

	wRet = OutPort(AUXCR, rhdf);
	wRet = OutPort(AUXCR, _hdfa);

	free(pChangeAdrs);
	return iRet;
}


/*****************************************************************************/
/*               gp_red                                                      */
/*****************************************************************************/
/*  解説   :トーカ指定した機器から、データ受信する                           */
/*  引数   :adrs   :GPIB機器アドレスを示す文字列                             */
/*         :buf    :受信文字列用バッファ                                     */
/*         :bufLen :受信文字列用バッファサイズ                               */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :61 :受信バッファオーバーフロー                                   */
/*         :63 :アドレスが正しくない                                         */
/*         :64 :受信文字列用バッファサイズが正しくない                       */
/*****************************************************************************/
int gp_red(char* adrs, char* buf, int bufLen)
{
	int			iRet;
	short		wRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	char*		pChangeAdrs;
	short		AdrsLen;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
		return 63;
	if(bufLen == 0)
		return 64;

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		return 63;
	}

	memset(cmdbuf, 0x00, sizeof(cmdbuf));

	cnt = 0;
	cmdbuf[cnt++] = UNL;
	cnt2 = 0;
	cmdbuf[cnt++] = TAG | pChangeAdrs[cnt2++];
	if((AdrsLen >= 2)&&(pChangeAdrs[cnt2] & SAG))
		cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2++];
	else
		;

	cmdbuf[cnt++] = LAG | (char)OpenCtl[MyUnitId].gpibMyAdrs;

	for(; cnt2 < AdrsLen; cnt2++)
	{
		if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
			cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
		else
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
	}

	cmdbuf[cnt] = 0x00;

	OutPort(AUXCR, hdfa);	// add 05/12/22
	wRet = OutPort(AUXCR, lon);	// set listn mode
	iRet = gp_wtb(cmdbuf);
	//iRet = clistn(cmdbuf);

	free(pChangeAdrs);

	if(iRet != 0)
	{
		return iRet;
	}

	iRet = gp_datain(buf, (unsigned int)bufLen, PROTRANSFER);
	wRet = OutPort(AUXCR, _lon);	// add 05/12/22
	wRet = OutPort(AUXCR, rhdf);	// add 05/12/22

	// 0が返ってきたときはデリミタ未確認でバッファサイズ分のデータ受信完了
	if(iRet == 0)
		return 61;
	// デリミタ受信正常終了のときは、UNTを発行
	else if(iRet == 24)
	{
		// UNTを発行し、ハンドシェイクを終了する
		gp_datain_End();
	    return 0;
	}
	// GPIBバスタイムアウトのときは、UNTを発行
	else if(iRet == 53)
	{
		gp_datain_End();
	    return iRet;
	}
	else
	    return iRet;
}

/*****************************************************************************/
/*               gp_redd                                                     */
/*****************************************************************************/
/*  解説   :GPIBバスからデータ受信。コマンドモードなしでデータモードのみ     */
/*  引数   :buf    :受信文字列用バッファ                                     */
/*         :bufLen :受信文字列用バッファサイズ                               */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :24 :デリミタを受信して正常終了                                   */
/*         :53 :バスタイムアウト                                             */
/*         :64 :受信文字列用バッファサイズが正しくない                       */
/*****************************************************************************/
int gp_redd(char* buf, int bufLen)
{
	int			iRet;
	short		wRet;

	if(bufLen == 0)
		return 64;

	iRet = gp_datain(buf, (unsigned int)bufLen, PROTRANSFER);

	if((iRet == 0)||(iRet == 24))
	{
		wRet = OutPort(AUXCR, rhdf);	// add 05/12/22
		wRet = OutPort(AUXCR, _hdfa);
		return iRet;
	}
	else
	    return iRet;
}

/*****************************************************************************/
/*               gp_redah                                                    */
/*****************************************************************************/
/*  解説   :トーカ指定した機器から、データ受信する                           */
/*  gp_red()との違いは、最後にUNTを送出しない点と最後にRFDホールドオフとなる */
/*  この関数は自作の機器相手で最後にハンドシェイクするとおかしくなる機器用   */
/*  引数   :adrs   :GPIB機器アドレスを示す文字列                             */
/*         :buf    :受信文字列用バッファ                                     */
/*         :bufLen :受信文字列用バッファサイズ                               */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :61 :受信バッファオーバーフロー                                   */
/*         :63 :アドレスが正しくない                                         */
/*         :64 :受信文字列用バッファサイズが正しくない                       */
/*****************************************************************************/
int gp_redah(char* adrs, char* buf, int bufLen)
{
	int			iRet;
	short		wRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	char*		pChangeAdrs;
	short		AdrsLen;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
		return 63;
	if(bufLen == 0)
		return 64;

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		return 63;
	}

	memset(cmdbuf, 0x00, sizeof(cmdbuf));

	cnt = 0;
	cmdbuf[cnt++] = UNL;
	cnt2 = 0;
	cmdbuf[cnt++] = TAG | pChangeAdrs[cnt2++];
	if((AdrsLen >= 2)&&(pChangeAdrs[cnt2] & SAG))
		cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2++];
	else
		;

	cmdbuf[cnt++] = LAG | (char)OpenCtl[MyUnitId].gpibMyAdrs;

	for(; cnt2 < AdrsLen; cnt2++)
	{
		if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
			cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
		else
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
	}

	cmdbuf[cnt] = 0x00;

	OutPort(AUXCR, hdfa);	// add 05/12/22
	wRet = OutPort(AUXCR, lon);	// set listn mode
	iRet = gp_wtb(cmdbuf);
	//iRet = clistn(cmdbuf);

	free(pChangeAdrs);

	if(iRet != 0)
	{
		return iRet;
	}

	iRet = gp_datain(buf, (unsigned int)bufLen, PROTRANSFER);

	if(iRet == 0)
		return 61;
	else if(iRet == 24)
	{
	    return 0;
	}
	else
	    return iRet;
}

/*****************************************************************************/
/*               gp_redrst                                                   */
/*****************************************************************************/
/*  解説   :自分自身のリスナ解除とRFDホールドオフ解除                        */
/*   この関数は自作の機器相手で最後にハンドシェイクするとおかしくなる機器用  */
/*  引数   :なし                                                             */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_redrst(void)
{
	unsigned short wRet;

	wRet = OutPort(AUXCR, _lon);	// 自分をリスナから解除
	wRet = OutPort(AUXCR, rhdf);	// RFDホールドオフを解除

	if(wRet == 0xFFFF)
		return -1;
	else
		return 0;
}


/*****************************************************************************/
/*               gp_lcl                                                      */
/*****************************************************************************/
/*  解説   :指定した機器をローカル状態に設定                                 */
/*  引数   : adrs :GPIB機器アドレスを示す文字列                              */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :63 :アドレスが正しくない                                         */
/*****************************************************************************/
int gp_lcl(char* adrs)
{
	int			iRet;
	unsigned short wRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	char*		pChangeAdrs;
	short		AdrsLen;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
	{
		// disable REN line
		wRet = OutPort(AUXCR, _sre);
		if(wRet == 0xFFFF)
			return -1;
		else
			iRet = 0;
	}
	else
	{
		// send GTL command
		pChangeAdrs = (char *)malloc(BufLength(adrs));
		AdrsLen = ChangeBuf(adrs, pChangeAdrs);
		if((AdrsLen == 0)||(AdrsLen > 868))
		{ 
			free(pChangeAdrs);
			return 63;
		}

		memset(cmdbuf, 0x00, sizeof(cmdbuf));
		cnt = 0;
		cmdbuf[cnt++] = UNL;
		for(cnt2 = 0; cnt2 < AdrsLen; cnt2++)
		{
			if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
				cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
			else
				cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
		}

		cmdbuf[cnt++] = GTL;
		cmdbuf[cnt] = 0x00;

		iRet = gp_wtb(cmdbuf);

		free(pChangeAdrs);
	}

    return iRet;
}

/*****************************************************************************/
/*               gp_llo                                                      */
/*****************************************************************************/
/*  解説   :LLO コマンド送信                                                 */
/*  引数   :なし                                                             */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*****************************************************************************/
int gp_llo(void)
{
	int				iRet;
	char			cmdbuf[4];

	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	cmdbuf[0] = LLO;
	cmdbuf[1] = 0x00;

	iRet = gp_wtb(cmdbuf);

    return iRet;
}

/*****************************************************************************/
/*               gp_count                                                    */
/*****************************************************************************/
/*  解説   :GPIB機器からの受信データ数、                                     */
/*          またはGPIB機器へ送信完了したデータ数を取得します。               */
/*  引数   :なし                                                             */
/*  戻り値 :受信データ数または送信データ数が返されます。                     */
/*****************************************************************************/
int gp_count(void)
{
	return g_DataCount;
}

/*****************************************************************************/
/*               gp_delm                                                     */
/*****************************************************************************/
/*  解説   :デリミタを設定します。                                           */
/*       送信時のデリミタコードはDLL内でデータ付加することとする。           */
/*       dlm = 2Byteで設定(拡張デリミタ)                                     */
/*       00001110xxxxxxxxb                                                   */
/*           |||---------------- CR+LF                                       */
/*           |------------------ None                                        */
/*           |------------------ EOI                                         */
/*  引数   :mode :トーカまたはリスナを指定する文字列                         */
/*         :dlm  :デリミタ設定パラメータ                                     */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*****************************************************************************/
int gp_delm(char *mode, unsigned int dlm)
{
	int iRet = 0;
	short MyUnitId;

	MyUnitId = g_MyUnitId;

	switch(mode[0] | 0x20)
	{
	case 't':	// talker
		OpenCtl[MyUnitId].delim1	= dlm & 0xFF; //080805 通常は1バイトで入力されるため上位バイトは削除
		break;
	case 'l':	// listener
		OpenCtl[MyUnitId].delim2	= dlm & 0xFF; //080805 通常は1バイトで入力されるため上位バイトは削除
		break;
	case 'b':
		OpenCtl[MyUnitId].delim1	= dlm;
		OpenCtl[MyUnitId].delim2	= dlm;
		break;
	default:
		iRet = -1;
		return iRet;
	}

#ifdef _DEBUG
	printf("gp_delm: delim1 = %04x, delim2 = %04x\n", 
			OpenCtl[MyUnitId].delim1, OpenCtl[MyUnitId].delim2);
#endif

	return iRet;
}

void gp_strtodbl(unsigned char *bPoint, double *val)
{
	double *retVal;

	retVal = (double *)bPoint; 
	*val = *retVal;
}

void gp_strtoflt(unsigned char *bPoint, float *val)
{
	float *retVal;

	retVal = (float *)bPoint; 
	*val = *retVal;
}

/*****************************************************************************/
/*               gp_tfrout                                                   */
/*****************************************************************************/
/*  解説   :リスナを指定し、GPIBバス上にデータ送信。                         */
/*          PCI-GPIB間のデータ転送はDMA転送にて行います。                    */
/*  引数   :adrs   :GPIB機器アドレスを示す文字列                             */
/*         :bufLen :送信データサイズ                                         */
/*         :buf    :送信データ格納バッファ                                   */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :63 :アドレスが正しくない                                         */
/*         :64 :データが正しくない                                           */
/*****************************************************************************/
int gp_tfrout(char *adrs, int bufLen, char *buf)
{
	int			iRet;
	unsigned short wRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	char*		pChangeAdrs;
	short		AdrsLen;
	unsigned short wtmp1, wtmp2;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
		return 63;
	if(bufLen == 0)
		return 64;

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		return 63;
	}

	// デリミタ指定を一次的に保存しておき、送信完了後、元に戻す
	wtmp1 = OpenCtl[MyUnitId].delim1;
	wtmp2 = OpenCtl[MyUnitId].delim2;

	//080806 - この設定ではトーカに0x00が設定されてしまうため修正
	//iRet = gp_delm("t", 0xC00);
	iRet = gp_delm("b", 0xC00);

	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	cnt = 0;
	cmdbuf[cnt++] = UNL;
	cmdbuf[cnt++] = TAG | (char)OpenCtl[MyUnitId].gpibMyAdrs;
	for(cnt2 = 0; cnt2 < AdrsLen; cnt2++)
	{
		if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
			cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
		else
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
	}

	cmdbuf[cnt] = 0x00;

	OutPort(AUXCR, ton);	// TMS9914 Talk only mode
	iRet = gp_wtb(cmdbuf);
	//iRet = ctalk( cmdbuf );

	free(pChangeAdrs);

	if(iRet != 0)
	{
		// デリミタ指定を元に戻す
		OpenCtl[MyUnitId].delim1 = wtmp1;
		OpenCtl[MyUnitId].delim2 = wtmp2;
		return iRet;
	}

	iRet = gp_dataout(buf, (int)bufLen, DMATRANSFER);

	if(iRet == 53)
	{
		if(g_DataCount != 0)
			g_DataCount--;

		OutPort(AUXCR, nbaf);
	}

	if( (OpenCtl[MyUnitId].delim1 & 0x800) // EOI
		|| ((OpenCtl[MyUnitId].delim1 <= 0xFF) && (OpenCtl[MyUnitId].delim1 & 0x80)) ) //080805 EOI条件追加
	{
		OutPort(AUXCR, _ton);
	}

	// デリミタ指定を元に戻す
	OpenCtl[MyUnitId].delim1 = wtmp1;
	OpenCtl[MyUnitId].delim2 = wtmp2;

    return iRet;
}

/*****************************************************************************/
/*               gp_tfroutd                                                  */
/*****************************************************************************/
/*  解説   :GPIBバス上にデータ送信。                                         */
/*          PCI-GPIB間のデータ転送はDMA転送にて行います。                    */
/*  引数   :bufLen :送信データサイズ                                         */
/*         :buf    :送信データ格納バッファ                                   */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :53 :バスタイムアウト                                             */
/*         :64 :データが正しくない                                           */
/*****************************************************************************/
int gp_tfroutd(int bufLen, char *buf)
{
	int			iRet;
	unsigned short wtmp1, wtmp2;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	if(bufLen == 0)
		return 64;

	// デリミタ指定を一次的に保存しておき、送信完了後、元に戻す
	wtmp1 = OpenCtl[MyUnitId].delim1;
	wtmp2 = OpenCtl[MyUnitId].delim2;

	//080806 - この設定ではトーカに0x00が設定されてしまうため修正
	//iRet = gp_delm("t", 0xC00);
	iRet = gp_delm("b", 0xC00);

	iRet = gp_dataout(buf, (int)bufLen, DMATRANSFER);
	if(iRet == 53)
	{
		if(g_DataCount != 0)
			g_DataCount--;

		OutPort(AUXCR, nbaf);
	}

	if( (OpenCtl[MyUnitId].delim1 & 0x800) // EOI
		|| ((OpenCtl[MyUnitId].delim1 <= 0xFF) && (OpenCtl[MyUnitId].delim1 & 0x80)) ) //080805 EOI条件追加
	{
		OutPort(AUXCR, _ton);
	}

	OpenCtl[MyUnitId].delim1 = wtmp1;
	OpenCtl[MyUnitId].delim2 = wtmp2;

    return iRet;
}

int gp_tfrout_unt(char *adrs, int bufLen, char *buf)
{
	int				iRet;
	char			cmdbuf[4];

	iRet = gp_tfrout(adrs, bufLen, buf);
	if(iRet != 0)
		return iRet;

	memset(cmdbuf, 0x00, sizeof(cmdbuf));

	cmdbuf[0] = UNT;
	cmdbuf[1] = 0x00;

	iRet = gp_wtb(cmdbuf);

	return iRet;
}

/*****************************************************************************/
/*               gp_tfrin                                                    */
/*****************************************************************************/
/*  解説   :トーカ指定した機器から、データ受信する。                         */
/*          PCI-GPIB間のデータ転送はDMA転送にて行います。                    */
/*  引数   :adrs   :GPIB機器アドレスを示す文字列                             */
/*         :bufLen :受信データサイズ                                         */
/*         :buf    :受信データ格納バッファ                                   */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :ioctlエラー                                                  */
/*         :61 :受信バッファオーバーフロー                                   */
/*         :53 :バスタイムアウト                                             */
/*         :63 :アドレスが正しくない                                         */
/*         :64 :データが正しくない                                           */
/*****************************************************************************/
int gp_tfrin(char *adrs, int bufLen, char *buf)
{
	int			iRet;
	unsigned short wRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	char*		pChangeAdrs;
	short		AdrsLen;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
		return 63;
	if(bufLen == 0)
		return 64;

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		return 63;
	}

	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	cnt = 0;
	cmdbuf[cnt++] = UNL;
	cnt2 = 0;
	cmdbuf[cnt++] = TAG | pChangeAdrs[cnt2++];
	if((AdrsLen >= 2)&&(pChangeAdrs[cnt2] >= 96)&&(pChangeAdrs[cnt2] <= 126))
		cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2++];
	else
		;

	cmdbuf[cnt++] = LAG | (char)OpenCtl[MyUnitId].gpibMyAdrs;

	for(; cnt2 < AdrsLen; cnt2++)
	{
		if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
			cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
		else
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
	}

	cmdbuf[cnt] = 0x00;

	iRet = gp_wtb(cmdbuf);
	//iRet = clistn(cmdbuf);
	wRet = OutPort(AUXCR, lon);	// set listn mode

	free(pChangeAdrs);

#if 0
	wRet = InPort(ISR0);
	if(wRet & 0x20)
		;
	else
	{
		OutPort(AUXCR, rhdf);
	}
#endif

	iRet = gp_datain(buf, (int)bufLen, DMATRANSFER);
	wRet = OutPort(AUXCR, rhdf);
	wRet = OutPort(AUXCR, _lon);	// reset listn only

	if(iRet == 0)
		return 61;
	else if(iRet == 24)
	{
		wRet = InPort(ISR0);
		gp_datain_End();
	    return 0;
	}
	else if(iRet == 53)
	{
		gp_datain_End();
	    return iRet;
	}
	else
	    return iRet;
}

int gp_tfrinit(char *adrs)
{
	int			iRet;
	unsigned short wRet;
	char		cmdbuf[1024];
	int			cnt, cnt2;
	char*		pChangeAdrs;
	short		AdrsLen;
	short		MyUnitId;

	MyUnitId = g_MyUnitId;

	if(BufLength(adrs) == 0)
		return 63;

	pChangeAdrs = (char *)malloc(BufLength(adrs));
	AdrsLen = ChangeBuf(adrs, pChangeAdrs);
	if((AdrsLen == 0)||(AdrsLen > 868))
	{ 
		free(pChangeAdrs);
		return 63;
	}

	memset(cmdbuf, 0x00, sizeof(cmdbuf));
	cnt = 0;
	cmdbuf[cnt++] = UNL;
	cnt2 = 0;
	cmdbuf[cnt++] = TAG | pChangeAdrs[cnt2++];
	if((AdrsLen >= 2)&&(pChangeAdrs[cnt2] >= 96)&&(pChangeAdrs[cnt2] <= 126))
		cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2++];
	else
		;

	cmdbuf[cnt++] = LAG | (char)OpenCtl[MyUnitId].gpibMyAdrs;

	for(; cnt2 < AdrsLen; cnt2++)
	{
		if((pChangeAdrs[cnt2] >= 0)&&(pChangeAdrs[cnt2] <= 30))
			cmdbuf[cnt++] = LAG | pChangeAdrs[cnt2];
		else
			cmdbuf[cnt++] = SAG | pChangeAdrs[cnt2];
	}

	cmdbuf[cnt] = 0x00;

	iRet = gp_wtb(cmdbuf);
	//iRet = clistn(cmdbuf);
	wRet = OutPort(AUXCR, lon);	// set listn mode

	free(pChangeAdrs);

    return iRet;
}

int gp_tfrins(int bufLen, char *buf)
{
	int				iRet;
	unsigned short  wRet;

	if(bufLen == 0)
		return 64;

	iRet = gp_datain(buf, (int)bufLen, DMATRANSFER);
	wRet = OutPort(AUXCR, rhdf);
	wRet = OutPort(AUXCR, _lon);	// reset listn only

	if(iRet == 24)
	{
		wRet = InPort(ISR0);
		iRet = 0;
	}

    return iRet;
}

void gp_tfrend(void)
{
	int				iRet;

	iRet = gp_datain_End();
}

/*****************************************************************************/
/*               gp_findlstn                                                 */
/*****************************************************************************/
/*  解説   :リスナ機器を検出します。(IEEE488.2仕様書 第17章より)             */
/*          検出したGPIBアドレスは、文字列に変換してadrsに格納されます。     */
/*          本関数呼び出し前に、gp_init,gp_clrを呼び出す必要があります。     */
/*  引数   :adrs:検出したGPIBアドレスを格納するバッファ                      */
/*         :adrsLen:adrsバッファの長さ                                       */
/*  戻り値 : 0     :リスナ未検出                                             */
/*         :-1     :システムコールエラー                                     */
/*         :整数値 :リスナ検出台数                                           */
/*         :53     :バスタイムアウト                                         */
/*         :64     :adrsLenエラー                                            */
/*****************************************************************************/
int gp_findlstn(char *adrs, int adrsLen)
{
	int				iRet;
	unsigned short  wRet;
	int				cnt, cnt2, cnt3, num;
	char			cmdbuf[256];

	if(adrsLen == 0)
		return 64;

	cnt3 = 0;
	num = 0;
	// コマンドモードで随時リスナアドレスを送信し(0〜30)、アクセプタハンドシェイクが
	// 完了すれば、リスナ検出となる。
	for(cnt = 0; cnt < 31; cnt++)
	{
		cmdbuf[0] = UNL;
		cmdbuf[1] = (char)(LAG + cnt);
		cmdbuf[2] = 0x00;
		iRet = gp_wtb(cmdbuf);
		if(iRet != 0)
		{
			return iRet;
		}	

		sleep(2/1000);

		OutPort(AUXCR, ton);	// ton送信しないとNDACの状態取得できない
		wRet = InPort(BSR);	// バスステータスレジスタ読み込み

		if(wRet & 0x20)	// NDACがTRUEであれば、GPIBアドレス検出
		{
			if(cnt3 + 3 > (int)adrsLen)
				return 64;

			if(cnt3 != 0)
			{
				adrs[cnt3++] = 0x2C;	// カンマをセット
			}
			// 他関数でそのまま使用できるよう、ASCIIに変換してアドレスセット
			if(cnt < 10)
			{
				adrs[cnt3++] = (char)(0x30 + cnt);
			}
			else if((cnt >= 10)&&(cnt < 20)) // 10-19
			{
				adrs[cnt3++] = (char)(0x31);
				adrs[cnt3++] = (char)(0x30 + (cnt-10));
			}
			else if((cnt >= 20)&&(cnt < 30)) // 20-29
			{
				adrs[cnt3++] = (char)(0x32);
				adrs[cnt3++] = (char)(0x30 + (cnt-20));
			}
			else // 30
			{
				adrs[cnt3++] = (char)(0x33);
				adrs[cnt3++] = (char)(0x30 + (cnt-30));
			}
			num++;	// 検出台数を更新
		}
		else
		{
			// 一時アドレスで検出できなければ、二次アドレス検出を行う
			// まず、二次アドレスを全て送信
			for(cnt2 = 0; cnt2 < 31; cnt2++)
			{
				cmdbuf[cnt2] = (char)(SAG + cnt2);
			}
			cmdbuf[cnt2] = 0x00;
			iRet = gp_wtb(cmdbuf);
			if(iRet != 0)
			{
				return iRet;
			}

			sleep(2/1000);

			OutPort(AUXCR, ton);
			wRet = InPort(BSR);
			if(!(wRet & 0x20))
			{
				continue;
			}

			// DNACがTRUEの場合、二次アドレスの検出を行う
			cmdbuf[0] = UNL;
			cmdbuf[1] = (char)(LAG + cnt);
			cmdbuf[2] = 0x00;

			iRet = gp_wtb(cmdbuf);
			if(iRet != 0)
			{
				return iRet;
			}	
			
			for(cnt2 = 0; (cnt2 < 31)&&(cnt3 < (int)(adrsLen-6)); cnt2++)
			{
				cmdbuf[0] = (char)(SAG + cnt2);
				cmdbuf[1] = 0x00;
				iRet = gp_wtb(cmdbuf);
				if(iRet != 0)
				{
					return iRet;
				}
				
				sleep(2/1000);

				OutPort(AUXCR, ton);
				wRet = InPort(BSR);
				if(wRet & 0x20)
				{
					if(cnt3 + 3 > (int)adrsLen)
						return 64;

					if(cnt3 != 0)
					{
						adrs[cnt3++] = 0x2C;
					}
					if(cnt < 10) // -9
					{
						adrs[cnt3++] = (char)(0x30 + cnt);
					}
					else if((cnt >= 10)&&(cnt < 20)) // 10-19
					{
						adrs[cnt3++] = (char)(0x31);
						adrs[cnt3++] = (char)(0x30 + (cnt-10));
					}
					else if((cnt >= 20)&&(cnt < 30)) // 20-29
					{
						adrs[cnt3++] = (char)(0x32);
						adrs[cnt3++] = (char)(0x30 + (cnt-20));
					}
					else // 30
					{
						adrs[cnt3++] = (char)(0x33);
						adrs[cnt3++] = (char)(0x30 + (cnt-30));
					}
					if(cnt3 + 4 > (int)adrsLen)
						return 64;

					adrs[cnt3++] = 0x2C;
					if(SAG+cnt2 < 100) // 96-99(SAG+address)
					{
						adrs[cnt3++] = (char)(0x39);
						adrs[cnt3++] = (char)(0x30 + (SAG+cnt2-90));
					}
					else if((SAG+cnt2 >= 100)&&(SAG+cnt2 < 110)) // 100-109
					{
						adrs[cnt3++] = (char)(0x31);
						adrs[cnt3++] = (char)(0x30);
						adrs[cnt3++] = (char)(0x30 + (SAG+cnt2-100));
					}
					else if((SAG+cnt2 >= 110)&&(SAG+cnt2 < 120)) // 110-119
					{
						adrs[cnt3++] = (char)(0x31);
						adrs[cnt3++] = (char)(0x31);
						adrs[cnt3++] = (char)(0x30 + (SAG+cnt2-110));
					}
					else // 120-126
					{
						adrs[cnt3++] = (char)(0x31);
						adrs[cnt3++] = (char)(0x32);
						adrs[cnt3++] = (char)(0x30 + (SAG+cnt2-120));
					}

					num++;

					cmdbuf[0] = UNL;
					cmdbuf[1] = (char)(LAG + cnt);
					cmdbuf[2] = 0x00;
					iRet = gp_wtb(cmdbuf);
					if(iRet != 0)
					{
						return iRet;
					}

				}
			} // End of for(cnt2 = 0; cnt2 < 31; cnt2++)
		}
	} // End of for(cnt = 0; cnt < 31; cnt++)

	cmdbuf[0] = UNL;
	cmdbuf[1] = 0x00;
	iRet = gp_wtb(cmdbuf);
	if(iRet != 0)
	{
		return iRet;
	}

	if(cnt3 >= (int)adrsLen)
		iRet = 64;
	else
	{
		// 正常終了時、最終データの後にNULLを付加しておく
		adrs[cnt3++] = 0x00;
		return (int)num;
	}

	return iRet;
}

#if 0 // デバイスの複数台接続時に使用します。

/*****************************************************************************/
/*               U2GpOpenUnit                                                */
/*****************************************************************************/
/*  解説   :指定のIDのユニットをオープンします。                             */
/*          プログラム終了時、U2GpCloseUnitによりユニットをクローズします。  */
/*  引数   :UnitId :REX-PCI20のUnitID                                        */
/*  戻り値 :正常にオープンした場合はファイルデスクリプタハンドルが返されます */
/*          エラーの場合は0を返します                                        */
/*****************************************************************************/
int U2GpOpenUnit(unsigned short UnitId)
{
	int		hDriver;
	int		retVal = 0;
	char	pathName[16];
	unsigned short instance;
	unsigned char GetUnitId;
	int		Status;
	unsigned short MyUnitId;

	if(hSys_one>0)
	{
		MyUnitId = g_MyUnitId;
		OpenCtl[MyUnitId].OpenMark = 0;
		close(hSys_one);
		hSys_one = 0;
	}

	for(instance = 0; instance < MAX_DEVICES; instance++)
	{
		memset(pathName, 0x00, sizeof(pathName));
		sprintf(pathName, "/dev/gpib%d", instance);
		hDriver = open(pathName,O_RDWR);

		if(hDriver<0)
			continue;
		else
			OpenCtl[UnitId].hDrv = hDriver;

#ifdef _DEBUG
		printf("PCI20_LIB_MASTER: U2GpOpenUnit: Unit open Instance = %d\n", instance);
#endif
		Status = gp_GetDeviceID(hDriver, UnitId, (char*)&GetUnitId);
		if(Status == 0xFF)
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: U2GpOpenUnit:gp_Get_DeviceId error\n");
#endif
			close(hDriver);
			retVal = -2;
			goto OPEN_UNIT_EXIT;
		}
		//if(GetUnitId == UnitId)
		if(UnitId < MAX_DEVICES)
		{
			OpenCtl[UnitId].UnitId = UnitId;
			OpenCtl[UnitId].hDrv = hDriver;
			OpenCtl[UnitId].OpenMark = 1;
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: U2GpOpenUnit: Driver Handle = 0x%x\n", OpenCtl[UnitId].hDrv);
#endif
			return hDriver;
		}
		else
		{
			;
		}
	}
	retVal = -3;

OPEN_UNIT_EXIT:
	return 0;
}

/*****************************************************************************/
/*               U2GpCloseUnit                                               */
/*****************************************************************************/
/*  解説   :ユニットをクローズします。                                       */
/*  引数   :hDriver :オープン時に取得したハンドル                            */
/*  戻り値 : 0 :正常終了                                                     */
/*          -1 :システムコールエラー                                         */
/*****************************************************************************/
int U2GpCloseUnit(int hDriver)
{
	int	iRet;
	unsigned short UnitNo;

	for(UnitNo = 0; UnitNo < MAX_DEVICES; UnitNo++)
	{
		if((OpenCtl[UnitNo].OpenMark == 1)&&(hDriver == OpenCtl[UnitNo].hDrv))
		{
			iRet = close(hDriver);
			OpenCtl[UnitNo].UnitId = 0;
			OpenCtl[UnitNo].hDrv = 0;
			OpenCtl[UnitNo].OpenMark = 0;
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: U2GpCloseUnit: UnitId %d Closed.\n", UnitNo);
#endif
			return iRet;
		}
	}
#ifdef _DEBUG
	printf("PCI20_LIB_MASTER: U2GpCloseUnit:did not do at all\n");
#endif
	iRet = -1;
	return iRet;
}

/*****************************************************************************/
/*               U2GpEnumUnit                                                */
/*****************************************************************************/
/*  解説   :接続されているユニットを列挙します。                             */
/*          全てのユニットがクローズされている状態で呼びだしてください。     */
/*          本関数は、複数台のGPIBユニットを同一バスに接続して               */
/*          利用する場合に必要になります。                                   */
/*          各ユニットはそれぞれ異なるIDを設定しておく必要があります。       */
/*          ユニットIDは0〜255の範囲で設定できます。                         */
/*  引数   :pUnitId :REX-PCI20のUnitIDの格納先アドレス                       */
/*         :MaxUnit :列挙する最大ユニット数                                  */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :システムコールエラー                                         */
/*         :-2 :ユニットID重複エラー                                         */
/*            この時呼び出し元のpUnitId[0]には重複ユニットIDがセットされます */
/*****************************************************************************/
int U2GpEnumUnit(unsigned short *pUnitId, unsigned short MaxUnit)
{
	int		hDriver;
	char	pathName[16];
	unsigned short Instance;
	unsigned short ScanNo;
	unsigned char GetUnitId;
	int		AttachedUnitNum;
	ENUM_UNIT	*EnumUnit;
	unsigned short MyUnitId;

	EnumUnit = (ENUM_UNIT *)malloc(sizeof(ENUM_UNIT)*MaxUnit);

	if(hSys_one>0)
	{
		MyUnitId = g_MyUnitId;
		OpenCtl[MyUnitId].OpenMark = 0;
		close(hSys_one);
		hSys_one = 0;
	}

	for(Instance = 0; Instance < MaxUnit; Instance++)
	{
		memset(pathName, 0x00, sizeof(pathName));
		sprintf(pathName, "/dev/gpib%d", Instance);
		hDriver = open(pathName,O_RDWR);

		if(hDriver>0)
		{
			if(gp_GetDeviceID(hDriver, 0, (char*)&GetUnitId) == 0xFF) // error
			{
				close(hDriver);
				free(EnumUnit);
				return -1;
			}
			else
			{
#ifdef _DEBUG
				printf("PCI20_LIB_MASTER: U2GpEnumUnit: Instance(%d) opened / unit id = 0x%02x\n", Instance, GetUnitId);
#endif
				EnumUnit[Instance].Attached	= 1;
				EnumUnit[Instance].UnitId = GetUnitId;
				close(hDriver);
			}
		}
		else
		{
			EnumUnit[Instance].Attached	= 0;
			EnumUnit[Instance].UnitId = 0;
		}
	} // EnumUnit[Instance] filled

    for(Instance = 0; Instance < MaxUnit; Instance++)
	{
		if(EnumUnit[Instance].Attached == 1)
		{
			for(ScanNo = 0; ScanNo < MaxUnit; ScanNo++)
			{
				if(EnumUnit[ScanNo].Attached == 1 && ScanNo != Instance)
				{
					if(EnumUnit[ScanNo].UnitId == EnumUnit[Instance].UnitId)
					{
						// ID repetition error
						pUnitId[0] = EnumUnit[ScanNo].UnitId;
						free(EnumUnit);
						return -2;
					}
				}
			}
		}
	}

	// 指定のMaxUnit台数分呼び出し元のpUnitIdバッファにユニットIDをセット
	AttachedUnitNum = 0;
    for(Instance = 0; Instance < MaxUnit; Instance++)
	{
		if(EnumUnit[Instance].Attached == 1)
		{
			pUnitId[AttachedUnitNum] = EnumUnit[Instance].UnitId;
			AttachedUnitNum++;
		}
	}

	free(EnumUnit);
	return AttachedUnitNum;
}

void GetUnitIdFromhDrv(int hDriver)
{
	unsigned short	cnt;

	for(cnt = 0; cnt < MAX_DEVICES; cnt++)
	{
		if(OpenCtl[cnt].hDrv == hDriver)
		{
			g_MyUnitId = OpenCtl[cnt].UnitId;
			break;
		}
	}
}

unsigned short U2OutPort(int hDriver, unsigned short Reg, unsigned short bOutVal)
{
	hSys = hDriver;
	return OutPort(Reg, bOutVal);
}
unsigned short U2InPort(int hDriver, unsigned short Reg)
{
	hSys = hDriver;
	return InPort(Reg);
}
int U2Gpinit(int hDriver, unsigned short GpAdrs)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_init(GpAdrs, 0, 0);
}
int U2Gpcli(int hDriver)
{
	hSys = hDriver;
	return gp_cli();
}
int U2Gpren(int hDriver)
{
	hSys = hDriver;
	return gp_ren();
}
int U2Gpclr(int hDriver, char *adrs)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_clr(adrs);
}
int U2Gpmyadr(int hDriver)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_myadr();
}
int U2Gpwrt(int hDriver, char *adrs, char *buf)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_wrt(adrs, buf);
}
int U2Gptrg(int hDriver, char *adrs)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_trg(adrs);
}
int U2Gpwsrq(int hDriver, int WaitSecTime)
{
	hSys = hDriver;
	return gp_wsrq(WaitSecTime);
}
int U2Gpwsrqb(int hDriver, int WaitSecTime)
{
	hSys = hDriver;
	return gp_wsrqb(WaitSecTime);
}
int U2Gprds(int hDriver, char *adrs, unsigned char *status_byte)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_rds(adrs, status_byte);
}
int U2Gprds1(int hDriver, char *adrs, unsigned char *status_byte)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_rds1(adrs, status_byte);;
}
int U2Gpred(int hDriver, char *adrs, char *buf, int bufLen)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_red(adrs, buf, bufLen);
}
int U2Gpredd(int hDriver, char *buf, int bufLen)
{
	hSys = hDriver;
	return gp_redd(buf, bufLen);
}
int U2Gpredah(int hDriver, char *adrs, char *buf, int bufLen)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_redah(adrs, buf, bufLen);
}
int U2Gpredrst(int hDriver)
{
	hSys = hDriver;
	return gp_redrst();
}
int U2Gpsrq(int hDriver, int SrqMode, PAPIFUNC pFunc)
{
	hSys = hDriver;
	return gp_srq(SrqMode, pFunc);
}
int U2Gplcl(int hDriver, char *adrs)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_lcl(adrs);
}
int U2Gpllo(int hDriver)
{
	hSys = hDriver;
	return gp_llo();
}
int U2Gptmout(int hDriver, int SecTime)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_tmout(SecTime);
}
int U2Gpsetdelay(int hDriver, int DelayTime)
{
	hSys = hDriver;
	return gp_setdelay(DelayTime);
}
int U2Gpcount(int hDriver)
{
	hSys = hDriver;
	return gp_count();
}
int U2Gpdelm(int hDriver, char *mode, unsigned int dlm)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_delm(mode, dlm);
}
int U2Gptfrout(int hDriver, char *adrs, int bufLen, char *buf)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_tfrout(adrs, bufLen, buf);
}
int U2Gptfrout_unt(int hDriver, char *adrs, int bufLen, char *buf)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_tfrout_unt(adrs, bufLen, buf);
}
int U2Gptfrin(int hDriver, char *adrs, int bufLen, char *buf)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_tfrin(adrs, bufLen, buf);
}
int U2Gptfrinit(int hDriver, char *adrs)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_tfrinit(adrs);
}
int U2Gptfrins(int hDriver, int bufLen, char *buf)
{
	hSys = hDriver;
	return gp_tfrins(bufLen, buf);
}
void U2Gptfrend(int hDriver)
{
	hSys = hDriver;
	gp_tfrend();
}
int U2Gpwtb(int hDriver, char *buf)
{
	hSys = hDriver;
	return gp_wtb(buf);
}

int U2GpsrqCheck(int hDriver)
{
	hSys = hDriver;
	return gp_srqCheck();
}
int U2Gpwrtd(int hDriver, char *buf, int bufLen)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_wrtd(buf, bufLen);
}
int U2Gptfroutd(int hDriver, int bufLen, char *buf)
{
	GetUnitIdFromhDrv(hDriver);
	hSys = hDriver;
	return gp_tfroutd(bufLen, buf);
}
int U2Gpfindlstn(int hDriver, char *adrs, int adrsLen)
{
	hSys = hDriver;
	return gp_findlstn(adrs, adrsLen);
}

#endif // #if 0


/*****************************************************************************/
/*               StartInterruptThread                                        */
/*****************************************************************************/
/*  解説   :割り込みイベント監視スレッド                                     */
/*  引数   :arg :コールバック関数用のポインタ                                */
/*  戻り値 : 0 :正常終了                                                     */
/*****************************************************************************/
void *StartInterruptThread(void *arg)
{
	int			iRet;
	unsigned char bData;
	PAPIFUNC pFunc;

#ifdef _DEBUG
	printf("PCI20_LIB_MASTER: ***** Start interrupt thread.\n");
#endif
	fInterruptComplete = 0;

	iRet=ioctl(hSys, IOCTL_GP_START_INTERRUPT, &bData);
	if(iRet<0)
	{
#ifdef _DEBUG
		printf("PCI20_LIB_MASTER: iRet<0 \n");
#endif
	}
	else
	{
		if(fStartEvent)
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: ***** interrupt thread success exit. : %x.\n", bData);
#endif
			fStartEvent = 0;
			// レジスタを読まないと割り込みビットが消えないためレジスタ読み出しを行う
			InPort(ISR1);
			// コールバック関数の呼び出し
			pFunc = (PAPIFUNC)arg;
			(*pFunc)();	//Cmd_OnEventOccur();
		}
		else
		{
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: ***** interrupt aborted : %x.\n", bData);
#endif
		}
	}

	fInterruptComplete = 1;

#ifdef _DEBUG
	printf("PCI20_LIB_MASTER: ***** Exit interrupt thread.\n");
#endif

	// スレッドの終了待ち
	pthread_join(thread_id, NULL);

    return 0;
}

/*****************************************************************************/
/*               gp_srq                                                      */
/*****************************************************************************/
/*  解説   :シリアルポールハードウエア割り込みの実行および解除               */
/*  引数   :SrqMode :モードフラグ(0:解除、1:登録)                            */
/*         :pFunc   :割り込み完了後に呼びだす関数へのポインタ                */
/*  戻り値 : 0 :正常終了                                                     */
/*         :-1 :システムコールエラー                                         */
/*         :71 :既にインタラプト転送は開始している                           */
/*****************************************************************************/
int gp_srq(int SrqMode, PAPIFUNC pFunc)
{
	int				iRet;

	// 既にインタラプト転送が開始している場合
	if(fStartEvent)
	{
		if(SrqMode)
		{ 
			return 71;
		}
		fStartEvent = 0;
		iRet=ioctl(hSys, IOCTL_GP_STOP_INTERRUPT, 0);
		// インタラプト転送完了されるまで待つ
		while(!fInterruptComplete){
			sleep(1/1000);
		}
	}

	// 080808 - 割込み処理開始時にマスク処理を行う
	if(SrqMode){
		iRet = OutPort(IMR0, 0x00);
		iRet = OutPort(IMR1, 0x02);
		iRet = OutPort(AUXCR, piimr2);
		iRet = OutPort(IMR2, 0xC0);
		iRet = OutPort(AUXCR, clrpi);
	}else{
		iRet = OutPort(IMR0, 0x00);
		iRet = OutPort(IMR1, 0x00);
		iRet = OutPort(AUXCR, piimr2);
		iRet = OutPort(IMR2, 0x00);
		iRet = OutPort(AUXCR, clrpi);
	}

	if(iRet<0)
	{
    		iRet = -1;
		return iRet;
	}
	else
	{
		iRet = 0;
	}


	if(SrqMode)
	{
		// 割り込みイベント監視スレッドの作成
		iRet=pthread_create(&thread_id,NULL,StartInterruptThread,(void *)pFunc);
		if(iRet!=0){
#ifdef _DEBUG
			fprintf(stderr,"PCI20_LIB_MASTER: pthread_create ERROR : %s",strerror(iRet));
#endif
		}
		else{
			fStartEvent = 1;
#ifdef _DEBUG
			printf("PCI20_LIB_MASTER: pthread_create success.. thread_id=%d\n",(int)thread_id);
#endif
		}
	}
	else
		; /* 何もしない */

	return iRet;
}



