/*
 *******************************************************************************
 *
 *  GPIB機器 制御プログラム
 *
 *  Compiler : gcc
 *
 *  (C) 2008 RATOC Systems, Inc.    Date : 2008.08.06
 *
 *  History : 
 *    2008.08.06    新規作成 (Windows版のソースを基にLinuxへ移植)
 *
 *  Feature : GPIB機器のアドレス取得、コマンドの送信およびデータの受信。
 *            コマンドデータ送受信時のデリミタ設定の変更。
 *            接続される機器のGPIBアドレス、デリミタ設定が不明の場合には
 *            本プログラムで確認してください。
 *
 *******************************************************************************
 */

//gcc -g gpibctrl.c -o gpibctrl -L/usr/local/lib/ -lrexpci20_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "rexpci20_gpib.h"

//#define _USE_U2
#define MAX_DEVICES				256

int fd=0;
short MyGpibAdrs = 0;		// PCI20のGPIBアドレス
char GpAdrsBuf[12];			// 使用するGPIB機器のGPIBアドレス
char szBuf[256];			// ワークバッファ
char szCmd[20];				// コマンド送信用バッファ

int InitFlag;				// 初期化フラグ(初期値:0, 完了時:1)
int g_tmout;				// GPIBタイムアウト時間
unsigned short g_delm;			// トーカ、リスナ時のデリミタ
int g_filelen;				// ファイルサイズ

const gchar *selected_filename;		// ファイル名

GtkWidget *entry_adrs;
GtkWidget *entry_tmout;
GtkWidget *check_button;
GtkWidget *radio_button;
GtkWidget *combo_box;

GtkWidget *fndlsn_button;
GtkWidget *default_button;
GtkWidget *setting_button;
GtkWidget *txquery_button;
GtkWidget *txcmd_button;
GtkWidget *rxdata_button;
GtkWidget *filesel_button;
GtkWidget *txbin_button;
GtkWidget *rxbin_button;
GtkWidget *serpoll_button;

GtkWidget *cmdtxlen_bar;
gint cmdtxlen_id;
GtkWidget *cmdrx_bar;
gint cmdrx_id;
GtkWidget *cmdrxlen_bar;
gint cmdrxlen_id;
GtkWidget *binfile_bar;
gint binfile_id;
GtkWidget *bintxlen_bar;
gint bintxlen_id;
GtkWidget *binrxlen_bar;
gint binrxlen_id;
GtkWidget *status_bar;
gint status_id;

/*
 *******************************************************************************
 *  関数名 : quick_message
 *  機  能 : メッセージを表示するダイアログ・ボックスを開く関数
 *  引  数 : message - メッセージ
 *  戻り値 : なし
 *******************************************************************************
 */

/* メッセージを表示するダイアログ・ボックスを開く関数 */
void quick_message(gchar *message) {

   GtkWidget *dialog, *label, *okay_button;
   
   /* ウィジットの生成 */
   
   dialog = gtk_dialog_new();
   label = gtk_label_new (message);
   okay_button = gtk_button_new_with_label("了解");
   
   /* ユーザが "了解" ボタンをクリックしたら、ダイアログ・ボックスを確実に destroy する */
   gtk_signal_connect_object (GTK_OBJECT (okay_button), "clicked",
                              GTK_SIGNAL_FUNC (gtk_widget_destroy), dialog);
   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
                      okay_button);

   /* ラベルを追加し、ダイアログに追加した全てを表示する。 */

   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
                      label);
   gtk_widget_show_all (dialog);
}

/*
 *******************************************************************************
 *  関数名 : store_filename
 *  機  能 : ファイル名を格納する
 *  引  数 : selector  - 
 *           user_data - 
 *  戻り値 : なし
 *******************************************************************************
 */
void store_filename(GtkFileSelection *selector, gpointer user_data) {
   GtkWidget *file_selector = GTK_WIDGET (user_data);

   selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector));
   gtk_statusbar_push (GTK_STATUSBAR (binfile_bar), GPOINTER_TO_INT (binfile_id), selected_filename);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdSelectFile
 *  機  能 : ファイル選択ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdSelectFile( GtkWidget *widget,
                      gpointer   data )
{
   GtkWidget *file_selector;

   /* セレクタを生成する */
   
   file_selector = gtk_file_selection_new("Please select a file.");
   
   gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
   			   "clicked", GTK_SIGNAL_FUNC (store_filename), file_selector);
   			   
   /* ユーザがボタンをクリックしたら、ダイアログ・ボックスを destroy するようにする。*/
   
   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
   					  "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
   					  (gpointer) file_selector);

   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
   					  "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
   					  (gpointer) file_selector);
   
   /* ダイアログを表示する */
   
   gtk_widget_show (file_selector);
}

/*
 *******************************************************************************
 *  関数名 : SaveDataFile
 *  機  能 : 受信データをファイル保存に保存する
 *  引  数 : buf    - データ格納バッファ
 *           bufLen - データ格納バッファサイズ
 *  戻り値 : ファイルデータ書込みサイズ、0のときはファイルデータ書込み失敗
 *******************************************************************************
 */
static int SaveDataFile(const void* buf, int bufLen)
{
	FILE *fp;	/* (1)ファイルポインタの宣言 */

	/* (2)ファイルのオープン */
	/*  ここで、ファイルポインタを取得する */
	if ((fp = fopen(selected_filename, "wb")) == NULL) {
		printf("file open error!!\n");
		return 0;
		//exit(1);	/* (3)エラーの場合は通常、異常終了する */
	}
	
	/* (4)ファイルの書き込み */
	bufLen = fwrite(buf, sizeof(char), bufLen, fp);
	fclose(fp);	/* (5)ファイルのクローズ */
	printf("fwrite = %s, size = %d\n", selected_filename, bufLen);
	
	return bufLen;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdRxBinary
 *  機  能 : ファイル受信ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdRxBinary( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;
	char*	RxBuf;
	int		RxLen;

	if(InitFlag == 0){
		quick_message("test messsage");
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "受信中");
	gtk_statusbar_push (GTK_STATUSBAR (binrxlen_bar), GPOINTER_TO_INT (binrxlen_id), " ");

	RxLen = g_filelen;
	RxBuf = (char *)malloc(RxLen);
	if(RxBuf == NULL)
	{
		printf("LocalAlloc error.\n");
		return;
	}

#ifdef _USE_U2
	GpStatus = U2Gptfrin( fd, GpAdrsBuf, RxLen, RxBuf );
#else
	GpStatus = gp_tfrin( GpAdrsBuf, RxLen, RxBuf );
#endif
	if(GpStatus != 0)
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");
		printf("gp_tfrin() error [%d]\n", GpStatus);
	}
	else
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "受信完了");
	}

	RxLen = gp_count();
	sprintf(szBuf, "%d", RxLen);
	gtk_statusbar_push (GTK_STATUSBAR (binrxlen_bar), GPOINTER_TO_INT (binrxlen_id), szBuf);

	// データ受信していれば、ファイル保存
	if(RxLen != 0)
	{
		SaveDataFile(RxBuf, RxLen);
	}
	else
		;

	free(RxBuf);
}

/*
 *******************************************************************************
 *  関数名 : FileOpenSetMem
 *  機  能 : ファイルからのデータ読み込み
 *  引  数 : buf    - データ格納バッファ
 *           bufLen - データ格納バッファサイズ
 *  戻り値 : ファイルデータ読み込みサイズ、0のときはファイルデータ読み込み失敗
 *******************************************************************************
 */
static int FileOpenSetMem(void* buf, int bufLen)
{
	FILE *fp;	/* (1)ファイルポインタの宣言 */

	/* (2)ファイルのオープン */
	/*  ここで、ファイルポインタを取得する */
	if ((fp = fopen(selected_filename, "rb")) == NULL) {
		printf("file open error!!\n");
		return 0;
		//exit(1);	/* (3)エラーの場合は通常、異常終了する */
	}
	
	/* (4)ファイルの読み込み */
	bufLen = fread(buf, sizeof(char), bufLen, fp);
	fclose(fp);	/* (5)ファイルのクローズ */
	
	printf("fread = %s, size = %d\n", selected_filename, bufLen);
	return bufLen;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdTxBinary
 *  機  能 : ファイル送信ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdTxBinary( GtkWidget *widget,
                      gpointer   data )
{
	int	GpStatus;
	char* TxBuf;
	int TxLen;

	if(InitFlag == 0){
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "送信中");
	gtk_statusbar_push (GTK_STATUSBAR (bintxlen_bar), GPOINTER_TO_INT (bintxlen_id), " ");

	TxLen = g_filelen;
	TxBuf = (char *)malloc(TxLen);
	if(TxBuf == NULL)
	{
		printf("LocalAlloc error.\n");
		return;
	}

	// ファイルからデータ読み込み
	TxLen = FileOpenSetMem(TxBuf, TxLen);
	if(TxLen == 0) // ファイルデータ読み込み失敗
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");
		free(TxBuf);
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "ファイルデータ送信中・・・");
#ifdef _USE_U2
	GpStatus = U2Gptfrout( fd, GpAdrsBuf, TxLen, TxBuf );
#else
	GpStatus = gp_tfrout(GpAdrsBuf, TxLen, TxBuf);
#endif
	if(GpStatus != 0)
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");
		printf("gp_tfrout() error [%d]\n", GpStatus);
	}
	else
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "送信完了");
	}

	TxLen = gp_count();
	sprintf(szBuf, "%d", TxLen);
	gtk_statusbar_push (GTK_STATUSBAR (bintxlen_bar), GPOINTER_TO_INT (bintxlen_id), szBuf);

	free(TxBuf);
}

/*
 *******************************************************************************
 *  関数名 : CmdRxData
 *  機  能 : データの受信を行う
 *  引  数 : なし
 *  戻り値 : なし
 *******************************************************************************
 */
static void CmdRxData()
{
	int		GpStatus;
	char	RxBuf[1024];
	int		RxLen;

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "受信中・・・");

	RxLen = sizeof(RxBuf);
	memset(RxBuf, 0x00, RxLen);
#ifdef _USE_U2
	GpStatus = U2Gpred( fd, GpAdrsBuf, RxBuf, RxLen );
#else
	GpStatus = gp_red(GpAdrsBuf, RxBuf, RxLen);
#endif
	if(GpStatus != 0)
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");
		printf("gp_red() error [%d]\n", GpStatus);
	}
	else
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "受信完了");
		// 受信データ表示
		gtk_statusbar_push (GTK_STATUSBAR (cmdrx_bar), GPOINTER_TO_INT (cmdrx_id), RxBuf);
	}

	RxLen = gp_count();
	sprintf(szBuf, "%d", RxLen);
	gtk_statusbar_push (GTK_STATUSBAR (cmdrxlen_bar), GPOINTER_TO_INT (cmdrxlen_id), szBuf);
}

/*
 *******************************************************************************
 *  関数名 : CmdTxCommand
 *  機  能 : コマンドの送信を行う
 *  引  数 : なし
 *  戻り値 : なし
 *******************************************************************************
 */
static int CmdTxCommand()
{
	int	GpStatus;
	char TxBuf[1024];
	int TxLen;
	gchar *buff;
	int	ret;

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "送信中・・・");

	memset(TxBuf, 0x00, sizeof(TxBuf));
	if(szCmd[0] == 0){
		buff = gtk_combo_box_get_active_text (GTK_COMBO_BOX (combo_box) );
		sprintf(TxBuf, "%s", buff);
	}else{
		sprintf(TxBuf, "%s", szCmd);
	}

	printf("CmdTxCommand [%s]\n", TxBuf);
#ifdef _USE_U2
	GpStatus = U2Gpwrt( fd, GpAdrsBuf, TxBuf );
#else
	GpStatus = gp_wrt(GpAdrsBuf, TxBuf);
#endif
	if(GpStatus != 0)
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");
		printf("gp_wrt() error [%d]\n", GpStatus);
		ret = 1;
	}
	else
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "送信完了");
		ret = 0;
	}
	TxLen = gp_count();
	sprintf(szBuf, "%d", TxLen);
	gtk_statusbar_push (GTK_STATUSBAR (cmdtxlen_bar), GPOINTER_TO_INT (cmdtxlen_id), szBuf);

	return ret;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdTxQuery
 *  機  能 : クエリボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdTxQuery( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	// クエリ送信
	if(CmdTxCommand())
		return;	// 送信エラーなら受信しないで終了
	// 応答データ受信
	CmdRxData();
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdTxCommand
 *  機  能 : 送信ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdTxCommand( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	CmdTxCommand();
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdRxData
 *  機  能 : 受信ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdRxData( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	CmdRxData();
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdGpInit
 *  機  能 : GPIB初期化ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice
	if(GpAdrsBuf[0] != 0){
#ifdef _USE_U2
		GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
		GpStatus = gp_clr( GpAdrsBuf );
#endif
	}else{
#ifdef _USE_U2
		GpStatus = U2Gpclr(fd, "");
#else
		GpStatus = gp_clr( "" );
#endif
	}
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	// 各種ボタンを有効にする
	gtk_widget_set_sensitive(fndlsn_button, TRUE);
	gtk_widget_set_sensitive(default_button, TRUE);
	gtk_widget_set_sensitive(setting_button, TRUE);
	gtk_widget_set_sensitive(txquery_button, TRUE);
	gtk_widget_set_sensitive(txcmd_button, TRUE);
	gtk_widget_set_sensitive(rxdata_button, TRUE);
	gtk_widget_set_sensitive(filesel_button, TRUE);
	gtk_widget_set_sensitive(txbin_button, TRUE);
	gtk_widget_set_sensitive(rxbin_button, TRUE);
	gtk_widget_set_sensitive(serpoll_button, TRUE);

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "初期化正常終了");
	InitFlag = 1;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdSerPoll
 *  機  能 : シリアルポールボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdSerPoll( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;
	unsigned char	StatusByte[16];

	if(InitFlag == 0){
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "シリアルポール中・・・");
	memset(StatusByte, 0x00, sizeof(StatusByte));

#ifdef _USE_U2
	GpStatus = U2GpsrqCheck(fd);
#else
	GpStatus = gp_srqCheck();
#endif
	if(GpStatus == 1)
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "SRQを受信中です。");

	// シリアルポールの実行
#ifdef _USE_U2
	GpStatus = U2Gprds(fd, GpAdrsBuf, StatusByte);
#else
	GpStatus = gp_rds(GpAdrsBuf, StatusByte);
#endif
	if(GpStatus != 0)
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");
		printf("gp_rds() error [%d]\n", GpStatus);
		return;
	}
	else
	{
		sprintf(szBuf, "ステータスバイト[%02X]Hを受信しました。", StatusByte[0]);
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), szBuf);
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCancel
 *  機  能 : 終了ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	int	GpStatus;

	if(InitFlag == 0){
		return;
	}

	//send LCL command
#ifdef _USE_U2
	GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
	GpStatus = gp_lcl( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_lcl() error [%d]\n", GpStatus);
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdFindLstn
 *  機  能 : アドレス検出ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdFindLstn( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), "リスナ検索中・・・");
	gtk_entry_set_text (GTK_ENTRY (entry_adrs), " ");
	memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));

#ifdef _USE_U2
	GpStatus = U2Gpfindlstn(fd, GpAdrsBuf, sizeof(GpAdrsBuf));
#else
	GpStatus = gp_findlstn(GpAdrsBuf, sizeof(GpAdrsBuf));
#endif
	if(GpStatus == 0)
	{
		sprintf(szBuf, "リスナを検出できませんでした。");
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), szBuf);
	}
	else if(GpStatus < 0 || GpStatus == 53 || GpStatus == 64)
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), " ");
		printf("gp_findlstn() error [%d]\n", GpStatus);
	}
	else // アドレス検出
	{
		sprintf(szBuf, "リスナ[%d]台 検出しました。GpAdrsBuf:%s", GpStatus, GpAdrsBuf);
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), szBuf);
		// これは何故かうまくいかない
		//gtk_entry_set_text (GTK_ENTRY (entry_adrs), GpAdrsBuf);
		sprintf(szBuf, "%s", GpAdrsBuf);
		gtk_entry_set_text (GTK_ENTRY (entry_adrs), szBuf);
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdDefault
 *  機  能 : デフォルトボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdDefault( GtkWidget    *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}

#ifdef _USE_U2
	U2Gptmout(fd, 10);
	U2Gpdelm(fd, "t", 0x00);	// CR+LF
	U2Gpdelm(fd, "l", 0x0a);
#else
	gp_tmout(10);
	gp_delm("t", 0x00);		// CR+LF
	gp_delm("l", 0x0a);
#endif

    gtk_entry_set_text (GTK_ENTRY (entry_tmout), "10");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button), FALSE);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), TRUE);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdSetting
 *  機  能 : 設定ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdSetting( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}

	//printf ("g_tmout = %d, g_delm = 0x%x \n", g_tmout, g_delm);
#ifdef _USE_U2
	U2Gptmout(fd, g_tmout);
	U2Gpdelm(fd, "b", g_delm);
#else
	gp_tmout(g_tmout);
	gp_delm("b", g_delm);
#endif
}

/*
 *******************************************************************************
 *  関数名 : enter_adrs
 *  機  能 : アドレス入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
  //printf ("GpAdrsBuf[0]: %02x\n", GpAdrsBuf[0]);
}

/*
 *******************************************************************************
 *  関数名 : enter_timeout
 *  機  能 : GPIBバスタイムアウト時間入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_timeout( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  g_tmout = atoi( entry_text );
  //printf ("g_tmout contents: %d\n", g_tmout);
}

/*
 *******************************************************************************
 *  関数名 : enter_command
 *  機  能 : 送信コマンド入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_command( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(szCmd, 0x00, sizeof(szCmd));
  sprintf(szCmd, "%s", entry_text);
  //printf ("szCmd[0]: %02x\n", szCmd[0]);
}

/*
 *******************************************************************************
 *  関数名 : enter_size
 *  機  能 : バッファサイズ入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_size( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  g_filelen = atoi( entry_text );
  //printf ("g_filelen contents: %d\n", g_filelen);
}

/*
 *******************************************************************************
 *  関数名 : toggle_check
 *  機  能 : EOIチェックボックスの状態を確認する
 *  引  数 : widget - 
 *           data   - 
 *  戻り値 : なし
 *******************************************************************************
 */
static void toggle_check( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
        /* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
		//printf ("changed button1:\n");
		g_delm |= 0x800;
    } else {
        /* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
		//printf ("changed button2:\n");
		g_delm &= 0xf7ff;
    }
}

/*
 *******************************************************************************
 *  関数名 : toggle_button1
 *  機  能 : ラジオボタン選択時の処理(無し)
 *  引  数 : widget - ラジオボタンのウィジット
 *           data   - 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void toggle_button1( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
		g_delm = 0x400;
    }
}

/*
 *******************************************************************************
 *  関数名 : toggle_button2
 *  機  能 : ラジオボタン選択時の処理(CR)
 *  引  数 : widget - ラジオボタンのウィジット
 *           data   - 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void toggle_button2( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
		g_delm = 0x0d;
    }
}

/*
 *******************************************************************************
 *  関数名 : toggle_button3
 *  機  能 : ラジオボタン選択時の処理(LF)
 *  引  数 : widget - ラジオボタンのウィジット
 *           data   - 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void toggle_button3( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
		g_delm = 0x0a;
    }
}

/*
 *******************************************************************************
 *  関数名 : toggle_button4
 *  機  能 : ラジオボタン選択時の処理(CR+LF)
 *  引  数 : widget - ラジオボタンのウィジット
 *           data   - 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void toggle_button4( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
		g_delm = 0x200;
    }
}

/*
 *******************************************************************************
 *  関数名 : combo_cmdbox
 *  機  能 : 送信コマンドコンボボックスの選択が変更された時の処理
 *  引  数 : widget - コンボボックスのウィジット
 *           data   - 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void combo_cmdbox( GtkWidget *widget,
                      gpointer   data )
{
	gchar *buff;

	buff = gtk_combo_box_get_active_text (GTK_COMBO_BOX (widget) );
	printf ("changed combo_box: %s\n", buff);
}

/*
 *******************************************************************************
 *  関数名 : create_window
 *  機  能 : GTK+ウィンドウを作成する
 *  引  数 : なし
 *  戻り値 : なし
 *******************************************************************************
 */
static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *table;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *temp_bar;
	GtkWidget *button;
	GSList *group;
	gint temp_id;
	char	buf[12];

    /* 新しいウィンドウを作成します */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 500, 540);
    gtk_window_set_title (GTK_WINDOW (window), "GPIBCTRL");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    vbox = gtk_vbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    /* GPIB 機器アドレス */
    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry_adrs = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_adrs), 1);
    g_signal_connect (G_OBJECT (entry_adrs), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry_adrs);
    memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
    gtk_box_pack_start (GTK_BOX (hbox), entry_adrs, FALSE, FALSE, 0);
    gtk_widget_show (entry_adrs);

    /* ボタン"アドレス検出" */
    fndlsn_button = gtk_button_new_with_label ("アドレス検出");
    g_signal_connect (G_OBJECT (fndlsn_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdFindLstn), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), fndlsn_button, TRUE, FALSE, 0);
    gtk_widget_show (fndlsn_button);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    /* REX-PCI20 アドレス */
    label = gtk_label_new ("REX-PCI20 アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    temp_bar = gtk_statusbar_new ();
    temp_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (temp_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (temp_bar), temp_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), temp_bar, TRUE, TRUE, 0);
    gtk_widget_show (temp_bar);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("パラメータ設定");
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    /* 縦横 4x4 のテーブルを生成します。*/
    table = gtk_table_new (8, 8, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table), 5);
    gtk_container_add (GTK_CONTAINER (frame), table);
    gtk_widget_show (table);

    /* GPIBバスタイムアウト時間 */
    label = gtk_label_new ("GPIBバスタイムアウト時間");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 3, 0, 2);
    gtk_widget_show (label);

    entry_tmout = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_tmout), 2);
    g_signal_connect (G_OBJECT (entry_tmout), "changed",
		      G_CALLBACK (enter_timeout),
		      (gpointer) entry_tmout);
    gtk_entry_set_text (GTK_ENTRY (entry_tmout), "10");
    gtk_table_attach_defaults (GTK_TABLE (table), entry_tmout, 3, 4, 0, 2);
    gtk_widget_show (entry_tmout);

    label = gtk_label_new ("秒");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 4, 5, 0, 2);
    gtk_widget_show (label);

    /* デリミタ設定 */
    label = gtk_label_new ("デリミタ設定");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 3, 2, 4);
    gtk_widget_show (label);

    /* チェックボタン"EOI" */
    check_button = gtk_check_button_new_with_label ("EOI");
    gtk_table_attach_defaults (GTK_TABLE (table), check_button, 3, 5, 2, 4);
    g_signal_connect (G_OBJECT (check_button), "clicked",
                      G_CALLBACK (toggle_check), NULL);
    gtk_widget_show (check_button);

    /* ラジオボタン"無し" */
    button = gtk_radio_button_new_with_label (NULL, "無し");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button1),
                      NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 1, 2, 4, 6);
    gtk_widget_show (button);

    /* ラジオボタン"CR" */
    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label (group, "CR");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button2),
                      NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 2, 3, 4, 6);
    gtk_widget_show (button);

    /* ラジオボタン"LF" */
    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
    radio_button = gtk_radio_button_new_with_label (group, "LF");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio_button), TRUE);
    g_signal_connect (G_OBJECT (radio_button), "clicked",
                      G_CALLBACK (toggle_button3),
                      NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), radio_button, 3, 4, 4, 6);
    gtk_widget_show (radio_button);

    /* ラジオボタン"CR+LF" */
    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
    button = gtk_radio_button_new_with_label (group, "CR+LF");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button4),
                      NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 4, 6, 4, 6);
    gtk_widget_show (button);

    /* ボタン"デフォルト" */
    default_button = gtk_button_new_with_label ("デフォルト");
    g_signal_connect (G_OBJECT (default_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdDefault), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), default_button, 4, 6, 6, 8);
    gtk_widget_show (default_button);

    /* ボタン"設定" */
    setting_button = gtk_button_new_with_label ("設定");
    g_signal_connect (G_OBJECT (setting_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdSetting), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), setting_button, 6, 8, 6, 8);
    gtk_widget_show (setting_button);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("コマンド・クエリ送信・受信");
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    /* 縦横 4x4 のテーブルを生成します。*/
    table = gtk_table_new (12, 12, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table), 5);
    gtk_container_add (GTK_CONTAINER (frame), table);
    gtk_widget_show (table);

    /* 送信コマンド入力 */
    label = gtk_label_new ("送信コマンド入力");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 3, 0, 4);
    gtk_widget_show (label);

    /* 送信コマンド入力コンボボックス */
    combo_box = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "*IDN?");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "*OPC?");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "*CLS");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "*RST");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "*TEST?");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "*STB?");
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
    g_signal_connect (G_OBJECT (combo_box), "changed",
                      G_CALLBACK (combo_cmdbox), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), combo_box, 3, 5, 0, 4);
    gtk_widget_show (combo_box);

    /* 送信コマンド入力フィールド */
    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 20);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_command),
		      (gpointer) entry);
    memset(szCmd, 0x00, sizeof(szCmd));
    //gtk_entry_set_text (GTK_ENTRY (entry), " ");
    gtk_table_attach_defaults (GTK_TABLE (table), entry, 5, 9, 0, 4);
    gtk_widget_show (entry);

    /* 送信データバイト数 */
    cmdtxlen_bar = gtk_statusbar_new ();
    cmdtxlen_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (cmdtxlen_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), cmdtxlen_bar, 9, 11, 0, 4);
    gtk_widget_show (cmdtxlen_bar);

    label = gtk_label_new ("Byte");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 11, 12, 0, 4);
    gtk_widget_show (label);

    /* 受信データ内容 */
    label = gtk_label_new ("受信データ");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 3, 4, 8);
    gtk_widget_show (label);

    cmdrx_bar = gtk_statusbar_new ();
    cmdrx_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (cmdrx_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), cmdrx_bar, 3, 9, 4, 8);
    gtk_widget_show (cmdrx_bar);

    /* 受信データバイト数 */
    cmdrxlen_bar = gtk_statusbar_new ();
    cmdrxlen_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (cmdrxlen_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), cmdrxlen_bar, 9, 11, 4, 8);
    gtk_widget_show (cmdrxlen_bar);

    label = gtk_label_new ("Byte");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 11, 12, 4, 8);
    gtk_widget_show (label);

    /* ボタン"クエリ" */
    txquery_button = gtk_button_new_with_label ("クエリ");
    g_signal_connect (G_OBJECT (txquery_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdTxQuery), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), txquery_button, 6, 8, 8, 12);
    gtk_widget_show (txquery_button);

    /* ボタン"送信" */
    txcmd_button = gtk_button_new_with_label ("送信");
    g_signal_connect (G_OBJECT (txcmd_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdTxCommand), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), txcmd_button, 8, 10, 8, 12);
    gtk_widget_show (txcmd_button);

    /* ボタン"受信" */
    rxdata_button = gtk_button_new_with_label ("受信");
    g_signal_connect (G_OBJECT (rxdata_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdRxData), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), rxdata_button, 10, 12, 8, 12);
    gtk_widget_show (rxdata_button);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("バイナリ転送");
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    /* 縦横 8x8 のテーブルを生成します。*/
    table = gtk_table_new (9, 9, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table), 5);
    gtk_container_add (GTK_CONTAINER (frame), table);
    gtk_widget_show (table);

    /* バッファサイズ */
    label = gtk_label_new ("バッファサイズ");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 2, 0, 3);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 8);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_size),
		      (gpointer) entry);
    gtk_entry_set_text (GTK_ENTRY (entry), "4096");
    gtk_table_attach_defaults (GTK_TABLE (table), entry, 2, 3, 0, 3);
    gtk_widget_show (entry);

    label = gtk_label_new ("Byte");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 3, 4, 0, 3);
    gtk_widget_show (label);

    /* ボタン"ファイル選択" */
    filesel_button = gtk_button_new_with_label ("ファイル選択");
    g_signal_connect (G_OBJECT (filesel_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdSelectFile), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), filesel_button, 4, 6, 0, 3);
    gtk_widget_show (filesel_button);

    /* ファイル名表示 */
    binfile_bar = gtk_statusbar_new ();
    binfile_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (binfile_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), binfile_bar, 6, 9, 0, 3);
    gtk_widget_show (binfile_bar);

    /* ボタン"ファイル送信" */
    txbin_button = gtk_button_new_with_label ("ファイル送信");
    g_signal_connect (G_OBJECT (txbin_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdTxBinary), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), txbin_button, 4, 6, 3, 6);
    gtk_widget_show (txbin_button);

    /* ファイル送信バイト数 */
    bintxlen_bar = gtk_statusbar_new ();
    bintxlen_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (bintxlen_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), bintxlen_bar, 6, 8, 3, 6);
    gtk_widget_show (bintxlen_bar);

    label = gtk_label_new ("Byte");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 8, 9, 3, 6);
    gtk_widget_show (label);

    /* ボタン"ファイル受信" */
    rxbin_button = gtk_button_new_with_label ("ファイル受信");
    g_signal_connect (G_OBJECT (rxbin_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdRxBinary), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), rxbin_button, 4, 6, 6, 9);
    gtk_widget_show (rxbin_button);

    /* ファイル受信バイト数 */
    binrxlen_bar = gtk_statusbar_new ();
    binrxlen_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (binrxlen_bar), "Statusbar example");
    gtk_table_attach_defaults (GTK_TABLE (table), binrxlen_bar, 6, 8, 6, 9);
    gtk_widget_show (binrxlen_bar);

    label = gtk_label_new ("Byte");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 8, 9, 6, 9);
    gtk_widget_show (label);

    /* ステータスバー */
    status_bar = gtk_statusbar_new ();
    status_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    gtk_box_pack_start (GTK_BOX (vbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    /* ボタン"GPIB初期化" */
	button = gtk_button_new_with_label ("GPIB初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

    /* ボタン"シリアルポール" */
	serpoll_button = gtk_button_new_with_label ("シリアルポール");
	g_signal_connect (G_OBJECT (serpoll_button), "clicked",
                      G_CALLBACK (Cmd_OnCmdSerPoll), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), serpoll_button, TRUE, TRUE, 0);
	gtk_widget_show (serpoll_button);

    /* ボタン"終了" */
	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	// ウィンドウの表示
	gtk_widget_show (window);
}

/*
 *******************************************************************************
 *  関数名 : main
 *  機  能 : メインルーチン
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_DEVICES];
	unsigned short MaxUnit = MAX_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたREX-PCI20をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつREX-PCI20をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	g_tmout = 10;
	g_delm = 0x0a; // LF
	//gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	// GPIB機器初期化などの準備ができるまでボタンを無効にする
	gtk_widget_set_sensitive(fndlsn_button, FALSE);
	gtk_widget_set_sensitive(default_button, FALSE);
	gtk_widget_set_sensitive(setting_button, FALSE);
	gtk_widget_set_sensitive(txquery_button, FALSE);
	gtk_widget_set_sensitive(txcmd_button, FALSE);
	gtk_widget_set_sensitive(rxdata_button, FALSE);
	gtk_widget_set_sensitive(filesel_button, FALSE);
	gtk_widget_set_sensitive(txbin_button, FALSE);
	gtk_widget_set_sensitive(rxbin_button, FALSE);
	gtk_widget_set_sensitive(serpoll_button, FALSE);
	
	gtk_main ();

	gp_detach(fd);

	return 0;
}
