/*
 *******************************************************************************
 *
 *  FUNCTION GENERATOR HP33120A 制御プログラム
 *
 *  Compiler : gcc
 *
 *  (C) 2008 RATOC Systems, Inc.    Date : 2008.07.17
 *
 *  History : 
 *    2008.07.17    新規作成 (Windows版のソースを基にLinuxへ移植)
 *
 *******************************************************************************
 */

//gcc -g hp33120.c -o hp33120 -L/usr/local/lib/ -lrexpci20_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "rexpci20_gpib.h"

//#define _USE_U2
#define MAX_DEVICES				16

int fd=0;
short MyGpibAdrs = 1;	// PCI20のGPIBアドレス
char GpAdrsBuf[12];		// 使用するGPIB機器のGPIBアドレス
int InitFlag;			// 初期化フラグ(初期値:0, 完了時:1)

char HMflag;
char freq[8];
char freq2[8];
char amp[8];
char offset[8];

GtkWidget *button_output;

/*
 *******************************************************************************
 *  関数名 : check_buf
 *  機  能 : バッファ内データのチェックを行う
 *  引  数 : TmpBuf - バッファアドレス
 *  戻り値 : 数値,+,-,.以外であればFALSEを返す
 *******************************************************************************
 */
static int check_buf(char *TmpBuf)
{
	int i;


	for( i = 0; TmpBuf[i] != 0x00; i++ )
	{
		if( TmpBuf[i] >= 0x30 && TmpBuf[i] <= 0x39 )
			;
		else if( TmpBuf[i] == 0x2B || TmpBuf[i] == 0x2D || TmpBuf[i] == 0x2E )
			;
		else
			return FALSE;
	}

	return TRUE;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdGpInit
 *  機  能 : 初期化ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// set 3 second GPIB bus timeout
#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 3);
#else
	GpStatus = gp_tmout(3);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	// Send LLO command
#ifdef _USE_U2
	GpStatus = U2Gpllo(fd);
#else
	GpStatus = gp_llo();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_llo() error [%d]\n", GpStatus);
		return;
	}

	/* 波形出力ボタンを有効にする */
	gtk_widget_set_sensitive(button_output, TRUE);

	InitFlag = 1;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdOutput
 *  機  能 : 波形出力ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdOutput( GtkWidget *widget,
                       GtkWidget *combo_box )
{
	int		GpStatus;
	char	szCmd33120[64] = "H0KM01";
	int		Waveform;
	char	TmpBuf[32];

	if(InitFlag == 0){
		return;
	}

	// 終端抵抗を選択
	if( HMflag )
		strcpy(szCmd33120, "OUTP:LOAD 50");
	else
		strcpy(szCmd33120, "OUTP:LOAD INF");

	printf ("Output1:szCmd33120: %s\n", szCmd33120);
	// Send HP33120 GPIB command
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szCmd33120);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szCmd33120 );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	// ex) "APPL:SIN 5.0E+3, 2.5, 1.25"
	strcpy(szCmd33120, "APPL:");
	Waveform = (int)gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
	switch( Waveform )
	{
		case 0: // 正弦波
			strcat(szCmd33120, "SIN ");
			break;
		case 1: // 方形波
			strcat(szCmd33120, "SQU ");
			break;
		case 2: // 三角波
			strcat(szCmd33120, "TRI ");
			break;
		case 3: // ランプ波
			strcat(szCmd33120, "RAMP ");
			break;
		case 4: // DC電圧
			strcat(szCmd33120, "DC ");
			break;
	}

	memset(TmpBuf, 0x00, sizeof(TmpBuf));
	memcpy(TmpBuf, freq, sizeof(freq));
	if( !check_buf(TmpBuf) )
	{
		printf("適切な周波数を設定してください。\n");
		return;
	}
	strcat(szCmd33120, TmpBuf);
	strcat(szCmd33120, "E+");

	memset(TmpBuf, 0x00, sizeof(TmpBuf));
	memcpy(TmpBuf, freq2, sizeof(freq2));
	if( !check_buf(TmpBuf) )
	{
		printf("適切な周波数を設定してください。\n");
		return;
	}
	strcat(szCmd33120, TmpBuf);
	strcat(szCmd33120, ", ");

	memset(TmpBuf, 0x00, sizeof(TmpBuf));
	memcpy(TmpBuf, amp, sizeof(amp));
	if( !check_buf(TmpBuf) )
	{
		printf("適切な振幅を設定してください。\n");
		return;
	}
	strcat(szCmd33120, TmpBuf);
	strcat(szCmd33120, ", ");

	memset(TmpBuf, 0x00, sizeof(TmpBuf));
	memcpy(TmpBuf, offset, sizeof(offset));
	if( !check_buf(TmpBuf) )
	{
		printf("適切なオフセットを設定してください。\n");
		return;
	}

	strcat(szCmd33120, TmpBuf);

	printf ("Output2:szCmd33120: %s\n", szCmd33120);
	// Send HP33120 GPIB command
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szCmd33120);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szCmd33120 );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error2 [%d]\n", GpStatus);
		return;
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCancel
 *  機  能 : 終了ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	int	GpStatus;

	if(InitFlag){
		//send LCL command
#ifdef _USE_U2
		GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
		GpStatus = gp_lcl( GpAdrsBuf );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_lcl() error [%d]\n", GpStatus);
		}
	}
}

/*
 *******************************************************************************
 *  関数名 : enter_adrs
 *  機  能 : アドレス入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}

/*
 *******************************************************************************
 *  関数名 : enter_freq
 *  機  能 : 周波数(Hz)入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_freq( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("freq contents: %s\n", entry_text);
  memset(freq, 0x00, sizeof(freq));
  sprintf(freq, "%s", entry_text);
}

/*
 *******************************************************************************
 *  関数名 : enter_freq
 *  機  能 : 周波数(x e)入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_freq2( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("freq2 contents: %s\n", entry_text);
  memset(freq2, 0x00, sizeof(freq2));
  sprintf(freq2, "%s", entry_text);
}

/*
 *******************************************************************************
 *  関数名 : enter_amp
 *  機  能 : 振幅(V)入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_amp( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("amp contents: %s\n", entry_text);
  memset(amp, 0x00, sizeof(amp));
  sprintf(amp, "%s", entry_text);
}

/*
 *******************************************************************************
 *  関数名 : enter_offset
 *  機  能 : オフセット(V)入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_offset( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("offset contents: %s\n", entry_text);
  memset(offset, 0x00, sizeof(offset));
  sprintf(offset, "%s", entry_text);
}

/*
 *******************************************************************************
 *  関数名 : toggle_button1
 *  機  能 : ラジオボタン選択時の処理(50Ω終端)
 *  引  数 : widget - ラジオボタンのウィジット
 *           data   - 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void toggle_button1( GtkWidget    *widget,
                      gpointer   data )
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
        /* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
		//printf ("changed button1:\n");
		HMflag = 1;
    } else {
        /* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
		//printf ("changed button2:\n");
		HMflag = 0;
    }
}

/*
 *******************************************************************************
 *  関数名 : create_window
 *  機  能 : GTK+ウィンドウを作成する
 *  引  数 : なし
 *  戻り値 : なし
 *******************************************************************************
 */
static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *table;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *status_bar;
	GtkWidget *combo_box;
	GtkWidget *button;
	GSList *group;
	gint context_id;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 480, 250);
    gtk_window_set_title (GTK_WINDOW (window), "hp33120");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    vbox = gtk_vbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 10);
    gtk_widget_show (hbox);

    /* make GPIB adrs */
    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 1);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "5");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    /* make PCI20 adrs */
    label = gtk_label_new ("REX-PCI20 アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    gtk_widget_show (label);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    /* 縦横 10x10 のテーブルを生成します。*/
    table = gtk_table_new (10, 10, TRUE);
    gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
    gtk_widget_show (table);

    // テキスト表示
    label = gtk_label_new ("出力波形");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 3, 0, 2);
    gtk_widget_show (label);

    // コンボボックス(出力波形)
    combo_box = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "正弦波");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "方形波");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "三角波");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "ランプ波");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "DC電圧");
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
    gtk_table_attach_defaults (GTK_TABLE (table), combo_box, 1, 3, 2, 4);
    gtk_widget_show (combo_box);

    // テキスト表示
    label = gtk_label_new ("終端抵抗");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 5, 8, 0, 2);
    gtk_widget_show (label);

    // ラジオボタン(50Ω終端)
    button = gtk_radio_button_new_with_label (NULL, "50Ω終端");
    group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button1),
                      NULL);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 6, 8, 2, 4);
    gtk_widget_show (button);

    // ラジオボタン(ハイインピーダンス)
    button = gtk_radio_button_new_with_label (group, "ハイインピーダンス");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
    gtk_table_attach_defaults (GTK_TABLE (table), button, 6, 9, 4, 6);
    gtk_widget_show (button);

    // 周波数入力部"周波数(Hz)"
    label = gtk_label_new ("周波数(Hz)");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 1, 4, 6, 8);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 8);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_freq),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "1");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table), entry, 1, 2, 8, 10);
    gtk_widget_show (entry);

    // 周波数入力部"× e"
    label = gtk_label_new ("× e");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 2, 3, 8, 10);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 8);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_freq2),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "3");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table), entry, 3, 4, 8, 10);
    gtk_widget_show (entry);

    // 振幅入力部
    label = gtk_label_new ("振幅(V)");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 4, 7, 6, 8);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 8);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_amp),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "0.1");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table), entry, 5, 6, 8, 10);
    gtk_widget_show (entry);

    // オフセット入力部
    label = gtk_label_new ("オフセット(V)");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 7, 9, 6, 8);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 8);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_offset),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "0");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table), entry, 7, 8, 8, 10);
    gtk_widget_show (entry);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    // 初期化ボタン
	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

    // 波形出力ボタン
	button_output = gtk_button_new_with_label ("波形出力");
	g_signal_connect (G_OBJECT (button_output), "clicked",
                      G_CALLBACK (Cmd_OnCmdOutput), (gpointer)combo_box);
	gtk_box_pack_start (GTK_BOX (hbox), button_output, TRUE, TRUE, 0);
	gtk_widget_show (button_output);

    // 終了ボタン
	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	// ウィンドウの表示
	gtk_widget_show (window);
}

/*
 *******************************************************************************
 *  関数名 : main
 *  機  能 : メインルーチン
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_DEVICES];
	unsigned short MaxUnit = MAX_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたREX-PCI20をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつREX-PCI20をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	//gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	/* 波形出力ボタンを無効にする */
	gtk_widget_set_sensitive(button_output, FALSE);

	gtk_main ();
	gp_detach(fd);

	return 0;
}

