/*
 *******************************************************************************
 *
 *  ユニバーサルカウンタSC-7201 制御プログラム
 *
 *  Compiler : gcc
 *
 *  (C) 2008 RATOC Systems, Inc.    Date : 2008.07.17
 *
 *  History : 
 *    2008.07.17    新規作成 (Windows版のソースを基にLinuxへ移植)
 *
 *******************************************************************************
 */

//gcc -g sc7201.c -o sc7201 -L/usr/local/lib/ -lrexpci20_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "rexpci20_gpib.h"

//#define _USE_U2
#define MAX_DEVICES				16

int fd=0;
short MyGpibAdrs = 1;	// PCI20のGPIBアドレス
char GpAdrsBuf[12];		// 使用するGPIB機器のGPIBアドレス
char szBuf[256];		// ワークバッファ

int InitFlag;	// 初期化フラグ(初期値:0, 完了時:1)
int DiffFlag;	// DIFFフラグ
int RangeFlag;	// レンジフラグ
int TimeFlag;	// タイムフラグ

int timer = 0;	// タイマー(停止時:0, 動作時:0以外)

GtkWidget *combo_box_range;
GtkWidget *combo_box_time;
GtkWidget *button_check;
GtkWidget *button_open;
GtkWidget *button_diff;
GtkWidget *button_reset;
GtkWidget *counter_bar;
gint counter_id;

/*
 *******************************************************************************
 *  関数名 : SC7201Command
 *  機  能 : SC7201のコマンドを作成し、実行する
 *  引  数 : なし
 *  戻り値 : gp_wrt()の戻り値
 *******************************************************************************
 */
static int SC7201Command(void)
{
	int		GpStatus;

	memset( szBuf, 0x00, sizeof(szBuf) );
	switch( RangeFlag )
	{
		case 0:	strcpy( szBuf, "F1" );	break;	/* 周波数測定200MHz */
		case 1:	strcpy( szBuf, "F2" );	break;	/* 周波数測定16MHz */
		case 2:	strcpy( szBuf, "F3" );	break;	/* 周期測定 */
		case 3:	strcpy( szBuf, "F4" );	break;	/* 周波数比測定 */
		case 4:	strcpy( szBuf, "F6" );	break;	/* 時間間隔測定 */
		case 5:	strcpy( szBuf, "F5" );	break;	/* デューレシオ測定 */
		case 6:	strcpy( szBuf, "F7" );	break;	/* 加算計数測定 */
		case 7:	strcpy( szBuf, "F8" );	break;	/* 加算計数測定 OPEN */
		case 8:	strcpy( szBuf, "F9" );	break;	/* 加算計数測定 CLOSE */
	}
	switch( TimeFlag )
	{
		case 0:	strcat( szBuf, "R3" );	break;	/* 1ms */
		case 1:	strcat( szBuf, "R5" );	break;	/* 0.1s */
		case 2:	strcat( szBuf, "R6" );	break;	/* 1s */
		case 3:	strcat( szBuf, "R7" );	break;	/* 10s */
		case 4:	strcat( szBuf, "R0" );	break;	/* 外部ゲートB */
	}
	switch( DiffFlag )
	{
		case 0:	strcat( szBuf, "P0" );	break;	/* DIFF OFF */
		case 1:	strcat( szBuf, "P1" );	break;	/* DIFF ON */
	}

	// 作成したコマンドどおりにSC7201を設定する
	printf ("send command: %s\n", szBuf);
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	return GpStatus;
}

/*
 *******************************************************************************
 *  関数名 : timeout_callback
 *  機  能 : タイマーによるコールバックルーチン
 *  引  数 : 未使用
 *  戻り値 : 正常終了時:TRUE, エラー時:FALSE
 *******************************************************************************
 */
static gboolean timeout_callback( gpointer data )
{
	int		GpStatus;
	int		length, i;

#ifdef _USE_U2
	GpStatus = U2Gptrg( fd, GpAdrsBuf );
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return FALSE;
	}

	memset(szBuf,0x00,sizeof(szBuf));
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf));
#else
	GpStatus = gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return FALSE;
	}

#ifdef _USE_U2
	length = U2Gpcount(fd);
#else
	length = gp_count();
#endif
	for(i=0;i<length;i++){
		if(szBuf[i]==0x0d){
			szBuf[0] = 0x20;
			szBuf[1] = 0x20;
			szBuf[2] = 0x20;
			szBuf[3] = 0x20;
			szBuf[4] = 0x20;
			szBuf[i] = 0x00;
			break;
		}
	}
	gtk_statusbar_push (GTK_STATUSBAR (counter_bar), GPOINTER_TO_INT (counter_id), szBuf);

	/* これがタイムアウト関数なので、
	* この関数が連続して呼び出されるように TRUE を返します。*/
	return TRUE;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdGpInit
 *  機  能 : 初期化ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	// set 15 second GPIB bus timeout
#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 15);
#else
	GpStatus = gp_tmout(15);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]\n", GpStatus);
		return;
	}

	GpStatus = SC7201Command();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	InitFlag = 1;
	if(timer){
		;
	}else{
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}

	/* ダイアログコントロールを有効にする */
	gtk_widget_set_sensitive(button_check,TRUE);
	gtk_widget_set_sensitive(combo_box_range,TRUE);
	gtk_widget_set_sensitive(combo_box_time,TRUE);
	gtk_widget_set_sensitive(button_diff,TRUE);
	gtk_widget_set_sensitive(button_reset,TRUE);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCancel
 *  機  能 : 終了ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}
}

/*
 *******************************************************************************
 *  関数名 : enter_adrs
 *  機  能 : アドレス入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdCheck
 *  機  能 : CHECKボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdCheck( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, "F0");
#else
		GpStatus = gp_wrt( GpAdrsBuf, "F0" );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
	} else {
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
#ifdef _USE_U2
		GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
		GpStatus = gp_clr( GpAdrsBuf );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_clr() error [%d]\n", GpStatus);
			return;
		}
	}

	GpStatus = SC7201Command();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 )
		timer = gtk_timeout_add (2000, timeout_callback, data);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdRange
 *  機  能 : Rangeコンボボックスの選択が変更された時時の処理
 *  引  数 : widget - 
 *           data   - 
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdRange( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 1);
		return;
	}

	if( RangeFlag != gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) )
	{
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
		if( ( RangeFlag == 7 || RangeFlag == 8 ) && gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) == 6 )
			return;
		RangeFlag = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
		if( RangeFlag == 6 )
		{
			gtk_widget_set_sensitive(button_open,TRUE);

			memset(szBuf, 0x00, sizeof(szBuf));
			gtk_statusbar_push (GTK_STATUSBAR (counter_bar), GPOINTER_TO_INT (counter_id), szBuf);
		}
		GpStatus = SC7201Command();
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		if( RangeFlag != 6 )
		{
			gtk_widget_set_sensitive(button_open,FALSE);

			timer = gtk_timeout_add (2000, timeout_callback, data);
		}
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdOpen
 *  機  能 : OPENボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdOpen( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
		return;
	}

	if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 ){
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
		return;
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
		RangeFlag = 7;
	} else {
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
		RangeFlag = 8;
	}

	GpStatus = SC7201Command();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdTime
 *  機  能 : Timeコンボボックスの選択が変更された時時の処理
 *  引  数 : widget - 
 *           data   - 
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdTime( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 1);
		return;
	}

	if( TimeFlag != gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) )
	{
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
		TimeFlag = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
		GpStatus = SC7201Command();
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 )
			timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdDiff
 *  機  能 : DIFFボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdDiff( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
		DiffFlag = 1;
	} else {
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
		DiffFlag = 0;
	}

	GpStatus = SC7201Command();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 )
		timer = gtk_timeout_add (2000, timeout_callback, data);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdReset
 *  機  能 : RESETボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdReset( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( DiffFlag == 0 )
	{
#ifdef _USE_U2
		GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
		GpStatus = gp_lcl( GpAdrsBuf );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_lcl() error [%d]\n", GpStatus);
			return;
		}
	}
	if( DiffFlag == 1 )
	{
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, "G1");
#else
		GpStatus = gp_wrt( GpAdrsBuf, "G1" );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		GpStatus = SC7201Command();
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}

		if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 )
			timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

/*
 *******************************************************************************
 *  関数名 : create_window
 *  機  能 : GTK+ウィンドウを作成する
 *  引  数 : なし
 *  戻り値 : なし
 *******************************************************************************
 */
static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *status_bar;
	GtkWidget *button;
	gint context_id;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 360, 240);
    gtk_window_set_title (GTK_WINDOW (window), "sc7201");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    vbox = gtk_vbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    /* make GPIB adrs */
    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 1);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "4");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    /* make PCI20 adrs */
    label = gtk_label_new ("REX-PCI20 アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("SC7201 UNIVERSAL COUNTER");
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
    gtk_container_add (GTK_CONTAINER (frame), hbox);
    gtk_widget_show (hbox);

    // ステータスバー
    label = gtk_label_new ("計測値");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    counter_bar = gtk_statusbar_new ();
    counter_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (counter_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " ", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (counter_bar), counter_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), counter_bar, TRUE, TRUE, 0);
    gtk_widget_show (counter_bar);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    // CHECKボタン
	button_check = gtk_toggle_button_new_with_label ("CHECK");
	g_signal_connect (G_OBJECT (button_check), "toggled",
                      G_CALLBACK (Cmd_OnCmdCheck), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button_check, TRUE, TRUE, 0);
	gtk_widget_show (button_check);              

    // Rangeコンボボックス
    combo_box_range = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_range), "FREQ A 200MHz");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_range), "FREQ A 16MHz");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_range), "PERIOD A");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_range), "f1/f2 A/B");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_range), "TIME INT A->B");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_range), "DUTY A");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_range), "MANUAL A");
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box_range), 1);
    g_signal_connect (G_OBJECT (combo_box_range), "changed",
                      G_CALLBACK (Cmd_OnCmdRange), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), combo_box_range, FALSE, FALSE, 0);
    gtk_widget_show (combo_box_range);

    // OPENボタン
	button_open = gtk_toggle_button_new_with_label ("OPEN");
	g_signal_connect (G_OBJECT (button_open), "toggled",
                      G_CALLBACK (Cmd_OnCmdOpen), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button_open, TRUE, TRUE, 0);
	gtk_widget_show (button_open);              

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    // Timeコンボボックス
    combo_box_time = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_time), "1ms");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_time), "0.1s");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_time), "1s");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_time), "10s");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box_time), "EXT GATEB");
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box_time), 1);
    g_signal_connect (G_OBJECT (combo_box_time), "changed",
                      G_CALLBACK (Cmd_OnCmdTime), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), combo_box_time, FALSE, FALSE, 0);
    gtk_widget_show (combo_box_time);

    // DIFFボタン
	button_diff = gtk_toggle_button_new_with_label ("DIFF");
	g_signal_connect (G_OBJECT (button_diff), "toggled",
                      G_CALLBACK (Cmd_OnCmdDiff), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button_diff, TRUE, TRUE, 0);
	gtk_widget_show (button_diff);              

    // RESETボタン
	button_reset = gtk_button_new_with_label ("RESET");
	g_signal_connect (G_OBJECT (button_reset), "clicked",
                      G_CALLBACK (Cmd_OnCmdReset), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button_reset, TRUE, TRUE, 0);
	gtk_widget_show (button_reset);              

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    // 初期化ボタン
	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

    // 終了ボタン
	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	// ウィンドウの表示
	gtk_widget_show (window);
}

/*
 *******************************************************************************
 *  関数名 : main
 *  機  能 : メインルーチン
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_DEVICES];
	unsigned short MaxUnit = MAX_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたREX-PCI20をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつREX-PCI20をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	RangeFlag = 1;
	TimeFlag = 1;
	DiffFlag = 0;
	//gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	/* ダイアログコントロールを無効にする */
	gtk_widget_set_sensitive(button_check,FALSE);
	gtk_widget_set_sensitive(combo_box_range,FALSE);
	gtk_widget_set_sensitive(button_open,FALSE);
	gtk_widget_set_sensitive(combo_box_time,FALSE);
	gtk_widget_set_sensitive(button_diff,FALSE);
	gtk_widget_set_sensitive(button_reset,FALSE);

	gtk_main ();
	gp_detach(fd);

	return 0;
}

