/*
 *******************************************************************************
 *
 *  デジタルパワーメータWT110E 制御プログラム
 *
 *  Compiler : gcc
 *
 *  (C) 2008 RATOC Systems, Inc.    Date : 2008.07.17
 *
 *  History : 
 *    2008.07.17    新規作成 (Windows版のソースを基にLinuxへ移植)
 *
 *******************************************************************************
 */

//gcc -g wt110e.c -o wt110e -L/usr/local/lib/ -lrexpci20_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <math.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "rexpci20_gpib.h"

//#define _USE_U2
#define MAX_DEVICES				16


int fd=0;
short MyGpibAdrs = 0;	// PCI20のGPIBアドレス
char GpAdrsBuf[12];		// 使用するGPIB機器のGPIBアドレス
char szBuf[256];		// ワークバッファ
int InitFlag;			// 初期化フラグ(初期値:0, 完了時:1)

int Intervalh;		// 設定するインターバル時間の時
int Intervalm;		// 設定するインターバル時間の分
int Intervals;		// 設定するインターバル時間の秒
char DisPlayAFlag;	// ディスプレイAのモードを示すフラグ
char DisPlayBFlag;	// ディスプレイBのモードを示すフラグ
char DisPlayCFlag;	// ディスプレイCのモードを示すフラグ
char VRangeFlag;	// 電圧レンジを示すフラグ
char ARangeFlag;	// 電流レンジを示すフラグ
char ModeFlag;		// モードを示すフラグ
char RecallFlag;	// リコールフラグ
char ButtonFlag;	// どのボタンが押されたか示すフラグ

char SetupFlag;		// Setup[]のどれを選んだかを示すフラグ
char OnOffFlag;		// OnOff[]のどれを選んだかを示すフラグ
char MathFlag;		// Math[]のどれを選んだかを示すフラグ
char MemoryFlag;	// Memory[]のどれを選んだかを示すフラグ
char FileFlag;		// File[]のどれを選んだかを示すフラグ
char IntegFlag;		// Integ[]のどれを選んだかを示すフラグ
char IntegModeFlag;	// IntegMode[]のどれを選んだかを示すフラグ
char AvgFlag;		// Avg[]のどれを選んだかを示すフラグ
char AvgModeFlag;	// AvgMode[]のどれを選んだかを示すフラグ
char IntervalFlag;	// Interval設定するためのフラグ

char *Vrange[]={ "15", "30", "60", "150", "300", "600", "Auto" };
char *Arange[]={ "0.5", "1", "2", "5", "10", "20", "Auto" };
char *Setup[]={ "FLT", "AVG", "SCALE", "PNLRST", "MATH"};
char *OnOff[]={ "on", "off", "type", "intvl"};
char *Math[]={ "A+B", "A-B", "A*B", "A/B", "A/(B*B)", "(A*A)/B" };
char *Memory[]={ "Store", "Recal", "Pnlstr", "Pnlrcl" };
char *File[]={ "File1", "File2", "File3", "File4" };
char *Integ[]={ "mode", "timer" };
char *IntegMode[]={ "nor", "Cont" };
char *Avg[]={ "Lin", "Ep" };
char *AvgMode[]={ "8", "16", "32", "64" };

int timer = 0;		// タイマー(停止時:0, 動作時:0以外)

GtkWidget *button_vrange;
GtkWidget *button_arange;
GtkWidget *button_hold;
GtkWidget *button_up;
GtkWidget *button_down;
GtkWidget *button_enter;
GtkWidget *button_start;
GtkWidget *button_stop;
GtkWidget *button_reset;
GtkWidget *button_local;
GtkWidget *button_setup;
GtkWidget *button_shift;
GtkWidget *button_funca;
GtkWidget *button_funcb;
GtkWidget *button_funcc;
GtkWidget *button_graph;
GtkWidget *entry_timeh;
GtkWidget *entry_timem;
GtkWidget *entry_times;
GtkWidget *entry_funca;
GtkWidget *entry_funcb;
GtkWidget *entry_funcc;
GtkWidget *time_bar;
gint time_id;

/*
 *******************************************************************************
 *  関数名 : MySetItemText
 *  機  能 : ディスプレイA,B,Cを設定
 *  引  数 : abuf - ディスプレイAの文字列
 *           bbuf - ディスプレイBの文字列
 *           cbuf - ディスプレイCの文字列
 *  戻り値 : なし
 *******************************************************************************
 */
static void MySetItemText(char *abuf, char *bbuf, char *cbuf)
{
	if( abuf != NULL ){
		gtk_entry_set_text (GTK_ENTRY (entry_funca), abuf);
	}else{
		gtk_entry_set_text (GTK_ENTRY (entry_funca), " ");
	}
	if( bbuf != NULL ){
		gtk_entry_set_text (GTK_ENTRY (entry_funcb), bbuf);
	}else{
		gtk_entry_set_text (GTK_ENTRY (entry_funcb), " ");
	}
	if( cbuf != NULL ){
		gtk_entry_set_text (GTK_ENTRY (entry_funcc), cbuf);
	}else{
		gtk_entry_set_text (GTK_ENTRY (entry_funcc), " ");
	}

	if( abuf == NULL ){
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), FALSE);
	}else{
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcc), FALSE);
	}
}

/*
 *******************************************************************************
 *  関数名 : WT110ERead
 *  機  能 : WT110Eのディスプレイを設定し直した後、
 *           表示されている値をダイアログに表示する
 *  引  数 : なし
 *  戻り値 : 正常終了時:TRUE, エラー時:FALSE
 *******************************************************************************
 */
static int WT110ERead(void)
{
	int		GpStatus;
	int		cnt;
	unsigned char	Code;
	double	Value;
	char	szTmp[128];

	if( RecallFlag == 1 ) // リコール時
	{
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "DA1;DB2;DC3");
#else
		gp_wrt( GpAdrsBuf, "DA1;DB2;DC3" );
#endif
		DisPlayAFlag = 1; // ディスプレイAは電圧値
		DisPlayBFlag = 2; // ディスプレイBは電流値
		DisPlayCFlag = 3; // ディスプレイCは電力値
	}

	/******** ディスプレイAの表示 ********/
	memset(szBuf,0x00,sizeof(szBuf));
	sprintf( szBuf, "OF1,%d,1;H0;OD", DisPlayAFlag ); // H0:ヘッダ無し
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return FALSE;
	}
	
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return FALSE;
	}

#ifdef _USE_U2
	GpStatus = U2Gprds(fd, GpAdrsBuf, &Code);
#else
	GpStatus = gp_rds( GpAdrsBuf, &Code );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_rds() error [%d]\n", GpStatus);
		return FALSE;
	}

	memset(szBuf,0x00,sizeof(szBuf));
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf));
#else
	GpStatus = gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return FALSE;
	}
	if( RecallFlag == 1 ) // リコール時
	{
		memset(szBuf,0x00,sizeof(szBuf));
#ifdef _USE_U2
		GpStatus = U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf));
#else
		GpStatus = gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_red() error [%d]\n", GpStatus);
			return FALSE;
		}
	}
	Value = (double)atof( szBuf );
	if( Value == 0 )
		sprintf(szTmp,"%d", 0 );
	else if( fabs(Value) <= 0.001 )
		sprintf(szTmp,"%.3f E-3", Value * 1000 );
	else if( fabs(Value) >= 1000 )
		sprintf(szTmp,"%.3f E+3", Value / 1000 );
	else
		sprintf(szTmp,"%.3f", Value );

	gtk_entry_set_text (GTK_ENTRY (entry_funca), szTmp);

	/******** ディスプレイBの表示 ********/
	memset(szBuf,0x00,sizeof(szBuf));
	sprintf( szBuf, "OF1,%d,1;H0;OD", DisPlayBFlag ); // H0:ヘッダ無し
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return FALSE;
	}
	
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return FALSE;
	}

#ifdef _USE_U2
	GpStatus = U2Gprds(fd, GpAdrsBuf, &Code);
#else
	GpStatus = gp_rds( GpAdrsBuf, &Code );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_rds() error [%d]\n", GpStatus);
		return FALSE;
	}

	memset(szBuf,0x00,sizeof(szBuf));
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf));
#else
	GpStatus = gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return FALSE;
	}
	if( RecallFlag == 1 )
	{
		for( cnt = 1; cnt <= 2; cnt++ )
		{
			memset(szBuf,0x00,sizeof(szBuf));
#ifdef _USE_U2
			GpStatus = U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf));
#else
			GpStatus = gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
			if ( GpStatus != 0 )
			{
				printf("gp_red() error [%d]\n", GpStatus);
				return FALSE;
			}
		}
	}
	Value = (double)atof( szBuf );
	if( Value == 0 )
		sprintf(szTmp,"%d", 0 );
	else if( fabs(Value) <= 0.001 )
		sprintf(szTmp,"%.3f E-3", Value * 1000 );
	else if( fabs(Value) >= 1000 )
		sprintf(szTmp,"%.3f E+3", Value / 1000 );
	else
		sprintf(szTmp,"%.3f", Value );

	gtk_entry_set_text (GTK_ENTRY (entry_funcb), szTmp);

	/******** ディスプレイCの表示 ********/
	memset(szBuf,0x00,sizeof(szBuf));
	sprintf( szBuf, "OF1,%d,1;H0;OD", DisPlayCFlag ); // H0:ヘッダ無し
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return FALSE;
	}
	
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return FALSE;
	}

#ifdef _USE_U2
	GpStatus = U2Gprds(fd, GpAdrsBuf, &Code);
#else
	GpStatus = gp_rds( GpAdrsBuf, &Code );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_rds() error [%d]\n", GpStatus);
		return FALSE;
	}

	memset(szBuf,0x00,sizeof(szBuf));
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf));
#else
	GpStatus = gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return FALSE;
	}
	if( RecallFlag == 1 )
	{
		for( cnt = 1; cnt <= 3; cnt++ )
		{
			memset(szBuf,0x00,sizeof(szBuf));
#ifdef _USE_U2
			GpStatus = U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf));
#else
			GpStatus = gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
			if ( GpStatus != 0 )
			{
				printf("gp_red() error [%d]\n", GpStatus);
				return FALSE;
			}
		}
	}
	Value = (double)atof( szBuf );
	if( Value == 0 )
		sprintf(szTmp,"%d", 0 );
	else if( fabs(Value) <= 0.001 )
		sprintf(szTmp,"%.3f E-3", Value * 1000 );
	else if( fabs(Value) >= 1000 )
		sprintf(szTmp,"%.3f E+3", Value / 1000 );
	else
		sprintf(szTmp,"%.3f", Value );

	gtk_entry_set_text (GTK_ENTRY (entry_funcc), szTmp);

	return TRUE;
}

/*
 *******************************************************************************
 *  関数名 : WT110ECommand
 *  機  能 : WT110Eのコマンドを作成し、実行する
 *  引  数 : なし
 *  戻り値 : gp_wrt()の戻り値
 *******************************************************************************
 */
static int WT110ECommand(void)
{
	int		GpStatus;

	memset( szBuf, 0x00, sizeof(szBuf) );
	switch( DisPlayAFlag )
	{
		case 1:	strcpy( szBuf, "DA1;EA1;");		break; // V
		case 2:	strcpy( szBuf, "DA2;EA1;");		break; // A
		case 3:	strcpy( szBuf, "DA3;EA1;");		break; // W
		case 4:	strcpy( szBuf, "DA4;EA1;");		break; // var
		case 5:	strcpy( szBuf, "DA5;EA1;");		break; // VA
		case 15:	strcpy( szBuf, "DA15;");		break; // TIME
	}
	switch( DisPlayBFlag )
	{
		case 1:	strcat( szBuf, "DB1;EB1;" );	break; // V
		case 2:	strcat( szBuf, "DB2;EB1;" );	break; // A
		case 3:	strcat( szBuf, "DB3;EB1;" );	break; // W
		case 6:	strcat( szBuf, "DB6;EB1;" );	break; // PF
		case 11:	strcat( szBuf, "DB11;EB1;" );	break; // deg
	}
	switch( DisPlayCFlag )
	{
		case 1:	strcat( szBuf, "DC1;EC1;" );	break; // V
		case 2:	strcat( szBuf, "DC2;EC1;" );	break; // A
		case 3:	strcat( szBuf, "DC3;EC1;" );	break; // W
		case 7:	strcat( szBuf, "DC7;EC1;" );	break; // V Hz
		case 8:	strcat( szBuf, "DC8;EC1;" );	break; // A Hz
		case 9:	strcat( szBuf, "DC9;EC1;" );	break; // Wh
		case 10:	strcat( szBuf, "DC10;EC1;" );	break; // Ah
		case 14:	strcat( szBuf, "DC14;" );		break; // MATH
		case 24:	strcat( szBuf, "DC24;EC1;" );	break; // Wh+
		case 25:	strcat( szBuf, "DC25;EC1;" );	break; // Wh-
		case 26:	strcat( szBuf, "DC26;EC1;" );	break; // Ah+
		case 27:	strcat( szBuf, "DC27;EC1;" );	break; // Ah-
	}
	switch( VRangeFlag )
	{
		case 1:	strcat( szBuf, "RV3;" );		break; // 15Vレンジ
		case 2:	strcat( szBuf, "RV4;" );		break; // 30Vレンジ
		case 3:	strcat( szBuf, "RV5;" );		break; // 60Vレンジ
		case 4:	strcat( szBuf, "RV7;" );		break; // 150Vレンジ
		case 5:	strcat( szBuf, "RV8;" );		break; // 300Vレンジ
		case 6:	strcat( szBuf, "RV9;" );		break; // 600Vレンジ
		case 7:	strcat( szBuf, "AV1;" );		break; // オートレンジ
	}
	switch( ARangeFlag )
	{
		case 1:	strcat( szBuf, "RA4;" );		break; // 0.5Aレンジ
		case 2:	strcat( szBuf, "RA5;" );		break; // 1Aレンジ
		case 3:	strcat( szBuf, "RA6;" );		break; // 2Aレンジ
		case 4:	strcat( szBuf, "RA7;" );		break; // 5Aレンジ
		case 5:	strcat( szBuf, "RA8;" );		break; // 10Aレンジ
		case 6:	strcat( szBuf, "RA9;" );		break; // 20Aレンジ
		case 7:	strcat( szBuf, "AA1;" );		break; // オートレンジ
	}
	switch( ModeFlag )
	{
		case 1:	strcat( szBuf, "MN0" );		break; // RMS
		case 2:	strcat( szBuf, "MN1" );		break; // V MEAN
		case 3:	strcat( szBuf, "MN2" );		break; // DC
	}

	// 作成したコマンドをWT110Eに設定する
	printf ("send command: %s\n", szBuf);
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	return GpStatus;
}

/*
 *******************************************************************************
 *  関数名 : timeout_callback
 *  機  能 : タイマーによるコールバックルーチン
 *  引  数 : 未使用
 *  戻り値 : 正常終了時:TRUE, エラー時:FALSE
 *******************************************************************************
 */
static gboolean timeout_callback( gpointer data )
{
	unsigned char	Code;

	//++ 080714
	gtk_editable_set_editable(GTK_EDITABLE(entry_funca), TRUE);
	gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), TRUE);
	gtk_editable_set_editable(GTK_EDITABLE(entry_funcc), TRUE);
	gtk_widget_set_sensitive(entry_timeh, FALSE);
	gtk_widget_set_sensitive(entry_timem, FALSE);
	gtk_widget_set_sensitive(entry_times, FALSE);
	//--

	if( WT110ERead() != TRUE ){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( RecallFlag == 1 )
	{
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "RO?");
		U2Gptrg(fd, GpAdrsBuf);
		U2Gprds(fd, GpAdrsBuf, &Code);
		memset(szBuf,0x00,sizeof(szBuf));
		U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf) );
#else
		gp_wrt( GpAdrsBuf, "RO?" );
		gp_trg( GpAdrsBuf );
		gp_rds( GpAdrsBuf, &Code);
		memset(szBuf,0x00,sizeof(szBuf));
		gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
		if( strncmp( szBuf, "RO0", 3 ) == 0 ) // リコールOFFの場合
		{
			gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), " ");
			RecallFlag = 0;
			DisPlayAFlag = 1; // ディスプレイAは電圧値
			DisPlayBFlag = 2; // ディスプレイBは電流値
			DisPlayCFlag = 3; // ディスプレイCは電力値
		}
	}

	/* これがタイムアウト関数なので、
	* この関数が連続して呼び出されるように TRUE を返します。*/
	return TRUE;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdGpInit
 *  機  能 : 初期化ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	/* ダイアログコントロールを有効にする */
	gtk_widget_set_sensitive(button_vrange, TRUE);
	gtk_widget_set_sensitive(button_arange, TRUE);
	gtk_widget_set_sensitive(button_hold, TRUE);
	gtk_widget_set_sensitive(button_up, TRUE);
	gtk_widget_set_sensitive(button_down, TRUE);
	gtk_widget_set_sensitive(button_enter, TRUE);
	gtk_widget_set_sensitive(button_start, TRUE);
	gtk_widget_set_sensitive(button_stop, TRUE);
	gtk_widget_set_sensitive(button_reset, TRUE);
	gtk_widget_set_sensitive(button_local, TRUE);
	gtk_widget_set_sensitive(button_setup, TRUE);
	gtk_widget_set_sensitive(button_shift, TRUE);
	gtk_widget_set_sensitive(button_funca, TRUE);
	gtk_widget_set_sensitive(button_funcb, TRUE);
	gtk_widget_set_sensitive(button_funcc, TRUE);
	// グラフ表示は未サポート
	//gtk_widget_set_sensitive(button_graph, TRUE);

	GpStatus = WT110ECommand();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	InitFlag = 1;
	if(timer){
		;
	}else{
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCancel
 *  機  能 : 終了ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}
}

/*
 *******************************************************************************
 *  関数名 : enter_adrs
 *  機  能 : アドレス入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}

/*
 *******************************************************************************
 *  関数名 : enter_Intervalh
 *  機  能 : Interval(時)入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_Intervalh( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  Intervalh = atoi( entry_text );
  //printf ("hInterval contents: %d\n", Intervalh);
}

/*
 *******************************************************************************
 *  関数名 : enter_Intervalh
 *  機  能 : Interval(分)入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_Intervalm( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  Intervalm = atoi( entry_text );
  //printf ("Intervalm contents: %d\n", Intervalm);
}

/*
 *******************************************************************************
 *  関数名 : enter_Intervalh
 *  機  能 : Interval(秒)入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_Intervals( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  Intervals = atoi( entry_text );
  //printf ("Intervals contents: %d\n", Intervals);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdFuncA
 *  機  能 : FUNCTION Aボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdFuncA( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	// DisPlayAFlag は 1,2,3,4,5,15 の値をとる
	if( ButtonFlag == 12 ) // 逆方向に切替え
	{
		ButtonFlag = 0;
		DisPlayAFlag = DisPlayAFlag - 1;
		if( DisPlayAFlag == 0 )	DisPlayAFlag = 15;
		if( DisPlayAFlag == 14) DisPlayAFlag = 5;
	}
	else // 順方向に切替え
	{
		DisPlayAFlag = DisPlayAFlag + 1;
		if( DisPlayAFlag == 6 ) DisPlayAFlag = 15;
		else if( DisPlayAFlag == 16) DisPlayAFlag = 1;
	}
	GpStatus = WT110ECommand();
	if( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	if( ButtonFlag != 3 ) // ホールド状態でないとき
		timer = gtk_timeout_add (2000, timeout_callback, data);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdFuncB
 *  機  能 : FUNCTION Bボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdFuncB( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	// DisPlayBFlag は 1,2,3,6,11 の値をとる
	if( ButtonFlag == 12 ) // 逆方向に切替え
	{
		ButtonFlag = 0;
		DisPlayBFlag = DisPlayBFlag - 1;
		if( DisPlayBFlag == 0 )	DisPlayBFlag = 11;
		if( DisPlayBFlag == 10) DisPlayBFlag = 6;
		if( DisPlayBFlag == 5 )	DisPlayBFlag = 3;
	}
	else // 順方向に切替え
	{
		DisPlayBFlag = DisPlayBFlag + 1;
		if( DisPlayBFlag == 4 )	DisPlayBFlag = 6;
		else if( DisPlayBFlag == 7 ) DisPlayBFlag = 11;
		else if( DisPlayBFlag == 12)	DisPlayBFlag = 1;
	}
	GpStatus = WT110ECommand();
	if( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	if( ButtonFlag != 3 ) // ホールド状態でないとき
		timer = gtk_timeout_add (2000, timeout_callback, data);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdFuncC
 *  機  能 : FUNCTION Cボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdFuncC( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	// DisPlayCFlag は 1,2,3,7,8,9,10,14,24,25,26,27 の値をとる
	if( ButtonFlag == 12 ) // 逆方向に切替え
	{
		ButtonFlag = 0;
		DisPlayCFlag = DisPlayCFlag - 1;
		if( DisPlayCFlag == 23)	DisPlayCFlag = 14;
		if( DisPlayCFlag == 13) DisPlayCFlag = 10;
		if( DisPlayCFlag == 6 )	DisPlayCFlag = 3;
		if( DisPlayCFlag == 0 )	DisPlayCFlag = 27;
	}
	else // 順方向に切替え
	{
		DisPlayCFlag = DisPlayCFlag + 1;
		if( DisPlayCFlag == 4 )	DisPlayCFlag = 7;
		else if( DisPlayCFlag == 11) DisPlayCFlag = 14;
		else if( DisPlayCFlag == 15)	DisPlayCFlag = 24;
		else if( DisPlayCFlag == 28)	DisPlayCFlag = 1;
	}
	GpStatus = WT110ECommand();
	if( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	if( ButtonFlag != 3 ) // ホールド状態でないとき
		timer = gtk_timeout_add (2000, timeout_callback, data);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdVRange
 *  機  能 : V RANGEボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdVRange( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( ButtonFlag == 0 ) // 電圧レンジ変更
	{
		MySetItemText( "VOLT", "RANGE", Vrange[VRangeFlag-1] );
		ButtonFlag = 1;
	}
	else if( ButtonFlag == 12 ) // モード変更
	{
		ModeFlag = ModeFlag + 1;
		if( ModeFlag == 4 ) ModeFlag = 1;
		GpStatus = WT110ECommand();
		if( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdARange
 *  機  能 : A RANGEボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdARange( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	MySetItemText( "AMA", "RANGE", Arange[ARangeFlag-1] );
	ButtonFlag = 2;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdHold
 *  機  能 : HOLDボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdHold( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( ButtonFlag == 0 ) // ホールド ON
	{
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, "HD1");
#else
		GpStatus = gp_wrt( GpAdrsBuf, "HD1" );
#endif
		if( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		ButtonFlag = 3;
	}
	else if( ButtonFlag == 3 ) // ホールド OFF
	{
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, "HD0");
#else
		GpStatus = gp_wrt( GpAdrsBuf, "HD0" );
#endif
		if( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		timer = gtk_timeout_add (2000, timeout_callback, data);
		ButtonFlag = 0;
	}
	else if( ButtonFlag == 12 ) // トリガ
	{
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, "ST");
#else
		GpStatus = gp_wrt( GpAdrsBuf, "ST" );
#endif
		if( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		if( WT110ERead() == TRUE )
			ButtonFlag = 3;
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdUp
 *  機  能 : ↑ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdUp( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( ButtonFlag == 1 ) // 電圧レンジ変更
	{
		VRangeFlag = VRangeFlag + 1;
		if( VRangeFlag == 8 )
			VRangeFlag = 7;
		MySetItemText( "VOLT", "RANGE", Vrange[VRangeFlag-1] );
	}
	else if( ButtonFlag == 2 ) // 電流レンジ変更
	{
		ARangeFlag = ARangeFlag + 1;
		if( ARangeFlag == 8 )
			ARangeFlag = 7;
		MySetItemText( "AMA", "RANGE", Arange[ARangeFlag-1] );
	}
	else if( ButtonFlag == 8 && OnOffFlag == 0 && FileFlag == 0 && AvgFlag == 0) // メモリメニュー変更
	{
		MemoryFlag = MemoryFlag + 1;
		if( MemoryFlag == 5 )MemoryFlag = 1;
		MySetItemText( "MEM", NULL, Memory[MemoryFlag-1] );
	}
	else if( MemoryFlag == 1 || MemoryFlag == 2 ) // ON, OFF変更
	{
		OnOffFlag = OnOffFlag + 1;
		if( OnOffFlag == 3 )OnOffFlag = 4;
		if( OnOffFlag == 5 )OnOffFlag = 1;
		MySetItemText( "MEM", Memory[MemoryFlag-1], OnOff[OnOffFlag-1] );
	}
	else if( MemoryFlag == 3 || MemoryFlag == 4 ) // File変更
	{
		FileFlag = FileFlag + 1;
		if( FileFlag == 5 )FileFlag = 1;
		MySetItemText( NULL, File[FileFlag-1], "File" );
	}
	else if( ButtonFlag == 11 && OnOffFlag == 0 && MathFlag == 0 ) // セットアップメニュー変更
	{
		SetupFlag = SetupFlag + 1;
		if( SetupFlag == 6 )SetupFlag = 1;
		MySetItemText( "SETUP", NULL, Setup[SetupFlag-1] );
	}
	else if( SetupFlag == 1 || SetupFlag == 3 ) // ON, OFF変更
	{
		OnOffFlag = OnOffFlag + 1;
		if( OnOffFlag == 3 )OnOffFlag = 1;
		MySetItemText( "SETUP", Setup[SetupFlag-1], OnOff[OnOffFlag-1] );
	}
	else if( SetupFlag == 2 && OnOffFlag == 3 && AvgFlag != 0 && AvgModeFlag == 0 ) // アベレージングTYPE
	{
		AvgFlag = AvgFlag + 1;
		if( AvgFlag == 3 )AvgFlag = 1;
		MySetItemText( "AVG", Avg[AvgFlag-1], "8" );
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), FALSE);
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcc), TRUE);
	}
	else if( SetupFlag == 2 && OnOffFlag == 3 && AvgFlag != 0 && AvgModeFlag != 0 ) // アベレージングTYPE
	{
		AvgModeFlag = AvgModeFlag + 1;
		if( AvgModeFlag == 5 )AvgModeFlag = 1;
		MySetItemText( "AVG", Avg[AvgFlag-1], AvgMode[AvgModeFlag-1] );
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), TRUE);
	}
	else if( SetupFlag == 2 ) // ON, OFF変更
	{
		OnOffFlag = OnOffFlag + 1;
		if( OnOffFlag == 4 )OnOffFlag = 1;
		MySetItemText( "SETUP", Setup[SetupFlag-1], OnOff[OnOffFlag-1] );
	}
	else if( SetupFlag == 5 ) // 四則演算変更
	{
		MathFlag = MathFlag + 1;
		if( MathFlag == 7 )MathFlag = 1;
		MySetItemText( "SETUP", Setup[SetupFlag-1], Math[MathFlag-1] );
	}
	else if( ButtonFlag == 9 && IntegModeFlag == 0 )
	{
		IntegFlag = IntegFlag + 1;
		if( IntegFlag == 3 )IntegFlag = 1;
		MySetItemText( "INTEG", NULL, Integ[IntegFlag-1] );
	}
	else if( ButtonFlag == 9 && IntegModeFlag != 0 )
	{
		IntegModeFlag = IntegModeFlag + 1;
		if( IntegModeFlag == 3 )IntegModeFlag = 1;
		MySetItemText( "INTEG", Integ[IntegFlag-1], IntegMode[IntegModeFlag-1] );
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdUp
 *  機  能 : ↓ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdDown( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( ButtonFlag == 1 ) // 電圧レンジ変更
	{
		VRangeFlag = VRangeFlag - 1;
		if( VRangeFlag == 0 )
			VRangeFlag = 1;
		MySetItemText( "VOLT", "RANGE", Vrange[VRangeFlag-1] );
	}
	else if( ButtonFlag == 2 ) // 電流レンジ変更
	{
		ARangeFlag = ARangeFlag - 1;
		if( ARangeFlag == 0 )
			ARangeFlag = 1;
		MySetItemText( "AMA", "RANGE", Arange[ARangeFlag-1] );
	}
	else if( ButtonFlag == 8 && OnOffFlag == 0 && FileFlag == 0 ) // メモリメニュー変更
	{
		MemoryFlag = MemoryFlag - 1;
		if( MemoryFlag == 0 )MemoryFlag = 4;
		MySetItemText( "MEM", NULL, Memory[MemoryFlag-1] );
	}
	else if( MemoryFlag == 1 || MemoryFlag == 2 ) // ON, OFF変更
	{
		OnOffFlag = OnOffFlag - 1;
		if( OnOffFlag == 0 )OnOffFlag = 4;
		if( OnOffFlag == 3 )OnOffFlag = 2;
		MySetItemText( "MEM", Memory[MemoryFlag-1], OnOff[OnOffFlag-1] );
	}
	else if( MemoryFlag == 3 || MemoryFlag == 4 ) // File変更
	{
		FileFlag = FileFlag - 1;
		if( FileFlag == 0 )FileFlag = 4;
		MySetItemText( NULL, File[FileFlag-1], "File" );
	}
	else if( ButtonFlag == 11 && OnOffFlag == 0 && MathFlag == 0 && AvgFlag == 0 ) // セットアップメニュー変更
	{
		SetupFlag = SetupFlag - 1;
		if( SetupFlag == 0 )SetupFlag = 5;
		MySetItemText( "SETUP", NULL, Setup[SetupFlag-1] );
	}
	else if( SetupFlag == 1 || SetupFlag == 3 ) // ON, OFF変更
	{
		OnOffFlag = OnOffFlag - 1;
		if( OnOffFlag == 0 )OnOffFlag = 2;
		MySetItemText( "SETUP", Setup[SetupFlag-1], OnOff[OnOffFlag-1] );
	}
	else if( SetupFlag == 2 && AvgFlag == 0 ) // ON, OFF変更
	{
		OnOffFlag = OnOffFlag - 1;
		if( OnOffFlag == 0 )OnOffFlag = 3;
		MySetItemText( "SETUP", Setup[SetupFlag-1], OnOff[OnOffFlag-1] );
	}
	else if( SetupFlag == 2 && OnOffFlag == 3 && AvgFlag != 0 && AvgModeFlag == 0 ) // アベレージングTYPE
	{
		AvgFlag = AvgFlag - 1;
		if( AvgFlag == 0 )AvgFlag = 2;
		MySetItemText( "AVG", Avg[AvgFlag-1], "8" );
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), FALSE);
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcc), TRUE);
	}
	else if( SetupFlag == 2 && OnOffFlag == 3 && AvgFlag != 0 && AvgModeFlag != 0 ) // アベレージングTYPE
	{
		AvgModeFlag = AvgModeFlag - 1;
		if( AvgModeFlag == 0 )AvgModeFlag = 4;
		MySetItemText( "AVG", Avg[AvgFlag-1], AvgMode[AvgModeFlag-1] );
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), TRUE);
	}
	else if( SetupFlag == 5 ) // 四則演算変更
	{
		MathFlag = MathFlag - 1;
		if( MathFlag == 0 )MathFlag = 6;
		MySetItemText( "SETUP", Setup[SetupFlag-1], Math[MathFlag-1] );
	}
	else if( ButtonFlag == 9 && IntegModeFlag == 0 )
	{
		IntegFlag = IntegFlag - 1;
		if( IntegFlag == 0 )IntegFlag = 2;
		MySetItemText( "INTEG", NULL, Integ[IntegFlag-1] );
	}
	else if( ButtonFlag == 9 && IntegModeFlag != 0 )
	{
		IntegModeFlag = IntegModeFlag - 1;
		if( IntegModeFlag == 0 )IntegModeFlag = 2;
		MySetItemText( "INTEG", Integ[IntegFlag-1], IntegMode[IntegModeFlag-1] );
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdEnter
 *  機  能 : ENTERボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdEnter( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( ButtonFlag == 1 || ButtonFlag == 2 ) // 電圧または電流レンジの決定
	{
		ButtonFlag = 0; 
		GpStatus = WT110ECommand();
		if( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( SetupFlag != 4 && SetupFlag != 5 && ButtonFlag == 11 && OnOffFlag == 0 ) // セットアップメニューの決定
	{
		OnOffFlag = 1;
		MySetItemText( "SETUP", Setup[SetupFlag-1], OnOff[OnOffFlag-1] );
	}
	else if( SetupFlag == 1 && OnOffFlag == 1 ) // フィルタON
	{
		SetupFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "FL1");
#else
		gp_wrt( GpAdrsBuf, "FL1" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( SetupFlag == 1 && OnOffFlag == 2 ) // フィルタOFF
	{
		SetupFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "FL0");
#else
		gp_wrt( GpAdrsBuf, "FL0" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( SetupFlag == 2 && OnOffFlag == 1 ) // アベレージングON
	{
		SetupFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "AG1");
#else
		gp_wrt( GpAdrsBuf, "AG1" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( SetupFlag == 2 && OnOffFlag == 2 ) // アベレージングOFF
	{
		SetupFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "AG0");
#else
		gp_wrt( GpAdrsBuf, "AG0" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( SetupFlag == 2 && OnOffFlag == 3 && AvgFlag == 0 ) // アベレージングTYPE
	{
		AvgFlag = 1;
		MySetItemText( "AVG", Avg[AvgFlag-1], "8" );
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), FALSE);
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcc), TRUE);
	}
	else if( SetupFlag == 2 && OnOffFlag == 3 && AvgFlag != 0 && AvgModeFlag == 0 ) // アベレージングTYPE
	{
		AvgModeFlag = 1;
		MySetItemText( "AVG", Avg[AvgFlag-1], "8" );
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), TRUE);
	}
	else if( SetupFlag == 2 && OnOffFlag == 3 && AvgFlag != 0 && AvgModeFlag != 0 ) // アベレージングTYPE
	{
		SetupFlag = 0;
		OnOffFlag = 0;
		memset( szBuf, 0x00, sizeof(szBuf) );
		if( AvgFlag == 1 ) // 移動平均演算
			strcpy( szBuf, "AT1");
		else if( AvgFlag == 2 ) // 指数化平均演算
			strcpy( szBuf, "AT0");
		if( AvgModeFlag == 1 ) // アベレージング係数8
			strcat( szBuf, ";AC1" );
		else if( AvgModeFlag == 2 ) // アベレージング係数16
			strcat( szBuf, ";AC2" );
		else if( AvgModeFlag == 3 ) // アベレージング係数32
			strcat( szBuf, ";AC3" );
		else if( AvgModeFlag == 4 ) // アベレージング係数64
			strcat( szBuf, ";AC4" );

#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
		gp_wrt( GpAdrsBuf, szBuf );
#endif
		ButtonFlag = 0;
		AvgFlag = 0;
		AvgModeFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}

	else if( SetupFlag == 3 && OnOffFlag == 1 ) // スケーリングON
	{
		SetupFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "SC1");
#else
		gp_wrt( GpAdrsBuf, "SC1" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( SetupFlag == 3 && OnOffFlag == 2 ) // スケーリングOFF
	{
		SetupFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "SC0");
#else
		gp_wrt( GpAdrsBuf, "SC0" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( SetupFlag == 4 ) // 初期化
	{
		SetupFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "RC");
#else
		gp_wrt( GpAdrsBuf, "RC" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( SetupFlag == 5 && MathFlag == 0 )	
	{
		MathFlag = 1;
		MySetItemText( "SETUP", Setup[SetupFlag-1], Math[MathFlag-1] );
	}
	else if( SetupFlag == 5 ) // 四則演算決定
	{
		SetupFlag = 0;
#ifdef _USE_U2
		if( MathFlag == 1 )	U2Gpwrt(fd, GpAdrsBuf, "DC14;MT7" ); // A + B
		if( MathFlag == 2 ) U2Gpwrt(fd, GpAdrsBuf, "DC14;MT8" ); // A - B
		if( MathFlag == 3 ) U2Gpwrt(fd, GpAdrsBuf, "DC14;MT9" ); // A * B
		if( MathFlag == 4 ) U2Gpwrt(fd, GpAdrsBuf, "DC14;MT10" ); // A / B
		if( MathFlag == 5 ) U2Gpwrt(fd, GpAdrsBuf, "DC14;MT11" ); // A / B^2
		if( MathFlag == 6 ) U2Gpwrt(fd, GpAdrsBuf, "DC14;MT12" ); // A^2 / B
#else
		if( MathFlag == 1 )	gp_wrt( GpAdrsBuf, "DC14;MT7" ); // A + B
		if( MathFlag == 2 ) gp_wrt( GpAdrsBuf, "DC14;MT8" ); // A - B
		if( MathFlag == 3 ) gp_wrt( GpAdrsBuf, "DC14;MT9" ); // A * B
		if( MathFlag == 4 ) gp_wrt( GpAdrsBuf, "DC14;MT10" ); // A / B
		if( MathFlag == 5 ) gp_wrt( GpAdrsBuf, "DC14;MT11" ); // A / B^2
		if( MathFlag == 6 ) gp_wrt( GpAdrsBuf, "DC14;MT12" ); // A^2 / B
#endif
		MathFlag = 0;
		ButtonFlag = 0;
		DisPlayCFlag = 14;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( MemoryFlag <= 2 && ButtonFlag == 8 && OnOffFlag == 0 ) // メモリメニューの決定
	{
		OnOffFlag = 1;
		MySetItemText( "MEM", Memory[MemoryFlag-1], OnOff[OnOffFlag-1] );
	}
	else if( MemoryFlag == 1 && OnOffFlag == 1 ) // ストアON
	{
		MemoryFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "SO1");
#else
		gp_wrt( GpAdrsBuf, "SO1" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( MemoryFlag == 1 && OnOffFlag == 2 ) // ストアOFF
	{
		MemoryFlag = 0;
		OnOffFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "SO0");
#else
		gp_wrt( GpAdrsBuf, "SO0" );
#endif
		ButtonFlag = 0;
		gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), " ");
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( MemoryFlag == 1 && OnOffFlag == 4 && IntervalFlag == 0 )
	{										
		gtk_widget_set_sensitive(entry_timeh, TRUE);
		gtk_widget_set_sensitive(entry_timem, TRUE);
		gtk_widget_set_sensitive(entry_times, TRUE);
		gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), "時 分 秒を入力後 Enterボタンクリック");
		IntervalFlag = 1;
	}
	else if( MemoryFlag == 1 && OnOffFlag == 4 && IntervalFlag != 0 ) // ストアインターバル設定
	{
		if( Intervalh < 0 || 100 <= Intervalh )
		{
			printf("時間 設定エラー \n");
			Intervalh = 0;
		}
		if( Intervalm < 0 || 60 <= Intervalm )
		{
			printf("分 設定エラー \n");
			Intervalm = 0;
		}
		if( Intervals < 0 || 60 <= Intervals )
		{
			printf("秒 設定エラー \n");
			Intervals = 0;
		}
		if( !( Intervalh == 0 && Intervalm == 0 && Intervals == 0 ) )
		{
			sprintf( szBuf, "%d時 %d分 %d秒", Intervalh, Intervalm, Intervals );
			gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), szBuf);
		}
		memset( szBuf, 0x00, sizeof(szBuf) );
		sprintf( szBuf, "SR%d,%d,%d", Intervalh, Intervalm, Intervals );
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
		gp_wrt( GpAdrsBuf, szBuf );
#endif
		ButtonFlag = 0;
		MemoryFlag = 0;
		OnOffFlag = 0;
		IntervalFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( MemoryFlag == 2 && OnOffFlag == 1 ) // リコールON
	{
		MemoryFlag = 0;
		OnOffFlag = 0;
		RecallFlag = 1;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "RO1");
#else
		gp_wrt( GpAdrsBuf, "RO1" );
#endif
		ButtonFlag = 0;
		MySetItemText( NULL, NULL, NULL );
		timer = gtk_timeout_add (2000, timeout_callback, data);
		gtk_editable_set_editable(GTK_EDITABLE(entry_funcc), TRUE);
	}
	else if( MemoryFlag == 2 && OnOffFlag == 2 ) // リコールOFF
	{
		MemoryFlag = 0;
		OnOffFlag = 0;
		RecallFlag = 0;
		DisPlayAFlag = 1; // ディスプレイAは電圧値
		DisPlayBFlag = 2; // ディスプレイBは電流値
		DisPlayCFlag = 3; // ディスプレイCは電力値
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "RO0");
#else
		gp_wrt( GpAdrsBuf, "RO0" );
#endif
		ButtonFlag = 0;
		gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), " ");
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( MemoryFlag == 2 && OnOffFlag == 4 && IntervalFlag == 0 )
	{										
		gtk_widget_set_sensitive(entry_timeh, TRUE);
		gtk_widget_set_sensitive(entry_timem, TRUE);
		gtk_widget_set_sensitive(entry_times, TRUE);
		gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), "時 分 秒を入力後 Enterボタンクリック");
		IntervalFlag = 1;
	}
	else if( MemoryFlag == 2 && OnOffFlag == 4 && IntervalFlag != 0 ) // リコールインターバル設定
	{
		if( Intervalh < 0 || 100 <= Intervalh )
		{
			printf("時間 設定エラー \n");
			Intervalh = 0;
		}
		if( Intervalm < 0 || 60 <= Intervalm )
		{
			printf("分 設定エラー \n");
			Intervalm = 0;
		}
		if( Intervals < 0 || 60 <= Intervals )
		{
			printf("秒 設定エラー \n");
			Intervals = 0;
		}
		if( !( Intervalh == 0 && Intervalm == 0 && Intervals == 0 ) )
		{
			sprintf( szBuf, "%d時 %d分 %d秒", Intervalh, Intervalm, Intervals );
			gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), szBuf);
		}
		memset( szBuf, 0x00, sizeof(szBuf) );
		sprintf( szBuf, "RR%d,%d,%d", Intervalh, Intervalm, Intervals );
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
		gp_wrt( GpAdrsBuf, szBuf );
#endif
		ButtonFlag = 0;
		MemoryFlag = 0;
		OnOffFlag = 0;
		IntervalFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( MemoryFlag >= 3 && ButtonFlag == 8 && FileFlag == 0 ) // メモリメニューの決定
	{
		FileFlag = 1;
		MySetItemText( NULL, File[FileFlag-1], "File" );
	}
	else if( MemoryFlag == 3 && FileFlag >= 1 ) // fileへ保存
	{
		MemoryFlag = 0;
#ifdef _USE_U2
		if( FileFlag == 1 )	U2Gpwrt(fd, GpAdrsBuf, "SS1" );
		else if( FileFlag == 2 )	U2Gpwrt(fd, GpAdrsBuf, "SS2" );
		else if( FileFlag == 3 )	U2Gpwrt(fd, GpAdrsBuf, "SS3" );
		else if( FileFlag == 4 )	U2Gpwrt(fd, GpAdrsBuf, "SS4" );
#else
		if( FileFlag == 1 )	gp_wrt( GpAdrsBuf, "SS1" );
		else if( FileFlag == 2 )	gp_wrt( GpAdrsBuf, "SS2" );
		else if( FileFlag == 3 )	gp_wrt( GpAdrsBuf, "SS3" );
		else if( FileFlag == 4 )	gp_wrt( GpAdrsBuf, "SS4" );
#endif
		FileFlag = 0;
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( MemoryFlag == 4 && FileFlag >= 1 ) // fileの読み出し
	{
		MemoryFlag = 0;
#ifdef _USE_U2
		if( FileFlag == 1 )	U2Gpwrt(fd, GpAdrsBuf, "SL1" );
		else if( FileFlag == 2 )	U2Gpwrt(fd, GpAdrsBuf, "SL2" );
		else if( FileFlag == 3 )	U2Gpwrt(fd, GpAdrsBuf, "SL3" );
		else if( FileFlag == 4 )	U2Gpwrt(fd, GpAdrsBuf, "SL4" );
#else
		if( FileFlag == 1 )	gp_wrt( GpAdrsBuf, "SL1" );
		else if( FileFlag == 2 )	gp_wrt( GpAdrsBuf, "SL2" );
		else if( FileFlag == 3 )	gp_wrt( GpAdrsBuf, "SL3" );
		else if( FileFlag == 4 )	gp_wrt( GpAdrsBuf, "SL4" );
#endif
		FileFlag = 0;
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( ButtonFlag == 9 && IntegFlag == 1 && IntegModeFlag == 0 ) // 積算モードの選択
	{
		IntegModeFlag = 1;
		MySetItemText( "INTEG", Integ[IntegFlag-1], IntegMode[IntegModeFlag-1] );
	}
	else if( ButtonFlag == 9 && IntegFlag == 2 && IntervalFlag == 0 ) // タイマ設定
	{
		gtk_widget_set_sensitive(entry_timeh, TRUE);
		gtk_widget_set_sensitive(entry_timem, TRUE);
		gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), "時 分 を入力後 Enterボタンクリック");
		IntervalFlag = 1;
	}
	else if( ButtonFlag == 9 && IntegFlag == 2 && IntervalFlag != 0 ) // 積算タイマ設定
	{
		if( Intervalh < 0 || 1000 <= Intervalh )
		{
			printf("時間 設定エラー \n");
		}
		if( Intervalm < 0 || 60 <= Intervalm )
		{
			printf("分 設定エラー \n");
		}
		/*
		if( !( Intervalh == 0 && Intervalm == 0 ) )
		{
			sprintf( szBuf, "%d時 %d分", Intervalh, Intervalm );
			gtk_statusbar_push (GTK_STATUSBAR (time_bar), GPOINTER_TO_INT (time_id), szBuf);
		}
		*/
		memset( szBuf, 0x00, sizeof(szBuf) );
		sprintf( szBuf, "TM%d,%d", Intervalh, Intervalm );
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
		gp_wrt( GpAdrsBuf, szBuf );
#endif
		ButtonFlag = 0;
		IntegFlag = 0;
		IntervalFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( ButtonFlag == 9 && IntegModeFlag == 1 ) // 標準積算モード
	{
		IntegFlag = 0;
		IntegModeFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "IC0");
#else
		gp_wrt( GpAdrsBuf, "IC0" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( ButtonFlag == 9 && IntegModeFlag == 2 ) // 連続積算モード
	{
		IntegFlag = 0;
		IntegModeFlag = 0;
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "IC1");
#else
		gp_wrt( GpAdrsBuf, "IC1" );
#endif
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}

	else if( ButtonFlag == 12 )
	{
		ButtonFlag = 0;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdStart
 *  機  能 : STARTボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdStart( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

#ifdef _USE_U2
	U2Gpwrt(fd, GpAdrsBuf, "IP");
#else
	gp_wrt( GpAdrsBuf, "IP" );
#endif
	timer = gtk_timeout_add (2000, timeout_callback, data);
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdStop
 *  機  能 : STOPボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdStop( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( ButtonFlag == 0 ) // 積算のストップ
	{
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "IP");
#else
		gp_wrt( GpAdrsBuf, "IP" );
#endif
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( ButtonFlag == 12 ) // 測定データをストアorリコールする
	{
		MemoryFlag = 1;
		ButtonFlag = 8;
		MySetItemText( "MEM", NULL, Memory[MemoryFlag-1] );
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdReset
 *  機  能 : RESETボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdReset( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( ButtonFlag == 0 ) // 積算のリセット
	{
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "IR");
#else
		gp_wrt( GpAdrsBuf, "IR" );
#endif
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( ButtonFlag == 12 ) // 積算モードor積算タイマの設定
	{
		IntegFlag = 1;
		ButtonFlag = 9;
		MySetItemText( "INTEG", NULL, Integ[IntegFlag-1] );
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdLocal
 *  機  能 : LOCALボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdLocal( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	//send LCL command
#ifdef _USE_U2
	GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
	GpStatus = gp_lcl( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_lcl() error [%d]\n", GpStatus);
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdSetup
 *  機  能 : SET UPボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdSetup( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	SetupFlag = 1;
	ButtonFlag = 11;
	MySetItemText( "SETUP", NULL, Setup[SetupFlag-1] );
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdShift
 *  機  能 : SHIFTボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdShift( GtkWidget    *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( ButtonFlag == 0 ) // なにも押されていなかった場合
	{
		ButtonFlag = 12;
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
	else if( ButtonFlag == 3 ) ButtonFlag = 12; // ホールドされている場合
	else // メニュー途中で抜けたいときのためのボタンの初期化
	{
		MySetItemText( NULL, NULL, NULL );
		ButtonFlag = 0;
		SetupFlag = 0;
		OnOffFlag = 0;
		MathFlag = 0;
		MemoryFlag = 0;
		FileFlag = 0;
		IntegFlag = 0;
		IntegModeFlag = 0;
		AvgFlag = 0;
		AvgModeFlag = 0;
		IntervalFlag = 0;

		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

/*
 *******************************************************************************
 *  関数名 : init_param
 *  機  能 : 各変数に初期値を入力する
 *  引  数 : なし
 *  戻り値 : なし
 *******************************************************************************
 */
static void init_param( void )
{
	InitFlag = 0;

	DisPlayAFlag = 1;
	DisPlayBFlag = 2;
	DisPlayCFlag = 3;
	VRangeFlag = 7;
	ARangeFlag = 7;
	ModeFlag = 1;
	RecallFlag = 0;

	ButtonFlag = 0;
	SetupFlag = 0;
	OnOffFlag = 0;
	MathFlag = 0;
	MemoryFlag = 0;
	FileFlag = 0;
	IntegFlag = 0;
	IntegModeFlag = 0;
	AvgFlag = 0;
	AvgModeFlag = 0;
	IntervalFlag = 0;
}

/*
 *******************************************************************************
 *  関数名 : create_window
 *  機  能 : GTK+ウィンドウを作成する
 *  引  数 : なし
 *  戻り値 : なし
 *******************************************************************************
 */
static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *table;
	GtkWidget *table1;
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *temp_bar;
	GtkWidget *button;
	gint temp_id;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 500, 400);
    gtk_window_set_title (GTK_WINDOW (window), "WT110E");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    /* 縦横 12x12 のテーブルを生成します。*/
    table = gtk_table_new (12, 12, TRUE);
    gtk_container_add (GTK_CONTAINER (window), table);
    gtk_widget_show (table);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("GPIB 機器アドレス設定");
    gtk_table_attach_defaults (GTK_TABLE (table), frame, 0, 5, 0, 3);
    gtk_widget_show (frame);

    /* 縦横 4x4 のテーブルを生成します。*/
    table1 = gtk_table_new (4, 4, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
    gtk_table_set_row_spacing( GTK_TABLE (table1), 0, 5 );
    gtk_container_add (GTK_CONTAINER (frame), table1);
    gtk_widget_show (table1);

    /* make GPIB adrs */
    label = gtk_label_new ("WT110E アドレス");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 3, 0, 2);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 2);
    g_signal_connect (G_OBJECT (entry), "changed",
                G_CALLBACK (enter_adrs),
                (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "1");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry, 3, 4, 0, 2);
    gtk_widget_show (entry);

    /* make PCI20 adrs */
    label = gtk_label_new ("REX-PCI20 アドレス");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 3, 2, 4);
    gtk_widget_show (label);

    temp_bar = gtk_statusbar_new ();
    temp_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (temp_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (temp_bar), temp_id, buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), temp_bar, 3, 4, 2, 4);
    gtk_widget_show (temp_bar);

    /* 縦横 6x6 のテーブルを生成します。*/
    table1 = gtk_table_new (6, 6, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
    gtk_table_set_row_spacings( GTK_TABLE (table1), 2 );
    gtk_table_attach_defaults (GTK_TABLE (table), table1, 0, 5, 5, 11);
    gtk_widget_show (table1);

	// FUNCTION A入力フィールド
    entry_funca = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (entry_funca), "");
    gtk_table_attach_defaults (GTK_TABLE (table1), entry_funca, 0, 5, 0, 1);
    gtk_widget_show (entry_funca);

	// FUNCTION Aボタン
    button_funca = gtk_button_new_with_label ("FUNCTION A");
    g_signal_connect (G_OBJECT (button_funca), "clicked",
                      G_CALLBACK (Cmd_OnCmdFuncA), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_funca, 2, 5, 1, 2);
    gtk_widget_show (button_funca);              

	// FUNCTION B入力フィールド
    entry_funcb = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (entry_funcb), "");
    gtk_table_attach_defaults (GTK_TABLE (table1), entry_funcb, 0, 5, 2, 3);
    gtk_widget_show (entry_funcb);

	// FUNCTION Bボタン
    button_funcb = gtk_button_new_with_label ("FUNCTION B");
    g_signal_connect (G_OBJECT (button_funcb), "clicked",
                      G_CALLBACK (Cmd_OnCmdFuncB), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_funcb, 2, 5, 3, 4);
    gtk_widget_show (button_funcb);              

	// FUNCTION C入力フィールド
    entry_funcc = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (entry_funcc), "");
    gtk_table_attach_defaults (GTK_TABLE (table1), entry_funcc, 0, 5, 4, 5);
    gtk_widget_show (entry_funcc);

	// FUNCTION Cボタン
    button_funcc = gtk_button_new_with_label ("FUNCTION C");
    g_signal_connect (G_OBJECT (button_funcc), "clicked",
                      G_CALLBACK (Cmd_OnCmdFuncC), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_funcc, 2, 5, 5, 6);
    gtk_widget_show (button_funcc);              

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("Interval設定");
    gtk_table_attach_defaults (GTK_TABLE (table), frame, 8, 12, 0, 4);
    gtk_widget_show (frame);

    /* 縦横 3x3 のテーブルを生成します。*/
    table1 = gtk_table_new (3, 3, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
    gtk_table_set_col_spacings( GTK_TABLE (table1), 2 );
    gtk_container_add (GTK_CONTAINER (frame), table1);
    gtk_widget_show (table1);

	// Interval(時)入力フィールド
    entry_timeh = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_timeh), 3);
    g_signal_connect (G_OBJECT (entry_timeh), "changed",
		      G_CALLBACK (enter_Intervalh),
		      (gpointer) entry_timeh);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "0");
    gtk_entry_set_text (GTK_ENTRY (entry_timeh), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry_timeh, 0, 1, 0, 1);
    gtk_widget_show (entry_timeh);

    label = gtk_label_new ("時");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 1, 1, 2);
    gtk_widget_show (label);

	// Interval(分)入力フィールド
    entry_timem = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_timem), 2);
    g_signal_connect (G_OBJECT (entry_timem), "changed",
		      G_CALLBACK (enter_Intervalm),
		      (gpointer) entry_timem);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "0");
    gtk_entry_set_text (GTK_ENTRY (entry_timem), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry_timem, 1, 2, 0, 1);
    gtk_widget_show (entry_timem);

    label = gtk_label_new ("分");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 1, 2, 1, 2);
    gtk_widget_show (label);

	// Interval(秒)入力フィールド
    entry_times = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_times), 2);
    g_signal_connect (G_OBJECT (entry_times), "changed",
		      G_CALLBACK (enter_Intervals),
		      (gpointer) entry_times);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "0");
    gtk_entry_set_text (GTK_ENTRY (entry_times), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry_times, 2, 3, 0, 1);
    gtk_widget_show (entry_times);

    label = gtk_label_new ("秒");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 2, 3, 1, 2);
    gtk_widget_show (label);

    time_bar = gtk_statusbar_new ();
    time_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (time_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "250ms");
    gtk_statusbar_push (GTK_STATUSBAR (time_bar), time_id, buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), time_bar, 0, 3, 2, 3);
    gtk_widget_show (time_bar);

    /* make botton */
    /* 縦横 6x6 のテーブルを生成します。*/
    table1 = gtk_table_new (6, 6, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 10);
    gtk_table_set_col_spacings( GTK_TABLE (table1), 5 );
    gtk_table_set_row_spacings( GTK_TABLE (table1), 5 );
    gtk_table_set_row_spacing( GTK_TABLE (table1), 1, 10 );
    gtk_table_set_row_spacing( GTK_TABLE (table1), 4, 10 );
    gtk_table_attach_defaults (GTK_TABLE (table), table1, 5, 12, 4, 11);
    gtk_widget_show (table1);

    // V RANGEボタン
    button_vrange = gtk_button_new_with_label ("V RANGE");
    g_signal_connect (G_OBJECT (button_vrange), "clicked",
                      G_CALLBACK (Cmd_OnCmdVRange), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_vrange, 0, 2, 0, 1);
    gtk_widget_show (button_vrange);              

    // A RANGEボタン
    button_arange = gtk_button_new_with_label ("A RANGE");
    g_signal_connect (G_OBJECT (button_arange), "clicked",
                      G_CALLBACK (Cmd_OnCmdARange), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_arange, 2, 4, 0, 1);
    gtk_widget_show (button_arange);              

    // HOLDボタン
    button_hold = gtk_button_new_with_label ("HOLD");
    g_signal_connect (G_OBJECT (button_hold), "clicked",
                      G_CALLBACK (Cmd_OnCmdHold), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_hold, 4, 6, 0, 1);
    gtk_widget_show (button_hold);              

    // テキスト表示"MODE"
    label = gtk_label_new ("MODE");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 2, 1, 2);
    gtk_widget_show (label);

    // テキスト表示"TRIG"
    label = gtk_label_new ("TRIG");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 4, 6, 1, 2);
    gtk_widget_show (label);

    // ↓ボタン
    button_down = gtk_button_new_with_label ("↓");
    g_signal_connect (G_OBJECT (button_down), "clicked",
                      G_CALLBACK (Cmd_OnCmdDown), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_down, 0, 2, 2, 3);
    gtk_widget_show (button_down);              

    // ↑ボタン
    button_up = gtk_button_new_with_label ("↑");
    g_signal_connect (G_OBJECT (button_up), "clicked",
                      G_CALLBACK (Cmd_OnCmdUp), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_up, 2, 4, 2, 3);
    gtk_widget_show (button_up);              

    // ENTERボタン
    button_enter = gtk_button_new_with_label ("ENTER");
    g_signal_connect (G_OBJECT (button_enter), "clicked",
                      G_CALLBACK (Cmd_OnCmdEnter), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_enter, 4, 6, 2, 3);
    gtk_widget_show (button_enter);              

    // STARTボタン
    button_start = gtk_button_new_with_label ("START");
    g_signal_connect (G_OBJECT (button_start), "clicked",
                      G_CALLBACK (Cmd_OnCmdStart), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_start, 0, 2, 3, 4);
    gtk_widget_show (button_start);              

    // STOPボタン
    button_stop = gtk_button_new_with_label ("STOP");
    g_signal_connect (G_OBJECT (button_stop), "clicked",
                      G_CALLBACK (Cmd_OnCmdStop), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_stop, 2, 4, 3, 4);
    gtk_widget_show (button_stop);              

    // RESETボタン
    button_reset = gtk_button_new_with_label ("RESET");
    g_signal_connect (G_OBJECT (button_reset), "clicked",
                      G_CALLBACK (Cmd_OnCmdReset), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_reset, 4, 6, 3, 4);
    gtk_widget_show (button_reset);              

    // テキスト表示"MEMORY"
    label = gtk_label_new ("MEMORY");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 2, 4, 4, 5);
    gtk_widget_show (label);

    // テキスト表示"INTEG SET"
    label = gtk_label_new ("INTEG SET");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 4, 6, 4, 5);
    gtk_widget_show (label);

    // LOCALボタン
    button_local = gtk_button_new_with_label ("LOCAL");
    g_signal_connect (G_OBJECT (button_local), "clicked",
                      G_CALLBACK (Cmd_OnCmdLocal), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_local, 0, 2, 5, 6);
    gtk_widget_show (button_local);              

    // SET UPボタン
    button_setup = gtk_button_new_with_label ("SET UP");
    g_signal_connect (G_OBJECT (button_setup), "clicked",
                      G_CALLBACK (Cmd_OnCmdSetup), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_setup, 2, 4, 5, 6);
    gtk_widget_show (button_setup);              

    // SHIFTボタン
    button_shift = gtk_button_new_with_label ("SHIFT");
    g_signal_connect (G_OBJECT (button_shift), "clicked",
                      G_CALLBACK (Cmd_OnCmdShift), NULL);
    gtk_table_attach_defaults (GTK_TABLE (table1), button_shift, 4, 6, 5, 6);
    gtk_widget_show (button_shift);              

	hbox = gtk_hbox_new (FALSE, 5);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
	gtk_table_attach_defaults (GTK_TABLE (table), hbox, 5, 12, 11, 12);
	gtk_widget_show (hbox);

    // 初期化ボタン
	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

    // グラフ表示ボタン
	button_graph = gtk_button_new_with_label ("グラフ表示");
	g_signal_connect (G_OBJECT (button_graph), "clicked",
                      G_CALLBACK (Cmd_OnCancel), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button_graph, TRUE, TRUE, 0);
	gtk_widget_show (button_graph);

    // 終了ボタン
	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	// ウィンドウの表示
	gtk_widget_show (window);
}

/*
 *******************************************************************************
 *  関数名 : main
 *  機  能 : メインルーチン
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_DEVICES];
	unsigned short MaxUnit = MAX_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたREX-PCI20をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつREX-PCI20をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	init_param();
	//gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	/* ダイアログコントロールを無効にする */
	gtk_widget_set_sensitive(button_vrange, FALSE);
	gtk_widget_set_sensitive(button_arange, FALSE);
	gtk_widget_set_sensitive(button_hold, FALSE);
	gtk_widget_set_sensitive(button_up, FALSE);
	gtk_widget_set_sensitive(button_down, FALSE);
	gtk_widget_set_sensitive(button_enter, FALSE);
	gtk_widget_set_sensitive(button_start, FALSE);
	gtk_widget_set_sensitive(button_stop, FALSE);
	gtk_widget_set_sensitive(button_reset, FALSE);
	gtk_widget_set_sensitive(button_local, FALSE);
	gtk_widget_set_sensitive(button_setup, FALSE);
	gtk_widget_set_sensitive(button_shift, FALSE);
	gtk_widget_set_sensitive(button_funca, FALSE);
	gtk_widget_set_sensitive(button_funcb, FALSE);
	gtk_widget_set_sensitive(button_funcc, FALSE);
	gtk_widget_set_sensitive(button_graph, FALSE);

	/* disable time entry */
	gtk_widget_set_sensitive(entry_timeh, FALSE);
	gtk_widget_set_sensitive(entry_timem, FALSE);
	gtk_widget_set_sensitive(entry_times, FALSE);
	/* read only */
	gtk_editable_set_editable(GTK_EDITABLE(entry_funca), TRUE);
	gtk_editable_set_editable(GTK_EDITABLE(entry_funcb), TRUE);
	gtk_editable_set_editable(GTK_EDITABLE(entry_funcc), TRUE);


	gtk_main ();
	gp_detach(fd);

	return 0;
}

