/*
 *******************************************************************************
 *
 *  横河電機　DC VOLTAGE CURRENT STANDARD　TYPE2553 制御プログラム
 *
 *  Compiler : gcc
 *
 *  (C) 2008 RATOC Systems, Inc.    Date : 2008.07.17
 *
 *  History : 
 *    2008.07.17    新規作成 (Windows版のソースを基にLinuxへ移植)
 *
 *  動作概要 : 
 *
 *    GPIB初期化
 *         ･･･ GPIBを初期化し、TYPE2553をリモート状態にします
 *
 *    定電圧電流出力ON/OFF
 *         ･･･ 電圧電流の各レンジ、極性、出力値のそれぞれ設定された値を
 *             定電圧および定電流出力します
 *
 *    スイープ電圧電流出力ON/OFF
 *         ･･･ 電圧電流の各レンジ、極性、出力値、スイープ方向の
 *             それぞれ設定された値を電圧電流スイープ出力します
 *
 *    終了
 *         ･･･プログラムを終了し、ダイアログを閉じます
 *
 *******************************************************************************
 */

//gcc -g yew2553.c -o yew2553 -L/usr/local/lib/ -lrexpci20_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "rexpci20_gpib.h"

//#define _DEBUG
//#define _USE_U2
#define MAX_DEVICES				16

int fd=0;
short MyGpibAdrs = 1;	// PCI20のGPIBアドレス
char GpAdrsBuf[12];		// 使用するGPIB機器のGPIBアドレス
char szBuf[256];		// ワークバッファ
char InBuf[256];		// 出力値入力用バッファ
int InitFlag;			// 初期化フラグ(初期値:0, 完了時:1)

double InputData;	// 出力値データ
double StartData;	// 開始値データ
double EndData;		// 終了値データ

GtkWidget *entry_input;
GtkWidget *entry_start;
GtkWidget *entry_end;

GtkWidget *button_10mV;
GtkWidget *button_100mV;
GtkWidget *button_1V;
GtkWidget *button_10V;
GtkWidget *button_1mA;
GtkWidget *button_10mA;
GtkWidget *button_100mA;
GtkWidget *button_plus;
GtkWidget *button_minus;

GtkWidget *button_sweep10mV;
GtkWidget *button_sweep100mV;
GtkWidget *button_sweep1V;
GtkWidget *button_sweep10V;
GtkWidget *button_sweep1mA;
GtkWidget *button_sweep10mA;
GtkWidget *button_sweep100mA;
GtkWidget *button_sweepplus;
GtkWidget *button_sweepminus;
GtkWidget *button_sweepup;
GtkWidget *button_sweepdown;

GtkWidget *button_on;
GtkWidget *button_off;
GtkWidget *button_sweepon;
GtkWidget *button_sweepoff;
GtkWidget *button_init;
GtkWidget *button_exit;
GtkWidget *status_bar;
gint context_id;

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmdGpInit
 *  機  能 : 初期化ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	// ステータスのクリア
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "");

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// GPIBバスタイムアウト時間を5秒に設定
#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 5);
#else
	GpStatus = gp_tmout(5);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	/* SDCコマンド送信後、必ず若干時間を経ってから次のコマンドを送信、
	   そうしないと、以下のコマンドは正常に送信できない可能性があります */
	gp_wait( 1 );

	// LLOコマンド送出
#ifdef _USE_U2
	GpStatus = U2Gpllo(fd);
#else
	GpStatus = gp_llo();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_llo() error [%d]¥n", GpStatus );
		return;
	}

	// YEW2553の表示値を"0"および出力"OFF"、"1Vﾚﾝｼﾞ"に初期設定する
	// 送信コマンド:V2P0S00000O0(次に説明）
	// 1Vﾚﾝｼﾞ:V2
	// 極性＋:P0
	// 出力値0V：S00000
	// 出力OFF:O0(ｵｰｾﾞﾛ）

	memset( szBuf, 0x00, sizeof( szBuf ) );
	strcpy( szBuf, "V2P0");
	strcat( szBuf, "S00000" );
	strcat( szBuf, "O0" );

	// YEW2553の出力電圧設定
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]¥n", GpStatus );
		return;
	}

	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error : %d", GpStatus );
		return;
	}

	// 「出力ON/OFF」ボタンを有効にする
	gtk_widget_set_sensitive(button_on, TRUE);
	gtk_widget_set_sensitive(button_off, TRUE);
	gtk_widget_set_sensitive(button_sweepon, TRUE);
	gtk_widget_set_sensitive(button_sweepoff, TRUE);

	// 「初期化」ボタンを無効にする
	gtk_widget_set_sensitive(button_init, FALSE);

	// 初期化正常終了を表示する
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "GPIB初期化正常終了");

	InitFlag = 1;
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmd_On
 *  機  能 : 出力ONボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmd_On( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;	// GPIBステータス用
	int		Revise;		// 各レンジでの補正値
	
	memset( szBuf, 0x00, sizeof( szBuf ) );

	// レンジ設定して選択されたレンジをオーバーしている場合はエラーメッセージの表示
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_10mV)) )
	{
		if( InputData <= 10 )
		{
			strcpy( szBuf, "V0");
			Revise = 1000;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_100mV)) )
	{
		if( InputData <= 100 )
		{
			strcpy( szBuf, "V1");
			Revise = 100;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}	
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_1V)) )
	{
		if( InputData <= 1 )
		{
			strcpy( szBuf, "V2");
			Revise = 10000;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}	
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_10V)) )
	{
		if( InputData <= 10 )
		{
			strcpy( szBuf, "V3");
			Revise = 1000;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}	
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_1mA)) )
	{		
		if( InputData <= 1 )
		{
			strcpy( szBuf, "A0");
			Revise = 10000;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_10mA)) )
	{
		if( InputData <= 10 )
		{
			strcpy( szBuf, "A1");
			Revise = 1000;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_100mA)) )
	{
		if( InputData <= 100 )
		{
			strcpy( szBuf, "A2");
			Revise = 100;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}
	}
	else
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジが選択されていません");
		return;
	}

	// 出力値をセットする
	sprintf( InBuf, "S%5d", (int)( InputData * Revise ) );

	// 極性設定値の状態を取得する
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_plus)) )
	{
		strcat( szBuf, "P0" );
	}
	else
	{
		strcat( szBuf, "P1" );
	}
	
	// 電圧・電流出力値をセットする
	strcat( szBuf, InBuf );

	//出力ON/OFF設定　O0(ｵｰｾﾞﾛ):出力 OFF
	strcat( szBuf, "O0" );

	// 出力設定の送信
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	
	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}

	// 出力ON/OFF設定　O1(ｵｰｲﾁ):出力 ON
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "O1");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "O1" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	
	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmd_Off
 *  機  能 : 出力OFFボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmd_Off( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;	// GPIBステータス用
	
	// 出力OFF設定 O0(ｵｰｾﾞﾛ):出力 OFF
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "O0");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "O0" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	
	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmd_SweepOn
 *  機  能 : 出力ON(スイープ)ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmd_SweepOn( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;	// GPIBステータス用
	int		Revise;	// 各レンジでの補正値
	int		RangeMax;	// 各レンジMax値
	int		RangeMin;	// 各レンジMin値
	char		Sweep[12];	// スイープUP、DOWN用

	//////////// スイープ開始値 /////////////
	
	memset( szBuf, 0x00, sizeof( szBuf ) );

	// レンジ設定して選択されたレンジをオーバーしている場合はエラーメッセージの表示
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweep10mV)) )
	{
		if( StartData <= 10 )
		{
			strcpy( szBuf, "V0");
			Revise = 1000;
			RangeMax = 10;
			RangeMin = 0;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweep100mV)) )
	{
		if( StartData <= 100 )
		{
			strcpy( szBuf, "V1");
			Revise = 100;
			RangeMax = 100;
			RangeMin = 0;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}	
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweep1V)) )
	{
		if( StartData <= 1 )
		{
			strcpy( szBuf, "V2");
			Revise = 10000;
			RangeMax = 1;
			RangeMin = 0;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}	
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweep10V)) )
	{
		if( StartData <= 10 )
		{
			strcpy( szBuf, "V3");
			Revise = 1000;
			RangeMax = 10;
			RangeMin = 0;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}	
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweep1mA)) )
	{
		if( StartData <= 1 )
		{
			strcpy( szBuf, "A0");
			Revise = 10000;
			RangeMax = 1;
			RangeMin = 0;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweep10mA)) )
	{
		if( StartData <= 10 )
		{
			strcpy( szBuf, "A1");
			Revise = 1000;
			RangeMax = 10;
			RangeMin = 0;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}
	}
	else if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweep100mA)) )
	{
		if( StartData <= 100 )
		{
			strcpy( szBuf, "A2");
			Revise = 100;
			RangeMax = 100;
			RangeMin = 0;
		}
		else
		{
			gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジオーバー");
			return;
		}
	}
	else
	{
		gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "レンジが選択されていません");
		return;
	}

	// 極性設定値の状態を取得する
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweepplus)) )
	{
		strcat( szBuf, "P0" );
	}
	else
	{
		strcat( szBuf, "P1" );
	}

	// 開始値をセットする
	sprintf( InBuf, "S%5d", (int)( StartData * Revise ) );
	strcat( szBuf, InBuf );

	// 出力ON/OFF設定　O0(ｵｰｾﾞﾛ):出力 OFF
	strcat( szBuf, "O0" );

	// 開始設定の送信
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	
	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}

	// 出力ON/OFF設定　O1(ｵｰｾﾞﾛ):出力 ON
	strcpy( szBuf, "O1" );

	// 出力設定の送信
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	
	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}
	
	//////////// スイープ終了値 /////////////

	// スイープ方向設定値より、スイープUPはレンジMax、DOWNは0を終了値にセット
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_sweepup)) )
	{
#ifdef _DEBUG
		printf("sweep-up \n");
#endif
		sprintf( szBuf, "%d",  RangeMax );
		gtk_entry_set_text (GTK_ENTRY (entry_end), szBuf);
		strcpy( Sweep, "C1" );
		strcpy( szBuf, "S10000" );
	}
	else
	{
#ifdef _DEBUG
		printf("sweep-down \n");
#endif
		sprintf( szBuf, "%d",  RangeMin );
		gtk_entry_set_text (GTK_ENTRY (entry_end), szBuf);
		strcpy( Sweep, "C2" );
		strcpy( szBuf, "" );
	}	

	// スイープ時間をセットする R1:16sec R2:32sec
	strcat( szBuf, "R1" );

	// スイープ方向をセットする
	strcat( szBuf, Sweep );

	// スイープ開始メッセージを表示する
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "スイープを開始します");	

	// スイープ設定の送信
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	
	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCmd_SweepOff
 *  機  能 : 出力OFF(スイープ)ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCmd_SweepOff( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;	// GPIBステータス用
	
	// スイープ解除設定
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "R0");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "R0" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}


	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}

	// 出力OFF設定 O0(ｵｰｾﾞﾛ):出力 OFF
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "O0");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "O0" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
	
	// トリガコマンド送信
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}

	gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, "");	
}

/*
 *******************************************************************************
 *  関数名 : Cmd_OnCancel
 *  機  能 : 終了ボタンが押された時の処理
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag){
		// 0V、OFF状態に戻す
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "S00000");
		U2Gpwrt(fd, GpAdrsBuf, "00");
		U2Gptrg(fd, GpAdrsBuf);
#else
		gp_wrt( GpAdrsBuf, "S00000" );
		gp_wrt( GpAdrsBuf, "O0" );
		gp_trg( GpAdrsBuf );
#endif
	}
}

/*
 *******************************************************************************
 *  関数名 : enter_adrs
 *  機  能 : アドレス入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}

/*
 *******************************************************************************
 *  関数名 : enter_input
 *  機  能 : 出力値入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_input( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;
  char	buf[12];

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(buf, 0x00, sizeof(buf));
  sprintf(buf, "%s", entry_text);
  InputData = atof( buf );
#ifdef _DEBUG
  printf ("InputData contents: %06.4lf\n", InputData);
#endif  
}

/*
 *******************************************************************************
 *  関数名 : enter_start
 *  機  能 : 開始値入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_start( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;
  char	buf[12];

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(buf, 0x00, sizeof(buf));
  sprintf(buf, "%s", entry_text);
  StartData = atof( buf );
#ifdef _DEBUG
  printf ("StartData contents: %06.4lf\n", StartData);
#endif
}

/*
 *******************************************************************************
 *  関数名 : enter_end
 *  機  能 : 終了値入力フィールドが変更された時の処理
 *  引  数 : widget - 未使用
 *           entry  - テキスト入力用フィールドのウィジット
 *  戻り値 : なし
 *******************************************************************************
 */
static void enter_end( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;
  char	buf[12];

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(buf, 0x00, sizeof(buf));
  sprintf(buf, "%s", entry_text);
  EndData = atof( buf );
#ifdef _DEBUG
  printf ("EndData contents: %06.4lf\n", EndData);
#endif
}

/*
 *******************************************************************************
 *  関数名 : create_window
 *  機  能 : GTK+ウィンドウを作成する
 *  引  数 : なし
 *  戻り値 : なし
 *******************************************************************************
 */
static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *main_hbox;
	GtkWidget *main_vbox;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *frame;
	GtkWidget *frame_sweep;
	GtkWidget *table;
	GtkWidget *table_sweep;
	GtkWidget *status_addr;
	GSList *group;
	GSList *group2;
	GSList *group3;
	GSList *group4;
	GSList *group5;
	char	buf[12];


    //////////// GPIB アドレス表示 ////////////

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 660, 640);
    gtk_window_set_title (GTK_WINDOW (window), "YEW2553");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    /* 次の行、列 */
    main_vbox = gtk_vbox_new (FALSE, 5);
    gtk_container_add (GTK_CONTAINER (window), main_vbox);
    gtk_widget_show (main_vbox);

    main_hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 2);
    gtk_box_pack_start (GTK_BOX (main_vbox), main_hbox, FALSE, FALSE, 0);
    gtk_widget_show (main_hbox);

    /* GPIB機器アドレス */
    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (main_hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 1);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "4");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (main_hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    /* 次の行 */
    main_hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 2);
    gtk_box_pack_start (GTK_BOX (main_vbox), main_hbox, FALSE, FALSE, 0);
    gtk_widget_show (main_hbox);

    /* REX-PCI20 アドレス */
    label = gtk_label_new ("REX-PCI20 アドレス");
    gtk_box_pack_start (GTK_BOX (main_hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    status_addr = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_addr), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (status_addr), context_id, buf);
    gtk_box_pack_start (GTK_BOX (main_hbox), status_addr, TRUE, TRUE, 0);
    gtk_widget_show (status_addr);

    /* 次の行 */
    main_hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 2);
    gtk_box_pack_start (GTK_BOX (main_vbox), main_hbox, FALSE, FALSE, 0);
    gtk_widget_show (main_hbox);


    //////////// YEW2553 定電圧・電流出力設定 ////////////

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("YEW2553 定電圧・電流出力設定");
    gtk_box_pack_start (GTK_BOX (main_hbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    /* 新しいテーブルを作成します。*/
    table = gtk_table_new (14, 2, TRUE);

    /* window に今作成した table を配置します。 */
    gtk_container_add (GTK_CONTAINER (frame), table);

    /* ラベルの表示 */
    label = gtk_label_new ("電圧レンジ設定");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 0, 1);
    gtk_widget_show (label);

    label = gtk_label_new ("電流レンジ設定");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 3, 4);
    gtk_widget_show (label);

    label = gtk_label_new ("極性設定");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 6, 7);
    gtk_widget_show (label);

    label = gtk_label_new ("出力値");
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
    gtk_table_attach_defaults (GTK_TABLE (table), label, 0, 1, 8, 9);
    gtk_widget_show (label);

    /* ラジオボタンの作成 */
    button_10mV = gtk_radio_button_new_with_label (NULL, "10mVレンジ");
    gtk_table_attach_defaults (GTK_TABLE (table), button_10mV, 0, 1, 1, 2);
    gtk_widget_show (button_10mV);

    /* ラジオボタンを group 化します。 */
    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button_10mV));
    /* 2番目以降のラジオボタンの場合、group を与える必要があります。 */
    button_100mV = gtk_radio_button_new_with_label (group, "100mVレンジ");
    gtk_table_attach_defaults (GTK_TABLE (table), button_100mV, 1, 2, 1, 2);
    gtk_widget_show (button_100mV);

    /* ラジオボタンを group 化します。 */
    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button_100mV));
    button_1V = gtk_radio_button_new_with_label (group, "1Vレンジ");
    gtk_table_attach_defaults (GTK_TABLE (table), button_1V, 0, 1, 2, 3);
    gtk_widget_show (button_1V);

    /* ラジオボタンを group 化します。 */
    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button_1V));
    button_10V = gtk_radio_button_new_with_label (group, "10Vレンジ");
    gtk_table_attach_defaults (GTK_TABLE (table), button_10V, 1, 2, 2, 3);
    gtk_widget_show (button_10V);

    /* ラジオボタンを group 化します。 */
    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button_10V));
    button_1mA = gtk_radio_button_new_with_label (group, "1mAレンジ");
    gtk_table_attach_defaults (GTK_TABLE (table), button_1mA, 0, 1, 4, 5);
    gtk_widget_show (button_1mA);

    /* ラジオボタンを group 化します。 */
    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button_1mA));
    button_10mA = gtk_radio_button_new_with_label (group, "10mAレンジ");
    gtk_table_attach_defaults (GTK_TABLE (table), button_10mA, 1, 2, 4, 5);
    gtk_widget_show (button_10mA);

    /* ラジオボタンを group 化します。 */
    group = gtk_radio_button_group (GTK_RADIO_BUTTON (button_10mA));
    button_100mA = gtk_radio_button_new_with_label (group, "100mAレンジ");
    gtk_table_attach_defaults (GTK_TABLE (table), button_100mA, 0, 1, 5, 6);
    gtk_widget_show (button_100mA);


    /* ラジオボタンの作成 */
    button_plus = gtk_radio_button_new_with_label (NULL, "＋極性");
    gtk_table_attach_defaults (GTK_TABLE (table), button_plus, 0, 1, 7, 8);
    gtk_widget_show (button_plus);

    /* ラジオボタンを group 化します。 */
    group2 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_plus));
    button_minus = gtk_radio_button_new_with_label (group2, "−極性");
    gtk_table_attach_defaults (GTK_TABLE (table), button_minus, 1, 2, 7, 8);
    gtk_widget_show (button_minus);


    /* エディットボックスの作成 */
    entry_input = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_input), 10);
    g_signal_connect (G_OBJECT (entry_input), "changed",
		      G_CALLBACK (enter_input),
		      (gpointer) entry_input);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "0.1");
    gtk_entry_set_text (GTK_ENTRY (entry_input), buf);
    gtk_table_attach_defaults (GTK_TABLE (table), entry_input, 1, 2, 8, 9);
    gtk_widget_show (entry_input);

	/* ボタンの作成 */
	button_on = gtk_button_new_with_label ("出力ON");
	g_signal_connect (G_OBJECT (button_on), "clicked",
                      G_CALLBACK (Cmd_OnCmd_On), NULL);
	gtk_table_attach_defaults (GTK_TABLE (table), button_on, 0, 1, 9, 10);
	gtk_widget_show (button_on); 

	button_off = gtk_button_new_with_label ("出力OFF");
	g_signal_connect (G_OBJECT (button_off), "clicked",
                      G_CALLBACK (Cmd_OnCmd_Off), NULL);
	gtk_table_attach_defaults (GTK_TABLE (table), button_off, 1, 2, 9, 10);
	gtk_widget_show (button_off); 


    //////////// YEW2553 スイープ出力設定 ////////////

    /* 新しいフレームを作成します。*/
    frame_sweep = gtk_frame_new ("YEW2553 スイープ出力設定");
    gtk_box_pack_start (GTK_BOX (main_hbox), frame_sweep, FALSE, FALSE, 0);
    gtk_widget_show (frame_sweep);

    /* 新しいテーブルを作成します。*/
    table_sweep = gtk_table_new (14, 2, TRUE);

    /* window に今作成した table を配置します。 */
    gtk_container_add (GTK_CONTAINER (frame_sweep), table_sweep);

	/* ラベルの表示 */
	label = gtk_label_new ("電圧レンジ設定");
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), label, 0, 1, 0, 1);
	gtk_widget_show (label);

	label = gtk_label_new ("電流レンジ設定");
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), label, 0, 1, 3, 4);
	gtk_widget_show (label);

	label = gtk_label_new ("極性設定");
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), label, 0, 1, 6, 7);
	gtk_widget_show (label);

	label = gtk_label_new ("スイープ方向設定");
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), label, 0, 1, 8, 9);
	gtk_widget_show (label);

	label = gtk_label_new ("開始値");
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), label, 0, 1, 10, 11);
	gtk_widget_show (label);

	label = gtk_label_new ("終了値");
	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);// ラベルを左寄せに設定する。
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), label, 0, 1, 11, 12);
	gtk_widget_show (label);

	label = gtk_label_new ("終了値は スイープUP：レンジMAX、\nスイープDOWN：レンジ0 となります");
	//gtk_misc_set_alignment (GTK_MISC (label), 0, 0);// ラベルを左寄せに設定する。
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), label, 0, 2, 12, 13);
	gtk_widget_show (label);


	/* ラジオボタンの作成 */
	button_sweep10mV = gtk_radio_button_new_with_label (NULL, "10mVレンジ");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweep10mV, 0, 1, 1, 2);
	gtk_widget_show (button_sweep10mV);

	/* ラジオボタンを group 化します。 */
	group3 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_sweep10mV));
	/* 2番目以降のラジオボタンの場合、group を与える必要があります。 */
	button_sweep100mV = gtk_radio_button_new_with_label (group3, "100mVレンジ");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweep100mV, 1, 2, 1, 2);
	gtk_widget_show (button_sweep100mV);

	/* ラジオボタンを group 化します。 */
	group3 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_sweep100mV));
	button_sweep1V = gtk_radio_button_new_with_label (group3, "1Vレンジ");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweep1V, 0, 1, 2, 3);
	gtk_widget_show (button_sweep1V);

	/* ラジオボタンを group 化します。 */
	group3 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_sweep1V));
	button_sweep10V = gtk_radio_button_new_with_label (group3, "10Vレンジ");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweep10V, 1, 2, 2, 3);
	gtk_widget_show (button_sweep10V);

	/* ラジオボタンを group 化します。 */
	group3 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_sweep10V));
	button_sweep1mA = gtk_radio_button_new_with_label (group3, "1mAレンジ");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweep1mA, 0, 1, 4, 5);
	gtk_widget_show (button_sweep1mA);

	/* ラジオボタンを group 化します。 */
	group3 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_sweep1mA));
	button_sweep10mA = gtk_radio_button_new_with_label (group3, "10mAレンジ");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweep10mA, 1, 2, 4, 5);
	gtk_widget_show (button_sweep10mA);

	/* ラジオボタンを group 化します。 */
	group3 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_sweep10mA));
	button_sweep100mA = gtk_radio_button_new_with_label (group3, "100mAレンジ");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweep100mA, 0, 1, 5, 6);
	gtk_widget_show (button_sweep100mA);


	/* ラジオボタンの作成 */
	button_sweepplus = gtk_radio_button_new_with_label (NULL, "＋極性");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweepplus, 0, 1, 7, 8);
	gtk_widget_show (button_sweepplus);

	/* ラジオボタンを group 化します。 */
	group4 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_sweepplus));
	button_sweepminus = gtk_radio_button_new_with_label (group4, "極性");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweepminus, 1, 2, 7, 8);
	gtk_widget_show (button_sweepminus);


	/* ラジオボタンの作成 */
	button_sweepup = gtk_radio_button_new_with_label (NULL, "スイープUP");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweepup, 0, 1, 9, 10);
	gtk_widget_show (button_sweepup);

	/* ラジオボタンを group 化します。 */
	group5 = gtk_radio_button_group (GTK_RADIO_BUTTON (button_sweepup));
	button_sweepdown = gtk_radio_button_new_with_label (group5, "スイープDOWN");
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweepdown, 1, 2, 9, 10);
	gtk_widget_show (button_sweepdown);


    /* エディットボックスの作成 */
    entry_start = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_start), 10);
    g_signal_connect (G_OBJECT (entry_start), "changed",
		      G_CALLBACK (enter_start),
		      (gpointer) entry_start);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "1");
    gtk_entry_set_text (GTK_ENTRY (entry_start), buf);
    gtk_table_attach_defaults (GTK_TABLE (table_sweep), entry_start, 1, 2, 10, 11);
    gtk_widget_show (entry_start);

    entry_end = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry_end), 10);
    g_signal_connect (G_OBJECT (entry_end), "changed",
		      G_CALLBACK (enter_end),
		      (gpointer) entry_end);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "10");
    gtk_entry_set_text (GTK_ENTRY (entry_end), buf);
    gtk_table_attach_defaults (GTK_TABLE (table_sweep), entry_end, 1, 2, 11, 12);
    gtk_widget_show (entry_end);


	/* ボタンの作成 */
	button_sweepon = gtk_button_new_with_label ("出力ON");
	g_signal_connect (G_OBJECT (button_sweepon), "clicked",
                      G_CALLBACK (Cmd_OnCmd_SweepOn), NULL);
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweepon, 0, 1, 13, 14);
	gtk_widget_show (button_sweepon); 

	button_sweepoff = gtk_button_new_with_label ("出力OFF");
	g_signal_connect (G_OBJECT (button_sweepoff), "clicked",
                      G_CALLBACK (Cmd_OnCmd_SweepOff), NULL);
	gtk_table_attach_defaults (GTK_TABLE (table_sweep), button_sweepoff, 1, 2, 13, 14);
	gtk_widget_show (button_sweepoff); 


    //////////// ステータス ////////////

    /* 次の行 */
    main_hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 2);
    gtk_box_pack_start (GTK_BOX (main_vbox), main_hbox, FALSE, FALSE, 0);
    gtk_widget_show (main_hbox);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    gtk_box_pack_start (GTK_BOX (main_hbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);


    //////////// 初期化・終了ボタン ////////////

     /* 次の行 */
    main_hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (main_hbox), 2);
    gtk_box_pack_end (GTK_BOX (main_vbox), main_hbox, FALSE, FALSE, 0);
    gtk_widget_show (main_hbox);

	button_init = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button_init), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (main_hbox), button_init, TRUE, TRUE, 0);
	gtk_widget_show (button_init);              

	button_exit = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button_exit), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button_exit), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (main_hbox), button_exit, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button_exit, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button_exit);
	gtk_widget_show (button_exit);


	/* ウィンドウの表示 */
	gtk_widget_show (table);
	gtk_widget_show (table_sweep);
	gtk_widget_show (window);
}

/*
 *******************************************************************************
 *  関数名 : main
 *  機  能 : メインルーチン
 *  引  数 : 未使用
 *  戻り値 : なし
 *******************************************************************************
 */
int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_DEVICES];
	unsigned short MaxUnit = MAX_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたREX-PCI20をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつREX-PCI20をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	//gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	// GPIB機器初期化などの準備ができるまで「出力ON/OFF」ボタンを無効にする
	gtk_widget_set_sensitive(button_on, FALSE);
	gtk_widget_set_sensitive(button_off, FALSE);
	gtk_widget_set_sensitive(button_sweepon, FALSE);
	gtk_widget_set_sensitive(button_sweepoff, FALSE);
	// 読み取り専用
	gtk_editable_set_editable(GTK_EDITABLE(entry_end), FALSE);

	gtk_main ();

	gp_detach(fd);

	return 0;
}

